/**************************************************************

phot_stars.c (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comfun.h"
#include "console.h"
#include "cmpack_common.h"
#include "cmpack_catfile.h"
#include "catselect.h"

/********************   Constants   ***********************/

#define ALLOC_BY 			64
#define SELECTION_COUNT		3

static const struct {
	int type;
	char *fullname, *prefix;
} Types[] = {
	{ CMPACK_SELECT_VAR,	"Variable stars",	"var" },
	{ CMPACK_SELECT_COMP,	"Comparison stars", "comp" },
	{ CMPACK_SELECT_CHECK,	"Check stars",		"chk" },
	{ -1 }
};

/*********************   Datatypes   *******************/

/* Star context */
struct _CmpackCatSelection
{
	CmpackContext gc;			/**< Global context */
	int *list;					/**< List of star identifiers */
	int count;					/**< Number of stars */
	int capacity;				/**< Number of allocated records */
};

/* Star context */
struct _CmpackCatSelectionTable
{
	CmpackCatSelection *list[SELECTION_COUNT];
};

/********************   Local functions    ***********************/

static void select_clean(void *ctx);

/* Create a new star */
static CmpackCatSelection *select_create(void)
{
	CmpackCatSelection *sel = (CmpackCatSelection*)gc_create(sizeof(CmpackCatSelection));
	sel->gc.clean = select_clean;
	return sel;
}

/* Clear selection */
static void select_clear(CmpackCatSelection *sel)
{
	cmpack_free(sel->list);
	sel->list = NULL;
	sel->count = sel->capacity = 0;
}

/* Free allocated memory */
static void select_clean(void *ctx)
{
	select_clear((CmpackCatSelection*)ctx);
	gc_clean(ctx);
}

/* Copy star attributes */
static void select_copy(CmpackCatSelection *to, const CmpackCatSelection *from)
{
	to->count = to->capacity = from->count;
	if (to->count>0) {
		to->list = (int*)cmpack_realloc(to->list, to->capacity*sizeof(int));
		memcpy(to->list, from->list, to->count*sizeof(int));
	} else {
		if (to->list) {
			cmpack_free(to->list);
			to->list = NULL;
		}
	}
}

/* Dump star attributes */
static void select_dump(const CmpackCatSelection *sel, CmpackConsole *con)
{
	int i;
	char aux[MAXLINE], aux1[32];

	sprintf(aux, "\t\t");
	for (i=0; i<sel->count; i++) {
		if (i>0) 
			sprintf(aux1, ", %d", sel->list[i]);
		else
			sprintf(aux1, "%d", sel->list[i]);
		strcat(aux, aux1);
	}
	printout(con, 1, aux);
}

/* Save star to file */
static void select_write(const CmpackCatSelection *sel, FILE *to, const char *prefix)
{	
	int i;
	char label[32];

	for (i=0; i<sel->count; i++) {
		if (i>0)
			sprintf(label, "%s%d", prefix, i+1);
		else
			sprintf(label, "%s", prefix);
		fprintf(to, "<select id=\"%d\" label=\"%s\"/>\n", sel->list[i], label);
	}
}

/* Init the table */
CmpackCatSelectionTable *cat_seltab_create()
{
	int i;

	CmpackCatSelectionTable *tab = (CmpackCatSelectionTable*)cmpack_malloc(sizeof(CmpackCatSelectionTable));
	for (i=0; i<SELECTION_COUNT; i++)
		tab->list[i] = select_create();
	return tab;
}

/* Clear the table */
void cat_seltab_clear(CmpackCatSelectionTable *tab)
{
	int i;

	for (i=0; i<SELECTION_COUNT; i++)
		select_clear(tab->list[i]);
}

/* Free allocated memory */
void cat_seltab_free(CmpackCatSelectionTable *tab)
{
	int i;

	for (i=0; i<SELECTION_COUNT; i++)
		cmpack_unref(tab->list[i]);
	cmpack_free(tab);
}

/* Dump the table */
void cat_seltab_dump(CmpackCatSelectionTable *tab, CmpackConsole *con)
{
	int i;
	char aux[MAXLINE];

	for (i=0; i<SELECTION_COUNT; i++) {
		sprintf(aux, "\t%s: %d items\n", Types[i].fullname, tab->list[i]->count);
		printout(con, 1, aux);
		select_dump(tab->list[i], con);
	}
}

/* Write table */
void cat_seltab_write(CmpackCatSelectionTable *tab, FILE *to)
{
	int i;
	
	fprintf(to, "<selection>\n");
	for (i=0; i<SELECTION_COUNT; i++) 
		select_write(tab->list[i], to, Types[i].prefix);
	fprintf(to, "</selection>\n");
}

/* Copy table of stars */
void cat_seltab_copy(CmpackCatSelectionTable *dsttab, const CmpackCatSelectionTable *srctab)
{
	int i;
	
	cat_seltab_clear(dsttab);
	for (i=0; i<SELECTION_COUNT; i++) 
		select_copy(dsttab->list[i], srctab->list[i]);
}

/* Creates a new star and returns its index */
CmpackCatSelection *cat_seltab_get(CmpackCatSelectionTable *tab, CmpackSelectionType type)
{
	switch (type)
	{
	case CMPACK_SELECT_VAR:
		return tab->list[0];
	case CMPACK_SELECT_COMP:
		return tab->list[1];
	case CMPACK_SELECT_CHECK:
		return tab->list[2];
	default:
		return NULL;
	}
}

/* Creates a new star and returns its index */
void cat_select_clear(CmpackCatSelection *sel)
{
	select_clear(sel);
}

/* Creates a new star and returns its index */
void cat_select_append(CmpackCatSelection *sel, const int *list, int count)
{
	if (sel->count+count > sel->capacity) {
		sel->capacity = ALLOC_BY*((sel->count+count+ALLOC_BY-1)/ALLOC_BY);
		sel->list = (int*)cmpack_realloc(sel->list, sel->capacity*sizeof(int));
	}
	memcpy(sel->list+sel->count, list, count*sizeof(int));
	sel->count += count;
}

/* Delete a star from the list */
void cat_select_delete(CmpackCatSelection *sel, int index)
{
	int j;

	if (index>=0 && index<sel->count) {
		for (j=index+1; j<sel->count; j++)
			sel->list[j-1] = sel->list[j];
		sel->count--;
	}
}

/* Creates a new star and returns its index */
int cat_select_get(CmpackCatSelection *sel, int *buf, int bufsize)
{
	if (bufsize>=sel->count) {
		memcpy(buf, sel->list, sel->count*sizeof(int));
		return sel->count;
	}
	return -1;
}

/* Set selection type */
void cat_seltab_set_star(CmpackCatSelectionTable *tab, int star_id, CmpackSelectionType type)
{
	int i, j;

	if (star_id>=0) {
		/* If the star is already of given type, skip */
		CmpackCatSelection *dsel = cat_seltab_get(tab, type);
		if (dsel) {
			for (j=0; j<dsel->count; j++) {
				if (dsel->list[j] == star_id)
					return;
			}
		}
		/* Find star in a selection and delete it */
		for (i=0; i<SELECTION_COUNT; i++) {
			CmpackCatSelection *sel = tab->list[i];
			if (sel!=dsel) {
				for (j=0; j<sel->count; j++) {
					if (sel->list[j] == star_id)
						cat_select_delete(sel, j);
				}
			}
		}
		/* Add selection to a new list (can be if new type is NONE) */
		if (dsel) 
			cat_select_append(dsel, &star_id, 1);
	}
}

/* Find star by its identifier, returns the selection type */
CmpackSelectionType cat_seltab_get_star(CmpackCatSelectionTable *tab, int star_id)
{
	int i, j;

	if (star_id>=0) {
		for (i=0; i<SELECTION_COUNT; i++) {
			const CmpackCatSelection *sel = tab->list[i];
			for (j=0; j<sel->count; j++) {
				if (sel->list[j] == star_id)
					return Types[i].type;
			}
		}
	}
	return CMPACK_SELECT_NONE;
}

/********************   Public functions    ***********************/

/* Clear the selection */
void cmpack_cat_selection_clear(CmpackCatSelection *sel)
{
	cat_select_clear(sel);
}

/* Append a list of stars */
void cmpack_cat_selection_append(CmpackCatSelection *sel, const int *list, int count)
{
	cat_select_append(sel, list, count);
}

/* Set selection */
void cmpack_cat_selection_set(CmpackCatSelection *sel, const int *list, int count)
{
	cat_select_clear(sel);
	cat_select_append(sel, list, count);
}

/* Get number of stars in a selection */
int cmpack_cat_selection_nitems(CmpackCatSelection *sel)
{
	return sel->count;
}

/* Get list of stars */
int cmpack_cat_selection_get(CmpackCatSelection *sel, int *list, int count)
{
	return cat_select_get(sel, list, count);
}

/* Get a star by index */
int cmpack_cat_selection_item(CmpackCatSelection *sel, int index)
{
	if (index>=0 && index<sel->count)
		return sel->list[index];
	else
		return -1;
}
