/**************************************************************

phot_stars.c (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "comfun.h"
#include "console.h"
#include "cmpack_common.h"
#include "cmpack_phtfile.h"
#include "cmpack_table.h"
#include "diffmag.h"
#include "lister.h"

/* Initialize variables */
int list_diff_init(CmpackLister *lc)
{
	int i, j, k;
	char buf[80], msg[MAXLINE];

	cmpack_tab_set_type(lc->tab, CMPACK_TABLE_LIGHTCURVE);

	if (lc->aperture<=0) {
		printout(lc->con, 0, "Invalid aperture index.");
		return CMPACK_ERR_INVALID_PAR;
	}
	if (lc->var.count!=1) {
		printout(lc->con, 0, "Exactly one variable star must be selected.");
		return CMPACK_ERR_INVALID_PAR;
	}
	if (lc->comp.count<1) {
		printout(lc->con, 0, "At least one comparison star must be selected.");
		return CMPACK_ERR_INVALID_PAR;
	}

	sprintf(msg, "Aperture: %d", lc->aperture);
	printout(lc->con, 1, msg);

    /* JD */
	cmpack_tab_add_column(lc->tab, "JD", CMPACK_COLUMN_DOUBLE, JD_PRECISION);

	/* V-C, ... */
	k = 1;
	sprintf(buf, "V-C");
	cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	sprintf(buf, "s%d", k++);
	cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);

	/* V1-K1, V1-K2, ... */
	for (j=0; j<lc->chk.count; j++) {
		sprintf(buf, "V-K%d", j+1);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
		sprintf(buf, "s%d", k++);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	}

	/* C1-K1, C1-K2, ... */
	for (i=0; i<lc->comp.count; i++) {
		for (j=0; j<lc->chk.count; j++) {
			sprintf(buf, "C%d-K%d", i+1, j+1);
			cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
			sprintf(buf, "s%d", k++);
			cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
		}
	}

	/* K1-K2, ... */
	for (i=0; i<lc->chk.count; i++) {
		for (j=i+1; j<lc->chk.count; j++) {
			sprintf(buf, "K%d-K%d", i+1, j+1);
			cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
			sprintf(buf, "s%d", k++);
			cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
		}
	}
	return 0;
}

/* Process one frame */
static int diff(CmpackLister *lc, double jd, int frame,
	const CmpackStar *var, const CmpackStar *comp, const CmpackStar *chk)
{
	char	msg[MAXLINE];
	int		i, n, j, k, row;
	double	cval, cerr, cmag;

	/* Debug printout */
	if (is_debug(lc->con)) {
		for (i=0; i<lc->var.count; i++) {
			sprintf(msg, "Variable star #%d:", i+1); 
			printout(lc->con, 1, msg);
			if (var[i].mask & CMPACK_MASK_MAG) {
				sprintf(msg, "\tMagnitude: %.3f +- %.3f mag", var[i].mag, var[i].err);
				printout(lc->con, 1, msg);
			}
		}
		for (i=0; i<lc->comp.count; i++) {
			sprintf(msg, "Comparison star #%d:", i+1); 
			printout(lc->con, 1, msg);
			if (comp[i].mask & CMPACK_MASK_MAG) {
				sprintf(msg, "\tMagnitude: %.3f +- %.3f mag", comp[i].mag, comp[i].err);
				printout(lc->con, 1, msg);
			}
		}
		for (i=0; i<lc->chk.count; i++) {
			sprintf(msg, "Check star #%d:", i+1); 
			printout(lc->con, 1, msg);
			if (chk[i].mask & CMPACK_MASK_MAG) {
				sprintf(msg, "\tMagnitude: %.3f +- %.3f mag", chk[i].mag, chk[i].err);
				printout(lc->con, 1, msg);
			}
		}
	}

	/* Check variable star */
	n = 0;
	for (i=0; i<lc->var.count; i++) {
		if ((var[i].mask & CMPACK_MASK_MAG) && (var[i].mask & CMPACK_MASK_ERR))
			n++;
	}
	if (n<1) {
		printout(lc->con, 0, "The variable star(s) have undefined value of mag. or std. err.");
		return 0;
	}

	/* Compute artifitial comparison star magnitude and error */
	cval = 0.0; cerr = 0.0; n = 0;
	for (i=0; i<lc->comp.count; i++) {
		if ((comp[i].mask & CMPACK_MASK_MAG) && (comp[i].mask & CMPACK_MASK_ERR)) {
			cval += pow(10.0, -0.4*comp[i].mag);
			cerr += comp[i].err;
			n++;
		}
	}

	/* Check comparison star */
	if (n!=lc->comp.count) {
		printout(lc->con, 0, "The comparison star(s) have undefined value of mag. or std. err.");
		return 0;
	}
	cmag = -2.5*log10(cval/n);
	cerr = (cerr/n)/sqrt(n);
	if (lc->comp.count>0 && is_debug(lc->con)) {
		sprintf(msg, "Comparison star: %.3f +- %.3f mag", cmag, cerr);
		printout(lc->con, 1, msg);
	}

	/* Export data to table */
	row = cmpack_tab_add_row(lc->tab);

	/* JD */
	cmpack_tab_ptdd(lc->tab, row, 0, jd);
	k = 1;

	/* V-C */
	if ((var[0].mask & CMPACK_MASK_MAG) && (var[0].mask & CMPACK_MASK_ERR)) {
		cmpack_tab_ptdd(lc->tab, row, k++, var[0].mag - cmag);
		cmpack_tab_ptdd(lc->tab, row, k++, sqrt(var[0].err*var[0].err + cerr*cerr));
	}

	/* V-C1, V-C2, ... */
	for (j=0; j<lc->chk.count; j++) {
		if ((var[0].mask & CMPACK_MASK_MAG) && (var[0].mask & CMPACK_MASK_ERR) && 
			(chk[j].mask & CMPACK_MASK_MAG) && (chk[j].mask & CMPACK_MASK_ERR)) {
			cmpack_tab_ptdd(lc->tab, row, k++, var[0].mag - chk[j].mag);
			cmpack_tab_ptdd(lc->tab, row, k++, sqrt(var[0].err*var[0].err + chk[j].err*chk[j].err));
		}
	}

	/* C1-K1, C1-K2, ... */
	for (i=0; i<lc->comp.count; i++) {
		for (j=0; j<lc->chk.count; j++) {
			if ((comp[i].mask & CMPACK_MASK_MAG) && (comp[i].mask & CMPACK_MASK_ERR) && 
				(chk[j].mask & CMPACK_MASK_MAG) && (chk[j].mask & CMPACK_MASK_ERR)) {
				cmpack_tab_ptdd(lc->tab, row, k++, comp[i].mag - chk[j].mag);
				cmpack_tab_ptdd(lc->tab, row, k++, sqrt(comp[i].err*comp[i].err + chk[j].err*chk[j].err));
			}
		}
	}

	/* K1-K2, ... */
	for (i=0; i<lc->chk.count; i++) {
		for (j=i+1; j<lc->chk.count; j++) {
			if ((chk[i].mask & CMPACK_MASK_MAG) && (chk[i].mask & CMPACK_MASK_ERR) && 
				(chk[j].mask & CMPACK_MASK_MAG) && (chk[j].mask & CMPACK_MASK_ERR)) {
				cmpack_tab_ptdd(lc->tab, row, k++, chk[i].mag - chk[j].mag);
				cmpack_tab_ptdd(lc->tab, row, k++, sqrt(chk[i].err*chk[i].err + chk[j].err*chk[j].err));
			}
		}
	}

	return 0;
}

/* Process photometry file */
int list_diff(CmpackLister *lc, CmpackPhtFile *pht, int frame)
{
	double	jd, radius;
	int		count, i, j, xref;
	const char *filter;
	CmpackPhtAperture *aper;
	CmpackPhtStar *st;
	CmpackPhtMeasurement data;
	CmpackStar *var, *comp, *chk, *in_st;

	/* Read julian date */
	jd = cmpack_pht_get_jd(pht);
	if (jd<=0)
		return CMPACK_ERR_READ_ERROR;
	/* Set active aperture */
	aper = cmpack_pht_find_aperture(pht, lc->aperture);
	if (!aper)
		return CMPACK_ERR_AP_NOT_FOUND;
	radius = cmpack_pht_aperture_get_radius(aper);
	cmpack_unref(aper);
	/* Get optical filter name */
	filter = cmpack_pht_get_filter(pht);
	/* Check parameters */
	if (lc->nframes==0) {
		lc->ap_radius = radius;
		cmpack_tab_pkyi(lc->tab, "Aperture", lc->aperture);
		if (filter) {
			lc->filter = cmpack_strdup(filter);
			if (strlen(lc->filter)>0)
				cmpack_tab_pkys(lc->tab, "Filter", lc->filter);
		}
	} else {
		if (radius!=lc->ap_radius)
			printout(lc->con, 0, "Different aperture radius!");
		if (filter) {
			if (strcmp(lc->filter, filter)!=0)
				printout(lc->con, 0, "Different optical filter!");
		} else {
			if (lc->filter)
				printout(lc->con, 0, "Different optical filter!");
		}
	}

	var = lc->var.list;
	comp = lc->comp.list;
	chk = lc->chk.list;

	/* Read magnitudes for selected stars */
	count = cmpack_pht_nstar(pht);
	for (i=0; i<count; i++) {
		st = cmpack_pht_get_star(pht, i);
		xref = cmpack_pht_star_get_refid(st);
		if (xref>=0) {
			in_st = NULL; 
			for (j=0; j<lc->var.count && !in_st; j++)
				if (lc->var.list[j].id==xref) in_st=&var[j]; 
			for (j=0; j<lc->comp.count && !in_st; j++)
				if (lc->comp.list[j].id==xref) in_st=&comp[j];
			for (j=0; j<lc->chk.count && !in_st; j++)
				if (lc->chk.list[j].id==xref) in_st=&chk[j];
			if (in_st) {
				/* Get magnitude for selected aperture */
				if (cmpack_pht_star_get_measurement(st, lc->aperture, &data)==0) {
					in_st->mask = data.mask & (CMPACK_MASK_MAG | CMPACK_MASK_ERR);
					in_st->mag = data.magnitude;
					in_st->err = data.error;
				} else {
					in_st->mask = 0; 
				}
			}
		}
		cmpack_unref(st);
	}

	return diff(lc, jd, frame, var, comp, chk);
}

/* Free allocated memory */
int list_diff_close(CmpackLister *lc)
{
	if (lc->var.list) {
		cmpack_free(lc->var.list);
		lc->var.list = NULL;
		lc->var.count = 0;
	}
	if (lc->comp.list) {
		cmpack_free(lc->comp.list);
		lc->comp.list = NULL;
		lc->comp.count = 0;
	}
	if (lc->chk.list) {
		cmpack_free(lc->chk.list);
		lc->chk.list = NULL;
		lc->chk.count = 0;
	}
	return 0;
}
