/**************************************************************

flat.c (C-Munipack project)
Flat correction
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comfun.h"
#include "console.h"
#include "robmean.h"
#include "ccdfile.h"
#include "cmpack_common.h"
#include "cmpack_flat.h"

/*******************   DATA TYPES   ***************************/

/* Flat correction context */
struct _CmpackFlatCorr
{
	CmpackContext gc;		/**< Global context */
	CmpackConsole *con;		/**< Console */
	CmpackBorder border;	/**< Border size */
	int nx;					/**< Width of flat-frame */
	int ny;					/**< Height of flat-frame */
	double *fdata;			/**< Data of flat-frame */
};

/*******************   LOCAL FUNCTIONS   *******************************/

/* Free allocated memory */
static void flat_clean(void *ctx)
{
	CmpackFlatCorr *lc = (CmpackFlatCorr*)ctx;

	cmpack_free(lc->fdata);
	cmpack_unref(lc->con);
	gc_clean(&lc->gc);
}

/* Computes the output image from source image and flat image (out=mean*sci/flat) */
static int flat_flat(CmpackFlatCorr *lc, CmpackCcdFile *infile, CmpackCcdFile *outfile)
{
	int res, i, nx, ny, x, y, underflow, overflow, divzero;
	double *sdata, minvalue, maxvalue;
	char msg[MAXLINE];
	CmpackBitpix bitpix;

	/* Check image size */
	nx = ny = 0;
	cmpack_ccd_get_size(infile, &nx, &ny);
	if (nx<=0 || nx>=65536 || ny<=0 || ny>=65536) 
		return CMPACK_ERR_INVALID_SIZE;
	if (lc->nx!=nx || lc->ny!=ny) 
		return CMPACK_ERR_DIFF_SIZE_FLAT;

	/* Get data format */
	bitpix = cmpack_ccd_get_bitpix(infile);
	cmpack_ccd_get_range(infile, &minvalue, &maxvalue);
	if (bitpix==CMPACK_BITPIX_UNKNOWN) {
		printout(lc->con, 0, "Unsupported image data format.");
		return CMPACK_ERR_READ_ERROR;
	}
	if (is_debug(lc->con)) {
		sprintf(msg, "Data format  : %s", pixformat(bitpix));
		printout(lc->con, 1, msg);
		sprintf(msg, "Min. value   : %.12g", minvalue); 
		printout(lc->con, 1, msg);
		sprintf(msg, "Max. value   : %.12g", maxvalue); 
		printout(lc->con, 1, msg);
		sprintf(msg, "Border     : %d %d %d %d", lc->border.left, lc->border.top,
			lc->border.right, lc->border.bottom);
		printout(lc->con, 1, msg);
	}

	/* Read exposure data */
	sdata = cmpack_malloc(nx*ny*sizeof(double));
	res = cmpack_ccd_get_image(infile, CMPACK_BITPIX_DOUBLE, 0, 0, sdata);
	if (res!=0) {
		cmpack_free(sdata);
		return res;
	}

	/* flat subtraction and range checking */
	underflow = overflow = divzero = 0;
	for (y=0;y<ny;y++) {
		for (x=0;x<nx;x++) {
			i = x + y*nx;
			if (y>=lc->border.top && y<ny-lc->border.bottom &&
				x>=lc->border.left && x<nx-lc->border.right) {
					if (lc->fdata[i]>0.0) {
						sdata[i] /= lc->fdata[i];
						if (sdata[i]<minvalue) {
							sdata[i] = minvalue;
							underflow = 1;
						}
						if (sdata[i]>maxvalue) {
							sdata[i] = maxvalue;
							overflow = 1;
						}
					} else {
						sdata[i] = maxvalue;
						divzero = 1;
					}
			} else {
				sdata[i] = 0.0;
			}
		}	
	}

	/* Store image data */
	res = ccd_write_image(outfile, CMPACK_BITPIX_DOUBLE, sdata);
	cmpack_free(sdata);

	/* Normal return */
	if (divzero) 
		printout(lc->con, 1, "Division by zero.");
	if (overflow)
		printout(lc->con, 1, "An overflow has been occurred during computation");
	if (underflow)
		printout(lc->con, 1, "An underflow has been occurred during computation");
	return res;
}

/***********************   PUBLIC FUNCTIONS   **************************/

/* Initializes the context */
CmpackFlatCorr *cmpack_flat_init(void)
{
	CmpackFlatCorr *lc = gc_create(sizeof(CmpackFlatCorr));
	lc->gc.clean = &flat_clean;
	return lc;
}

/* Attach console */
void cmpack_flat_set_console(CmpackFlatCorr *ctx, CmpackConsole *con)
{
	if (con!=ctx->con) {
		cmpack_unref(ctx->con);
		ctx->con = cmpack_addref(con);
	}
}

/* Set image border */
void cmpack_flat_set_border(CmpackFlatCorr *lc, const CmpackBorder *border)
{
	if (border)
		lc->border = *border;
	else
		memset(&lc->border, 0, sizeof(CmpackBorder));
}

/* Set image border */
void cmpack_flat_get_border(CmpackFlatCorr *lc, CmpackBorder *border)
{
	if (border)
		*border = lc->border;
}

/* Loads the flat-frame */
int cmpack_flat_rflat(CmpackFlatCorr *lc, CmpackCcdFile *flat)
{
	int res, x, y, nx, ny, pixels;
	double *temp, fmed, fsig;

	/* Clear previous flat frame */
	cmpack_free(lc->fdata);
	lc->fdata = NULL;
	lc->nx = lc->ny = 0;

    /* Check file name */
    if (!flat) {
        printout(lc->con, 0, "Invalid flat frame context.");
 	    return CMPACK_ERR_INVALID_PAR;
    }

    /* Read and check dimensions */
	nx = ny = 0;
	cmpack_ccd_get_size(flat, &nx, &ny);
	if (nx<=0 || nx>=65536 || ny<=0 || ny>=65536) {
		printout(lc->con, 1, "Invalid dimensions of the flat frame.");
		return CMPACK_ERR_INVALID_SIZE;
	}
	lc->nx = nx;
	lc->ny = ny;

	/* Read flat frame data */
	lc->fdata = cmpack_malloc(lc->nx*lc->ny*sizeof(double));
	res = cmpack_ccd_get_image(flat, CMPACK_BITPIX_DOUBLE, 0, 0, lc->fdata);
	if (res!=0) {
		cmpack_free(lc->fdata);
		lc->fdata = NULL;
		return res;
	}

    /* Median computation */
	temp = cmpack_malloc(lc->nx*lc->ny*sizeof(double));
	pixels = 0;
	for (y=lc->border.top; y<lc->ny-lc->border.bottom; y++) {
		for (x=lc->border.left; x<lc->nx-lc->border.right; x++) 
			temp[pixels++] = lc->fdata[x+y*lc->nx];
	}
	if (pixels>0) 
		cmpack_robustmean(pixels, temp, &fmed, &fsig);
	else
		fmed = 0.0;
	cmpack_free(temp);
	if (fmed==0) {
		cmpack_free(lc->fdata);
		lc->fdata = NULL;
    	return CMPACK_ERR_MEAN_ZERO;
	}
    /* Divide all flat values by median of the frame */
	for (y=lc->border.top; y<lc->ny-lc->border.bottom; y++) {
		for (x=lc->border.left; x<lc->nx-lc->border.right; x++) 
			lc->fdata[x+y*lc->nx] /= fmed;
	}

    /* Normal return */
	if (is_debug(lc->con)) {
		printout(lc->con, 1, "Flat correction frame:");
		printpari(lc->con, "Width", 1, lc->nx);
		printpari(lc->con, "Height", 1, lc->ny);
		printpard(lc->con, "Median", 1, fmed, 2);
		printpard(lc->con, "Std. dev.", 1, fsig, 2);
	}
    return 0;
}

/* Computes the output image from source image and flat image (out=mean*sci/flat) */
int cmpack_flat(CmpackFlatCorr *lc, CmpackCcdFile *file)
{
	/* Check flat file */
	if (!lc->fdata) {
		printout(lc->con, 0, "Missing flat frame.");
		return CMPACK_ERR_NO_FLAT_FRAME;
	}

  	/* Check parameters */
	if (!file) {
		printout(lc->con, 0, "Invalid frame context.");
		return CMPACK_ERR_INVALID_PAR;
	}

	/* Flat correction */
	return flat_flat(lc, file, file);
}

/* Computes the output image from source image and flat image (out=mean*sci/flat) */
int cmpack_flat_ex(CmpackFlatCorr *lc, CmpackCcdFile *infile, CmpackCcdFile *outfile)
{
	int res, nx, ny;
	CmpackBitpix bitpix;

	/* Check flat file */
	if (!lc->fdata) {
		printout(lc->con, 0, "Missing flat frame.");
		return CMPACK_ERR_NO_FLAT_FRAME;
	}

  	/* Check parameters */
	if (!infile) {
		printout(lc->con, 0, "Invalid input frame context.");
		return CMPACK_ERR_INVALID_PAR;
	}
	if (!outfile) {
		printout(lc->con, 0, "Invalid output frame context.");
		return CMPACK_ERR_INVALID_PAR;
	}

	/* Check image size */
	nx = ny = 0;
	cmpack_ccd_get_size(infile, &nx, &ny);
	if (nx<=0 || nx>=65536 || ny<=0 || ny>=65536) {
		printout(lc->con, 0, "Invalid frame size.");
		return CMPACK_ERR_INVALID_SIZE;
	}

	/* Get data format */
	bitpix = cmpack_ccd_get_bitpix(infile);
	if (bitpix==CMPACK_BITPIX_UNKNOWN) {
		printout(lc->con, 0, "Unsupported image data format.");
		return CMPACK_ERR_READ_ERROR;
	}

	/* Init output image */
	res = ccd_prepare(outfile, nx, ny, bitpix);
	if (res!=0) 
		return res;

	/* Copy file header */
	res = ccd_copy_header(outfile, infile);
	if (res!=0) 
		return res;

	/* Flat correction */
	return flat_flat(lc, infile, outfile);
}
