/**************************************************************

instmag.c (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "comfun.h"
#include "console.h"
#include "cmpack_common.h"
#include "cmpack_phtfile.h"
#include "cmpack_table.h"
#include "instmag.h"
#include "lister.h"

/* Initialize variables */
int list_inst_init(CmpackLister *lc)
{
	int i, k, nstar = lc->var.count + lc->comp.count + lc->chk.count;
	char buf[80], msg[MAXLINE];

	cmpack_tab_set_type(lc->tab, CMPACK_TABLE_LIGHTCURVE);

	if (lc->aperture<=0) {
		printout(lc->con, 0, "Invalid aperture index.");
		return CMPACK_ERR_INVALID_PAR;
	}
    if (nstar<1) {
		printout(lc->con, 0, "At least one star must be selected.");
		return CMPACK_ERR_INVALID_PAR;
	}

	sprintf(msg, "Aperture: %d", lc->aperture);
	printout(lc->con, 1, msg);

    /* JD */
	cmpack_tab_add_column(lc->tab, "JD", CMPACK_COLUMN_DOUBLE, JD_PRECISION);

	/* V1, V2, ... */
	k = 1;
	for (i=0; i<lc->var.count; i++) {
		sprintf(buf, "V%d", i+1);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
		sprintf(buf, "s%d", k++);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	}
	/* C1, C2, ... */
	for (i=0; i<lc->comp.count; i++) {
		sprintf(buf, "C%d", i+1);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
		sprintf(buf, "s%d", k++);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	}
	/* K1-K2, ... */
	for (i=0; i<lc->chk.count; i++) {
		sprintf(buf, "K%d", i+1);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
		sprintf(buf, "s%d", k++);
		cmpack_tab_add_column(lc->tab, buf, CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	}
	return 0;
}

/* Process single frame */
static int inst(CmpackLister *lc, double jd, int frame,
	const CmpackStar *var, const CmpackStar *comp, const CmpackStar *chk)
{
	int i, k, row;

	/* Export data to table */
	row = cmpack_tab_add_row(lc->tab);

	/* JD */
	cmpack_tab_ptdd(lc->tab, row, 0, jd);
	k = 1;

	/* Variables */
	for (i=0; i<lc->var.count; i++) {
		if ((var[i].mask & CMPACK_MASK_MAG) && (var[i].mask & CMPACK_MASK_ERR)) {
			cmpack_tab_ptdd(lc->tab, row, k++, var[i].mag);
			cmpack_tab_ptdd(lc->tab, row, k++, var[i].err);
		}
	}

	/* Comparison stars */
	for (i=0; i<lc->comp.count; i++) {
		if ((comp[i].mask & CMPACK_MASK_MAG) && (comp[i].mask & CMPACK_MASK_ERR)) {
			cmpack_tab_ptdd(lc->tab, row, k++, comp[i].mag);
			cmpack_tab_ptdd(lc->tab, row, k++, comp[i].err);
		}
	}

	/* Check stars */
	for (i=0; i<lc->chk.count; i++) { 
		if ((chk[i].mask & CMPACK_MASK_MAG) && (chk[i].mask & CMPACK_MASK_ERR)) {
			cmpack_tab_ptdd(lc->tab, row, k++, chk[i].mag);
			cmpack_tab_ptdd(lc->tab, row, k++, chk[i].err);
		}
	}

	return 0;
}

/* Process photometry file */
int list_inst(CmpackLister *lc, CmpackPhtFile *pht, int frame)
{
	int		count, i, j, xref;
	double	jd, radius;
	const char *filter;
	CmpackPhtAperture *aper;
	CmpackPhtStar *st;
	CmpackPhtMeasurement data;
	CmpackStar *var, *comp, *chk, *in_st;

	/* Read header information */
	jd = cmpack_pht_get_jd(pht);
	if (jd<=0)
		return CMPACK_ERR_READ_ERROR;
	/* Set active aperture */
	aper = cmpack_pht_find_aperture(pht, lc->aperture);
	if (!aper)
		return CMPACK_ERR_AP_NOT_FOUND;
	radius = cmpack_pht_aperture_get_radius(aper);
	cmpack_unref(aper);
	/* Get optical filter name */
	filter = cmpack_pht_get_filter(pht);
	/* Check parameters */
	if (lc->nframes==0) {
		lc->ap_radius = radius;
		cmpack_tab_pkyi(lc->tab, "Aperture", lc->aperture);
		if (filter) {
			lc->filter = cmpack_strdup(filter);
			if (strlen(lc->filter)>0)
				cmpack_tab_pkys(lc->tab, "Filter", lc->filter);
		}
	} else {
		if (radius!=lc->ap_radius)
			printout(lc->con, 0, "Different aperture radius!");
		if (filter) {
			if (strcmp(lc->filter, filter)!=0)
				printout(lc->con, 0, "Different optical filter!");
		} else {
			if (lc->filter)
				printout(lc->con, 0, "Different optical filter!");
		}
	}

	var = lc->var.list;
	comp = lc->comp.list;
	chk = lc->chk.list;

	/* Read magnitudes */
	count = cmpack_pht_nstar(pht);
	for (i=0; i<count; i++) {
		st = cmpack_pht_get_star(pht, i);
		xref = cmpack_pht_star_get_refid(st);
		if (xref>=0) {
			in_st = NULL; 
			for (j=0; j<lc->var.count && !in_st; j++)
				if (lc->var.list[j].id==xref) in_st=&var[j]; 
			for (j=0; j<lc->comp.count && !in_st; j++)
				if (lc->comp.list[j].id==xref) in_st=&comp[j];
			for (j=0; j<lc->chk.count && !in_st; j++)
				if (lc->chk.list[j].id==xref) in_st=&chk[j];
			if (in_st) {
				in_st->mask = 0; 
				/* Get magnitude for selected aperture */
				if (cmpack_pht_star_get_measurement(st, lc->aperture, &data)==0) {
					in_st->mask = data.mask & (CMPACK_MASK_MAG | CMPACK_MASK_ERR);
					in_st->mag = data.magnitude;
					in_st->err = data.error;
				} else {
					in_st->mask = 0; 
				}
			}
		}
		cmpack_unref(st);
	}

	return inst(lc, jd, frame, var, comp, chk);
}

/* Free allocated memory */
int list_inst_close(CmpackLister *lc)
{
	if (lc->var.list) {
		cmpack_free(lc->var.list);
		lc->var.list = NULL;
		lc->var.count = 0;
	}
	if (lc->comp.list) {
		cmpack_free(lc->comp.list);
		lc->comp.list = NULL;
		lc->comp.count = 0;
	}
	if (lc->chk.list) {
		cmpack_free(lc->chk.list);
		lc->chk.list = NULL;
		lc->chk.count = 0;
	}
	return 0;
}
