/**************************************************************

mdark.c (C-Munipack project)
Master-dark tool
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "context.h"
#include "comfun.h"
#include "console.h"
#include "list.h"
#include "robmean.h"
#include "ccdfile.h"
#include "cmpack_common.h"
#include "cmpack_mdark.h"

/***********************   DATA TYPES   ***********************/

/* Master-dark configuration context */
struct _CmpackMasterDark
{
	CmpackContext gc;			/**< Global context */
	CmpackConsole *con;			/**< Console */
	CmpackCcdFile *outfile;		/**< Output file context */
	CmpackBitpix bitpix;		/**< Required image data format */
	int scalable;		   		/**< Make scalable master-dark */
	CmpackBorder border;		/**< Border size */
	int in_bitpix;         		/**< Output image data format */
	int in_width;				/**< Width of frames */
	int in_height;         		/**< Height of frames */
	double exptime;				/**< Exposure duration in seconds */
	CmpackCcdFile *header;		/**< Header of first frame */
	CmpackList *frames;    		/**< List of source frames */
};

typedef struct _MDarkFrame
{
	CmpackBitpix bitpix;
	double minvalue, maxvalue;
	void *ccd; 
} MDarkFrame;

static void frame_free(void *frame)
{
	cmpack_free(((MDarkFrame*)frame)->ccd);
	cmpack_free(frame);
}

/**********************   LOCAL FUNCTIONS   ***********************/

/* Clean configuration context */
static void mdark_clean(void *ctx)
{
	CmpackMasterDark *lc = (CmpackMasterDark*)ctx;

	list_free_with_items(lc->frames, &frame_free);
	cmpack_unref(lc->header);
	cmpack_unref(lc->outfile);
	cmpack_unref(lc->con);
	gc_clean(lc);
}

/* Get pixel value */
static double getpixel(const void *ccd, CmpackBitpix bitpix, int pixel)
{
	switch (bitpix)
	{
	case CMPACK_BITPIX_SCHAR:  
		return ((char*)ccd)[pixel];
	case CMPACK_BITPIX_UCHAR:  
		return ((unsigned char*)ccd)[pixel];
	case CMPACK_BITPIX_SSHORT:
		return ((short*)ccd)[pixel];
	case CMPACK_BITPIX_USHORT:
		return ((unsigned short*)ccd)[pixel];
	case CMPACK_BITPIX_SLONG:
		return ((long*)ccd)[pixel];
	case CMPACK_BITPIX_ULONG:
		return ((unsigned long*)ccd)[pixel];
	case CMPACK_BITPIX_FLOAT:
		return ((float*)ccd)[pixel];
	default:
		return ((double*)ccd)[pixel];
	}
}

/********************   PUBLIC FUNCTIONS   **************************/

/* Initializes the context */
CmpackMasterDark *cmpack_mdark_init(void)
{
	CmpackMasterDark *lc = gc_create(sizeof(CmpackMasterDark));
	lc->gc.clean = &mdark_clean;
	lc->header = cmpack_ccd_new();
	return lc;
}

/* Attach console */
void cmpack_mdark_set_console(CmpackMasterDark *ctx, CmpackConsole *con)
{
	if (con!=ctx->con) {
		cmpack_unref(ctx->con);
		ctx->con = cmpack_addref(con);
	}
}

/* Set output image data format */
int cmpack_mdark_set_bitpix(CmpackMasterDark *lc, CmpackBitpix bitpix)
{
	lc->bitpix = bitpix;
	return 0;
}

/* Set output image data format */
int cmpack_mdark_get_bitpix(CmpackMasterDark *lc, CmpackBitpix *bitpix)
{
	if (bitpix) *bitpix = lc->bitpix;
	return 0;
}

/* Set output image data format */
int cmpack_mdark_set_scalable(CmpackMasterDark *lc, int scalable)
{
	lc->scalable = scalable;
	return 0;
}

/* Set output image data format */
int cmpack_mdark_get_scalable(CmpackMasterDark *lc, int *scalable)
{
	if (scalable) *scalable = lc->scalable;
	return 0;
}

/* Set image border */
void cmpack_mdark_set_border(CmpackMasterDark *lc, const CmpackBorder *border)
{
	if (border)
		lc->border = *border;
	else
		memset(&lc->border, 0, sizeof(CmpackBorder));
}

/* Set image border */
void cmpack_mdark_get_border(CmpackMasterDark *lc, CmpackBorder *border)
{
	if (border)
		*border = lc->border;
}

/* Reads first image and opens destination file */
int cmpack_mdark_open(CmpackMasterDark *lc, CmpackCcdFile *outfile)
{
	char msg[256];

	/* Print configuration parameters */
	if (is_debug(lc->con)) {
		printout(lc->con, 1, "Master-dark context:");
		printpari(lc->con, "BitPix", 1, lc->bitpix);
		printpari(lc->con, "Scalable", 1, lc->scalable);
		sprintf(msg, "Border     : %d %d %d %d", lc->border.left, lc->border.top,
			lc->border.right, lc->border.bottom);
		printout(lc->con, 1, msg);
	}

	/* Initialize context */
	lc->outfile = cmpack_addref(outfile);
	ccd_clear_header(lc->header);
	list_free_with_items(lc->frames, &frame_free);
	lc->frames = NULL;
    return 0;
}

/* Reads the input frame and stores it in the image list */
int cmpack_mdark_read(CmpackMasterDark *lc, CmpackCcdFile *file)
{
	int res, nx, ny;
    double exptime, minvalue, maxvalue;
	MDarkFrame *img;
	CmpackBitpix bitpix;

    /* Check source file */
    if (!file) {
        printout(lc->con, 0, "Invalid file context.");
 	    return CMPACK_ERR_INVALID_PAR;
    }

    /* Read parameters of source file */
	nx = ny = 0;
    cmpack_ccd_get_size(file, &nx, &ny);
    if (nx<=0 || ny<=0 || nx>=65536 || ny>=65536) 
	    return CMPACK_ERR_INVALID_SIZE;

	bitpix = cmpack_ccd_get_bitpix(file);
	cmpack_ccd_get_range(file, &minvalue, &maxvalue);
	if (bitpix==0)
		return CMPACK_ERR_INVALID_BITPIX;

	exptime = 0.0;
	if (cmpack_ccd_get_exptime(file, &exptime)!=0)
		return CMPACK_ERR_READ_ERROR;
	if (exptime<=0.0)
		return CMPACK_ERR_INVALID_EXPTIME;

    if (lc->in_width==0 && lc->in_height==0) {
        /* First image */
        lc->in_width = nx;
        lc->in_height= ny;
		lc->exptime = exptime;
		lc->in_bitpix = bitpix;
		ccd_copy_header(file, lc->header);
	} else {
		/* Next image */
		if (nx!=lc->in_width || ny!=lc->in_height) 
			return CMPACK_ERR_DIFF_SIZE_SRC;
		if (bitpix!=lc->in_bitpix)
			return CMPACK_ERR_DIFF_BITPIX_SRC;
	}

    /* read image data */
	img = (MDarkFrame*)cmpack_malloc(sizeof(MDarkFrame));
	img->bitpix = bitpix;
	img->minvalue = minvalue;
	img->maxvalue = maxvalue;
	img->ccd = image_new(nx, ny, bitpix);
	res = cmpack_ccd_get_image(file, bitpix, 0, 0, img->ccd);
	if (res!=0) {
		/* Failed */
		cmpack_free(img->ccd);
		cmpack_free(img);
		return res;
	}

	/* Add this image to image list */
	lc->frames = list_prepend(lc->frames, img);
	return 0;
}

int cmpack_mdark_close(CmpackMasterDark *lc)
{
	int res, x, y, ix, jx, iy, jy, i, k, nframes, nx, ny;
	int underflow = 0, overflow = 0;
	double a, b, *outf, *fbuf, minvalue, maxvalue;
	char msg[MAXLINE];
	MDarkFrame **inf;
	CmpackList *ptr;
	CmpackBitpix bitpix;

	/* Check context */
	if (!lc->outfile) 
		return CMPACK_ERR_NO_OUTPUT_FILE;

	/* Check input data */
	nframes = list_count(lc->frames);
	if (nframes<=0) {
		cmpack_unref(lc->outfile);
		lc->outfile = NULL;
		return CMPACK_ERR_NO_INPUT_FILES;
	}
	nx = lc->in_width;
	ny = lc->in_height;
	if (nx<=0 || ny<=0) {
		cmpack_unref(lc->outfile);
		lc->outfile = NULL;
		return CMPACK_ERR_INVALID_SIZE;
	}

	if (lc->bitpix==CMPACK_BITPIX_AUTO) {
		/* The output frame have the same format as the first source frame */
		bitpix = lc->in_bitpix;
	} else {
		/* Use required image data format */
		bitpix = lc->bitpix;
	}
	if (!pixrange(bitpix, &minvalue, &maxvalue)) {
		/* Invalid image data format */
		cmpack_unref(lc->outfile);
		lc->outfile = NULL;
		return CMPACK_ERR_INVALID_BITPIX;
	}

	/* makes array of images for faster access to data */
	inf = (MDarkFrame**)cmpack_malloc(nframes*sizeof(MDarkFrame*));
	for (ptr=lc->frames, k=0; ptr!=NULL; ptr=ptr->next, k++)
		inf[k] = (MDarkFrame*)ptr->ptr;

	outf = cmpack_calloc(nx*ny, sizeof(double));
	fbuf = cmpack_malloc(nframes*sizeof(double));
	ix = lc->border.left; jx = nx - lc->border.right;
	iy = lc->border.top;  jy = ny - lc->border.bottom;
	for (y=iy; y<jy; y++) {
		for (x=ix; x<jx; x++) {
			i = x + y*nx;
			/* Computation of mean */
			for (k=0; k<nframes; k++) 
				fbuf[k]=getpixel(inf[k]->ccd, inf[k]->bitpix, i);
			cmpack_robustmean(nframes, fbuf, &a, &b);
			/* Range checking */
			if (a < minvalue) {
				underflow++;
				a = minvalue;
			} else
			if (a > maxvalue) {
				overflow++;
				a = maxvalue;
			}
			/* Store the mean into output frame */
			outf[i] = a;
		}
	}

	/* Clear input frames */
	list_free_with_items(lc->frames, &frame_free);
	lc->frames = NULL;
	cmpack_free(inf);
	cmpack_free(fbuf);

	/* Write header information */
	ccd_prepare(lc->outfile, nx, ny, bitpix);
	ccd_copy_header(lc->header, lc->outfile);
	ccd_set_object(lc->outfile, "Master-dark frame");
	ccd_set_frames(lc->outfile, nframes);
	ccd_set_exptime(lc->outfile, lc->exptime);
	ccd_set_bool(lc->outfile, "SCALABLE", lc->scalable, "SCALABLE DARK FRAME");
	ccd_set_origin(lc->outfile);
	ccd_set_pcdate(lc->outfile);

	if (is_debug(lc->con)) {
		sprintf(msg, "Data format  : %d", bitpix); 
		printout(lc->con, 1, msg);
		sprintf(msg, "Min. value   : %.12g", minvalue); 
		printout(lc->con, 1, msg);
		sprintf(msg, "Max. value   : %.12g", maxvalue); 
		printout(lc->con, 1, msg);
		sprintf(msg, "Scalable     : %s", (lc->scalable ? "yes" : "no"));
		printout(lc->con, 1, msg);
	}

	/* Write image data */
	res = ccd_write_image(lc->outfile, bitpix, outf);
	cmpack_free(outf);

	/* Range check */
	if (overflow>0) {
		sprintf(msg,"An overflow has been occurred on %d of %d pixels during computation (max.=%.12g).", 
			overflow, nx*ny, maxvalue);
		printout(lc->con, 0, msg);
	}
	if (underflow>0) {
		sprintf(msg,"An underflow has been occurred on %d of %d pixels during computation (min.=%.12g).", 
			underflow, nx*ny, minvalue);
		printout(lc->con, 0, msg);
	}

	/* Clear file name */
	ccd_clear_header(lc->header);
	cmpack_unref(lc->outfile);
	lc->outfile = NULL;
	return res;
}
