/**************************************************************

mfind.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cmpack_common.h"
#include "cmpack_mfind.h"
#include "context.h"
#include "console.h"
#include "comfun.h"

/* Default threshold in % */
#define DEF_THRESHOLD		60.0

/* Clipping threshold */
#define SIGMA				3.0

/* Max. number of stars used in automatic finding of best comparison star */
#define RSTARS 30

struct _CmpackStarData
{
	int npoints;
	double value;
};
typedef struct _CmpackStarData CmpackStarData;

/* Master-flat frame context */
struct _CmpackMuniFind
{
	CmpackContext gc;			/**< Global context */
	CmpackConsole *con;			/**< Console */
	int comp_star;				/**< Comparison star identifier */
	double threshold;			/**< Required image data format */
};

/********************   PUBLIC FUNCTIONS   ******************************/

/* Initializes the context */
CmpackMuniFind *cmpack_mfind_init(void)
{
	CmpackMuniFind *lc = gc_create(sizeof(CmpackMuniFind));
	lc->comp_star = -1;
	lc->threshold = DEF_THRESHOLD;
	return lc;
}

/* Attach console */
void cmpack_mfind_set_console(CmpackMuniFind *ctx, CmpackConsole *con)
{
	if (con!=ctx->con) {
		cmpack_unref(ctx->con);
		ctx->con = cmpack_addref(con);
	}
}

/* Set output image data format */
int cmpack_mfind_get_comparison(CmpackMuniFind *lc)
{
	return lc->comp_star;
}

/* Set output image data format */
void cmpack_mfind_set_threshold(CmpackMuniFind *lc, double threshold)
{
	lc->threshold = threshold;
}

/* Make mag-dev graph */
int cmpack_mfind(CmpackMuniFind *lc, CmpackAllFile *file, CmpackTable *table,
				 int comp_star)
{
	char msg[256];
	int i, j, c, v, row, count, vframes, valid, npoints, mask, nmeasur, vmask, cmask;
	int nframes, nstars, nreferr, nrequired, nwritten, nbadpoints, cmp_npoints;
	double *frames, q0, sum, vmag, cmag, mean, stddev;

	/* Clear the table */
	cmpack_tab_clear(table);
	cmpack_tab_set_type(table, CMPACK_TABLE_MUNIFIND);

	/* Check parameters */
	if (lc->threshold<0 || lc->threshold>100) {
		sprintf(msg, "Invalid value of threshold (%.1f %%)", lc->threshold);
		printout(lc->con, 0, msg); 
		return CMPACK_ERR_INVALID_PAR;
	} 

	/* Print configuration parameters */
	if (is_debug(lc->con)) {
		printout(lc->con, 1, "Munifind parametets:");
		if (comp_star>=0)
			printpari(lc->con, "Comparison star", 1, comp_star);
		else
			printpars(lc->con, "Comparison star", 1, "autodetection");
		printpard(lc->con, "Threshold", 1, lc->threshold, 1);
	}

	/* Init table */
	cmpack_tab_add_column(table, "INDEX", CMPACK_COLUMN_INTEGER, 0);
	cmpack_tab_add_column(table, "MEAN_MAG", CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	cmpack_tab_add_column(table, "STDEV", CMPACK_COLUMN_DOUBLE, MAG_PRECISION);
	cmpack_tab_add_column(table, "GOODPOINTS", CMPACK_COLUMN_INTEGER, 0);

	nframes = cmpack_all_nframes(file);
	nstars = cmpack_all_nstars(file);
	if (nframes<=1 || nstars<=1) {
		sprintf(msg, "Not enough input data");
		printout(lc->con, 0, msg); 
		return CMPACK_ERR_INVALID_PAR;
	} 

	/* Reference star autodetection */
	if (comp_star<0) {
		/* Memory allocation */
		double *fmag = (double*)cmpack_malloc(nframes*sizeof(double));
		double *sdev = (double*)cmpack_malloc(nstars*sizeof(double));
		int *sok = (int*)cmpack_malloc(nstars*sizeof(int));
		/* Compute the number of frames which at least one star is present on */
		vframes = 0;
		for (i=0; i<nstars; i++) {
			count = 0;
			for (j=0; j<nframes; j++) {
				cmpack_all_get(file, j, i, &mask, NULL, NULL);
				if (mask & CMPACK_MASK_MAG)
					count++;
			}
			if (vframes < count)
				vframes = count;
		}
		/* Set scnt to 1 for stars which have the same number of valid measurements */
		for (i=0; i<nstars; i++) {
			count = 0;
			for (j=0; j<nframes; j++) {
				cmpack_all_get(file, j, i, &mask, NULL, NULL);
				if (mask & CMPACK_MASK_MAG)
					count++;
			}
			sok[i] = (count == vframes);
		}
		for (c=0; c<nstars; c++) {
			if (sok[c]) {
				count = 0; sum = 0.0;
				for (v=0; v<nstars; v++) {
					if (v!=c && sok[v]) {
						i = 0;
						for (j=0; j<nframes; j++) {
							cmpack_all_get(file, j, c, &cmask, &cmag, NULL);
							cmpack_all_get(file, j, v, &vmask, &vmag, NULL);
							if (vmask & cmask & CMPACK_MASK_MAG)
								fmag[i++] = vmag - cmag;
						}
						if (i>=2) {
							cmpack_robustmean(i, fmag, NULL, &stddev);
							sum += stddev;
							count++;
						}
					}
				}
				if (count>0) {
					sdev[c] = sum/count;
				} else {
					sok[c] = 0;
				}
			}
		}
		/* Select star with lowest deviation */
		j = -1; q0 = 99.9e9;
		for (i=0; i<nstars; i++) {
			if (sok[i] && sdev[i] < q0) {
				j = i;
				q0 = sdev[i];
			}
		}
		cmpack_free(fmag);
		cmpack_free(sdev);
		cmpack_free(sok);
		comp_star = j;
	}

	/* Check comparison star */
	cmp_npoints = 0;
	if (comp_star>=0 && comp_star<nstars) {
		for (j=0; j<nframes; j++) {
			cmpack_all_get(file, j, comp_star, &mask, NULL, NULL);
			if (mask & CMPACK_MASK_MAG)
				cmp_npoints++;
		}
	}
	if (cmp_npoints<2) {
		sprintf(msg, "Invalid comparison star");
		printout(lc->con, 0, msg); 
		return CMPACK_ERR_REF_NOT_FOUND;
	}
	lc->comp_star = comp_star;

	/* Compute minimum required measurements */
	nreferr = nframes - cmp_npoints;
	nrequired = (int)(0.01 * lc->threshold * cmp_npoints);
	if (nrequired<2) 
		nrequired = 2;

	/* computing average and std. dev. for each star */
	frames = (double*)cmpack_malloc(nframes*sizeof(double));
	nwritten = nbadpoints = nmeasur = 0;
	for (i=0; i<nstars; i++) {
		valid = 0;
		if (i!=comp_star) {
			count = 0;
			for (j=0; j<nframes; j++) {
				cmpack_all_get(file, j, i, &vmask, &vmag, NULL);
				cmpack_all_get(file, j, comp_star, &cmask, &cmag, NULL);
				if ((cmask & vmask) & CMPACK_MASK_MAG)
					frames[count++] = vmag - cmag;
			}
			if (count>=nrequired) {
				cmpack_robustmean(count, frames, &mean, &stddev);
				valid = 1;
			}
			npoints = count;
		} else {
			npoints = cmp_npoints;
			valid = 0;
		}
		/* Store data to table */
		row = cmpack_tab_add_row(table);
		cmpack_tab_ptdi(table, row, 0, i);
		if (valid) {
			cmpack_tab_ptdd(table, row, 1, mean);
			cmpack_tab_ptdd(table, row, 2, stddev);
		} else {
			cmpack_tab_ptdd(table, row, 1, 99.9999);
			cmpack_tab_ptdd(table, row, 2, 9.9999);
		}
		cmpack_tab_ptdi(table, row, 3, npoints); 
		nbadpoints += (cmp_npoints - npoints);
		nwritten++;
	}
	cmpack_free(frames);
	
	/* Print messages */
	sprintf(msg,"Number of analyzed stars:           %d (w/o reference star)", nstars-1);
	printout(lc->con, 1, msg);
	sprintf(msg,"Number of source files:             %d", nframes);
	printout(lc->con, 1, msg);
	sprintf(msg,"Total number of data points:        %d", nmeasur);
	printout(lc->con, 1, msg);
	sprintf(msg,"Reference star used:                %d", comp_star);
	printout(lc->con, 1, msg);
	sprintf(msg,"Reference star errors:              %d (%.0f%%)", nreferr, 100.0*nreferr/nmeasur);
	printout(lc->con, 1, msg);
	sprintf(msg,"Number of good points required:     %d", nrequired);
	printout(lc->con, 1, msg);
	sprintf(msg,"Number of stars written to output:  %d (%.0f%%)", nwritten, 100.0*nwritten/(nstars-1));
	printout(lc->con, 1, msg);
	sprintf(msg,"Number of points ruled out:         %d (%.0f%%)", nbadpoints, 100.0*nbadpoints/nmeasur);
	printout(lc->con, 1, msg);
	
    return 0;
}
