/**************************************************************

phot_head.c (C-Munipack project)
Photometry file header
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "context.h"
#include "console.h"
#include "comfun.h"
#include "cmpack_common.h"
#include "cmpack_phtfile.h"
#include "phthead.h"

/*************************   Constants  *************************/

#define ALLOC_BY 	64

/********************   Local functions    **********************/

/* Init head item */
static CmpackHeadItem *headitem_create(const char *key, const char *val, const char *com)
{
	CmpackHeadItem *item = cmpack_malloc(sizeof(CmpackHeadItem));
	item->key = cmpack_strdup(key);
	item->val = cmpack_strdup(val);
	item->com = cmpack_strdup(com);
	return item;
}

/* Makes copy of header item */
static void headitem_copy(CmpackHeadItem *dst, const CmpackHeadItem *src)
{
	dst->key = cmpack_strset(dst->key, src->key);
	dst->val = cmpack_strset(dst->val, src->val);
	dst->com = cmpack_strset(dst->com, src->com);
}

/* Change value of header item */
static void headitem_setval(CmpackHeadItem *item, const char *val)
{
	item->val = cmpack_strset(item->val, val);
}

/* Change comment of header item */
static void headitem_setcom(CmpackHeadItem *item, const char *com)
{
	item->com = cmpack_strset(item->com, com);
}

/* Destroy head item */
static void headitem_free(CmpackHeadItem *item)
{
	cmpack_free(item->key);
	cmpack_free(item->val);
	cmpack_free(item->com);
	cmpack_free(item);
}

/* Dump head item to stdout */
static void headitem_dump(CmpackConsole *con, CmpackHeadItem *item)
{
	char aux[MAXLINE];

	if (item->key && item->val) {
		sprintf(aux, "\t\t%s", item->key);
		if (item->val) {
			strcat(aux, " = ");
			strcat(aux, item->val);
		}
		if (item->com) {
			strcat(aux, " ; ");
			strcat(aux, item->com);
		}
		printout(con, 1, aux);
	}
}

/* Write item to stdout */
static void headitem_write(CmpackHeadItem *item, FILE *to)
{
	if (item->key && item->val) {
		if (item->com) 
			fprintf(to, "\t<%s>%s <!-- %s --></%s>\n", item->key, item->val, item->com, item->key);
		else
			fprintf(to, "\t<%s>%s</%s>\n", item->key, item->val, item->key);
	}
}

/* Clear all parameters in the section */
static void head_clear(CmpackHeader *head)
{
	int j;

	for (j=0; j<head->count; j++)
		headitem_free(head->list[j]);
	cmpack_free(head->list);
	head->list = NULL;
	head->count = head->capacity = 0;
	hash_clear(&head->hash);
}

/* Search for a headitem by its name */
static int head_search(CmpackHeader *head, const char *name)
{
	int i;

	CmpackHeadItem *item = hash_search(&head->hash, name);
	if (item) {
		for (i=0; i<head->count; i++) {
			if (head->list[i] == item)
				return i;
		}
	}
	return -1;
}

/********************   Public functions    ***********************/

/* Init header */
void pht_head_init(CmpackHeader *head)
{
	memset(head, 0, sizeof(CmpackHeader));
	hash_init(&head->hash);
}	

/* Free allocated memory */
void pht_head_clear(CmpackHeader *head)
{
	head_clear(head);
}	

/* Make copy of header */
void pht_head_copy(CmpackHeader *dst, const CmpackHeader *src)
{
	int j;
	
	head_clear(dst);
	if (src->count>0) {
		dst->capacity = dst->count = src->count;
		dst->list = (CmpackHeadItem**) cmpack_calloc(dst->capacity, sizeof(CmpackHeadItem*));
		for (j=0; j<src->count; j++) {
			dst->list[j] = headitem_create(NULL, NULL, NULL);
			headitem_copy(dst->list[j], src->list[j]);
			hash_insert(&dst->hash, dst->list[j]->key, dst->list[j]);
		}
	}
}

/* Dump content of header */
void pht_head_dump(CmpackConsole *con, CmpackHeader *head)
{
	int i;
	
	printout(con, 1, "\tHeader:");
	for (i=0; i<head->count; i++) {
		headitem_dump(con, head->list[i]);
	}
}	

/* Write content to the file */
void pht_head_write(CmpackHeader *head, FILE *to)
{
	int i;

	for (i=0; i<head->count; i++) {
		headitem_write(head->list[i], to);
	}
}	

void pht_head_set(CmpackHeader *head, const char *key, const char *val, const char *com)
{
	CmpackHeadItem *item = (CmpackHeadItem*) hash_search(&head->hash, key);
	if (item) {
		/* Modify existing item */
		if (val) headitem_setval(item, val);
		if (com) headitem_setcom(item, com);
	} else {
		/* Add new item */
		if (head->count>=head->capacity) {
			head->capacity += ALLOC_BY;
			head->list = (CmpackHeadItem**)cmpack_realloc(head->list, head->capacity*sizeof(CmpackHeadItem*));
		}
		head->list[head->count] = headitem_create(key, val, com);
		hash_insert(&head->hash, key, head->list[head->count]);
		head->count++;
	}
}

CmpackHeadItem *pht_head_get(CmpackHeader *head, const char *key)
{
	return hash_search(&head->hash, key);
}

void pht_head_delete(CmpackHeader *head, const char *key)
{
	int index = head_search(head, key);
	if (index>=0) {
		CmpackHeadItem *item = head->list[index];
		/* Delete item */
		hash_delete(&head->hash, item->key, item);
		headitem_free(item);
		head->count--;
		if (head->count==0) {
			/* If there is no item, clear the list */
			cmpack_free(head->list);
			head->list = NULL;
			head->count = head->capacity = 0;
		} else {
			/* Pack the list (move all following items to the left) */
			size_t len = head->count - index;
			if (len>0) {
				CmpackHeadItem **aux = (CmpackHeadItem**)cmpack_malloc(len*sizeof(CmpackHeadItem*));
				memcpy(aux, item+1, len*sizeof(CmpackHeadItem*));
				memcpy(item, aux, len*sizeof(CmpackHeadItem*));
				cmpack_free(aux);
			}
		}
	}
}
