/**************************************************************

phwrite.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "comfun.h"
#include "console.h"
#include "robmean.h"
#include "cmpack_common.h"
#include "cmpack_phtfile.h"
#include "phot.h"
#include "phfun.h"
#include "phwrite.h"

int SrtWrite(CmpackPhot *kc, CmpackPhotFrame *frame, CmpackPhtFile *pht)
/* Writes the photometry file */
{
	int		i, stid;
	CmpackDateTime dt;
	char	datestr[64], timestr[64], actdate[64], origin[128];
	time_t	tsec;
	struct tm *actim;
	CmpackPhotStar *st;
	CmpackPhtMeasurement data;

	/* Clear photometry file */
	cmpack_pht_clear(pht);

	/* Frame information */
	cmpack_pht_pkyi(pht, "width", frame->ncol, "Image width in pixels");
	cmpack_pht_pkyi(pht, "height", frame->nrow, "Image height in pixels");
	cmpack_pht_pkyd(pht, "jd", frame->jd, JD_PRECISION, "Julian date of observation");
	cmpack_decodejd(frame->jd, &dt);
	cmpack_datetostr(&dt.date, datestr, 64);
	cmpack_pht_pkys(pht, "date", datestr, "Date of observation");
	cmpack_timetostr(&dt.time, timestr, 64);
	cmpack_pht_pkys(pht, "time", timestr, "Time of observation");
	cmpack_pht_pkys(pht, "filter", frame->filter, "Optical filter");
	cmpack_pht_pkyd(pht, "exptime", frame->exptime, EXP_PRECISION, "Exposure duration in seconds");
	if (frame->ccdtemp>-999 && frame->ccdtemp<999)
		cmpack_pht_pkyd(pht, "temp", frame->ccdtemp, TEMP_PRECISION, "CCD temperature in deg.C");
	if (frame->object)
		cmpack_pht_pkys(pht, "object", frame->object, "Object's designation");
	sprintf(origin, "%s %s", PACKAGE_NAME, VERSION);
	cmpack_pht_pkys(pht, "origin", origin, "Reduction software");

	/* Set actual date and time */
	tsec = time(&tsec);
	actim = localtime(&tsec);
	sprintf(actdate, "%04d-%02d-%02d %02d:%02d:%02d", actim->tm_year+1900, actim->tm_mon+1, 
		actim->tm_mday, actim->tm_hour, actim->tm_min, actim->tm_sec);
	cmpack_pht_pkys(pht, "crdate", actdate, "Date of creation");

	/* Photometry process information */
	cmpack_pht_pkyi(pht, "phot_stars", frame->nstar, "Number of stars");
	cmpack_pht_pkyd(pht, "phot_datalo", frame->datalo, 3, "Lowest good pixel value");
	cmpack_pht_pkyd(pht, "phot_datahi", frame->datahi, 3, "Highest good pixel value");
	cmpack_pht_pkyd(pht, "phot_gain", kc->gain, 3, "Gain in electrons per ADU");
	cmpack_pht_pkyd(pht, "phot_rnoise", kc->readns, 3, "Readout noise in ADU");
	cmpack_pht_pkyd(pht, "phot_fwhm_exp", kc->fwhm, 3, "Expected FWHM of objects");
	cmpack_pht_pkyd(pht, "phot_fwhm_mean", frame->fwhm_med, 3, "Mean FWHM of objects");
	cmpack_pht_pkyd(pht, "phot_fwhm_err", frame->fwhm_err, 3, "Std. error of FWHM of objects");
	cmpack_pht_pkyd(pht, "phot_thresh", kc->thresh, 3, "Detection threshold");
	cmpack_pht_pkyd(pht, "phot_losharp", kc->shrplo, 3, "Low sharpness cutoff");
	cmpack_pht_pkyd(pht, "phot_hisharp", kc->shrphi, 3, "High sharpness cutoff");
	cmpack_pht_pkyd(pht, "phot_loround", kc->rndlo, 3, "Low roundness cutoff");
	cmpack_pht_pkyd(pht, "phot_hiround", kc->rndhi, 3, "High roundness cutoff");
	if (frame->fravg>1)
		cmpack_pht_pkyi(pht, "fravg", frame->fravg, "Number of frames averaged");
	if (frame->frsum>1)
		cmpack_pht_pkyi(pht, "frsum", frame->frsum, "Number of frames summed");

	/* Apertures */
	for (i=0; i<frame->naper; i++) {
		CmpackPhtAperture *ap = cmpack_pht_aperture_new(frame->apid[i]);
		cmpack_pht_aperture_set_radius(ap, frame->aper[i]);
		cmpack_pht_add_aperture(pht, ap);
	}

	/* Star data */
	st = FirstStar(frame);
	stid = 1;
	while (st) {
		CmpackPhtStar *star = cmpack_pht_star_new(stid++);
		cmpack_pht_star_set_center(star, st->xcen, st->ycen);
		/* Parameters */
		/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		cmpack_pht_pstd(pht, star, "skymod", st->skymod, 3);
		cmpack_pht_pstd(pht, star, "skysig", st->skysig, 3);
		cmpack_pht_pstd(pht, star, "sharpness", st->sharpness, 3);
		cmpack_pht_pstd(pht, star, "roundness", st->roundness, 3);
		cmpack_pht_pstd(pht, star, "fwhm", st->fwhm, 3);
		cmpack_pht_pstd(pht, star, "height", st->height, 3);
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
		/* Magnitudes */
		for (i=0; i<frame->naper; i++) {
			if (st->apmag[i]<99) {
				data.mask = CMPACK_MASK_MAG | CMPACK_MASK_ERR;
				data.code = 0;
				data.magnitude = st->apmag[i];
				data.error = st->magerr[i];
				cmpack_pht_star_set_measurement(star, frame->apid[i], &data);
			}
		}
		cmpack_pht_add_star(pht, star);
		st = NextStar(frame, st);
	}
	return 0;
}
