/**************************************************************

checkfiles_dlg.cpp (C-Munipack project)
Filter for files that are being appended to the project
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "addfilter_dlg.h"
#include "utils.h"
#include "main.h"

struct tPageData
{
	GtkWidget *box, *scrwnd, *title, *check, *view, *total;
	GtkTreeModel *model, *fmodel;
	bool allfiles, skip;
};

struct tFileData
{
	char *path;
	char *filter, *object;
	double jd, exptime;
};

enum tTreeColumns
{
	DATA_CHECK,
	DATA_VALUE,
	DATA_FILES,
	DATA_COLUMNS
};

static void UpdateTotal(tPageData *data);

static void PrintFiles(char *buf, int files)
{
	if (files==0)
		strcpy(buf, "No file");
	else if (files==1)
		strcpy(buf, "1 file");
	else 
		sprintf(buf, "%d files", files); 
}

static void RenderFilter(GtkTreeViewColumn *col, GtkCellRenderer *cell,
	GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	int files;
	char buf[256], aux[64], *filter;

	gtk_tree_model_get(model, iter, DATA_VALUE, &filter, DATA_FILES, &files, -1);
	PrintFiles(aux, files);
	if (filter)
		g_snprintf(buf, sizeof(buf), "%s (%s)", filter, aux); 
	else
		g_snprintf(buf, sizeof(buf), "%s (%s)", "<empty string>", aux); 
	g_object_set(cell, "text", buf, NULL);
	g_free(filter);
}

static void RenderExposure(GtkTreeViewColumn *col, GtkCellRenderer *cell,
	GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	int files;
	double exptime;
	char aux[64], buf[256];

	gtk_tree_model_get(model, iter, DATA_VALUE, &exptime, DATA_FILES, &files, -1);
	PrintFiles(aux, files);
	g_snprintf(buf, sizeof(buf), "%.2f seconds (%s)", exptime, aux);
	g_object_set(cell, "text", buf, NULL);
}

static void RenderObject(GtkTreeViewColumn *col, GtkCellRenderer *cell,
	GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	int files;
	char buf[256], aux[64], *object;

	gtk_tree_model_get(model, iter, DATA_VALUE, &object, DATA_FILES, &files, -1);
	PrintFiles(aux, files);
	if (object)
		g_snprintf(buf, sizeof(buf), "%s (%s)", object, aux); 
	else
		g_snprintf(buf, sizeof(buf), "%s (%s)", "<empty string>", aux); 
	g_object_set(cell, "text", buf, NULL);
	g_free(object);
}

static void OnCellToggled(GtkCellRendererToggle *cell, char *path, tPageData *pData)
{
	GtkTreeIter fiter, iter;

	if (gtk_tree_model_get_iter_from_string(pData->fmodel, &fiter, path)) {
		gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(pData->fmodel),
			&iter, &fiter);
		gtk_list_store_set(GTK_LIST_STORE(pData->model), &iter, DATA_CHECK,
			!gtk_cell_renderer_toggle_get_active(cell), -1);
		UpdateTotal(pData);
	}
}

static void OnCheckToggled(GtkToggleButton *button, tPageData *pData)
{
	pData->allfiles = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pData->check));
	gtk_widget_set_sensitive(pData->view, !pData->allfiles);
	UpdateTotal(pData);
}

static gboolean FilterRows(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
	int files;

	gtk_tree_model_get(model, iter, DATA_FILES, &files, -1);
	return files>0;
}

static tPageData *MakePage(const char *feature, GtkListStore *model,
						   GtkTreeCellDataFunc datafunc)
{
	char buf[512];
	tPageData *data = (tPageData*)g_malloc0(sizeof(tPageData));
	GtkTreeViewColumn *col;
	GtkCellRenderer *renderer;

	data->box = gtk_vbox_new(FALSE, 8);

	// Title
	data->title = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(data->title), 0.0, 0.5);
	gtk_widget_set_size_request(data->title, 400, -1);
	sprintf(buf, "<b>The selection consists of CCD frames of different %s. "
		"Which files do you want to add to the project?</b>", feature);
	gtk_label_set_markup(GTK_LABEL(data->title), buf);
	gtk_label_set_line_wrap(GTK_LABEL(data->title), TRUE);
	gtk_box_pack_start(GTK_BOX(data->box), data->title, FALSE, TRUE, 0);

	// Check box (all files)
	data->check = gtk_check_button_new_with_label("All files");
	g_signal_connect(G_OBJECT(data->check), "toggled", G_CALLBACK(OnCheckToggled), data);
	gtk_box_pack_start(GTK_BOX(data->box), data->check, FALSE, TRUE, 0);

	// Scrolled window
	data->scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(data->scrwnd),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(data->scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_box_pack_start(GTK_BOX(data->box), data->scrwnd, TRUE, TRUE, 0);

	// Tree model
	data->model = GTK_TREE_MODEL(model);
	g_object_ref(data->model);
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(data->model), DATA_VALUE, GTK_SORT_ASCENDING);
	data->fmodel = gtk_tree_model_filter_new(data->model, NULL);
	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(data->fmodel), FilterRows, NULL, NULL);
	g_object_ref(data->fmodel);

	// Tree view 
	data->view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(data->fmodel));
	gtk_container_add(GTK_CONTAINER(data->scrwnd), data->view);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(data->view), FALSE);
	// Value column (check box + text)
	col = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(data->view), col);
	renderer = gtk_cell_renderer_toggle_new();
	g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(OnCellToggled), data);
	g_object_set(renderer, "activatable", TRUE, NULL);
	gtk_tree_view_column_pack_start(col, renderer, FALSE);
	gtk_tree_view_column_add_attribute(col, renderer, "active", DATA_CHECK);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_set_cell_data_func(col, renderer, datafunc, NULL, NULL);
	
	// Number of selected files
	data->total = gtk_label_new(NULL);
	gtk_widget_set_size_request(data->title, 400, -1);
	gtk_misc_set_alignment(GTK_MISC(data->total), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(data->box), data->total, FALSE, TRUE, 0);	

	return data;
}

static void UpdateTotal(tPageData *data)
{
	gboolean ok, checked;
	char buf[256];
	int count, files;
	GtkTreeIter iter;

	// For each filter, compute number of files
	files = 0;
	ok = gtk_tree_model_get_iter_first(data->fmodel, &iter);
	while (ok) {
		gtk_tree_model_get(data->fmodel, &iter, DATA_FILES, &count, DATA_CHECK, &checked, -1);
		if (data->allfiles || checked)
			files += count;
		ok = gtk_tree_model_iter_next(data->fmodel, &iter);
	}
	// Update label
	if (files==0)
		gtk_label_set_text(GTK_LABEL(data->total), "No file selected.");
	else if (files==1)
		gtk_label_set_text(GTK_LABEL(data->total), "1 file selected.");
	else  {
		sprintf(buf, "%d files selected.", files);
		gtk_label_set_text(GTK_LABEL(data->total), buf);
	}
}

static GtkWidget *MakeSummary(void)
{
	return gtk_label_new(NULL);
}


static void PageFree(tPageData *data, gpointer *user_data)
{
	if (data->model)
		g_object_unref(data->model);
	if (data->fmodel)
		g_object_unref(data->fmodel);
	g_free(data);
}

static tFileData *MakeFile(const char *fpath)
{
	char	*f, *str;
	const char *filter, *object;
	CmpackCcdFile *file;
	CmpackDateTime dt;
	tFileData *data = NULL;

	if (Global.pProject->FileInProject(fpath))
		return NULL;

	f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
	if (cmpack_ccd_open(&file, f, CMPACK_OPEN_HEADONLY)==0) {
		data = (tFileData*)g_malloc0(sizeof(tFileData));
		data->path = g_strdup(fpath);
		filter = cmpack_ccd_get_filter(file);
		if (filter) {
			str = g_locale_to_utf8(filter, -1, NULL, NULL, NULL);
			if (str)
				g_strstrip(str);
			if (str && strlen(str)>0)
				data->filter = str;
			else
				g_free(str);
		}
		if (cmpack_ccd_get_datetime(file, &dt)==0) 
			data->jd = cmpack_encodejd(&dt);
		cmpack_ccd_get_exptime(file, &data->exptime);
		object = cmpack_ccd_get_object(file);
		if (object) {
			str = g_locale_to_utf8(object, -1, NULL, NULL, NULL);
			if (str)
				g_strstrip(str);
			if (str && strlen(str)>0)
				data->object = str;
			else
				g_free(str);
		}
		cmpack_unref(file);
	}
	g_free(f);
	return data;
}

static void FileFree(tFileData *data, gpointer *user_data)
{
	if (data) {
		g_free(data->path);
		g_free(data->filter);
		g_free(data->object);
		g_free(data);
	}
}

static gint CompareByTime(const tFileData *a, const tFileData *b)
{
	// Compare by time
	if (a->jd < b->jd)
		return -1;
	else if (a->jd > b->jd)
		return 1;
	
	// If they are equal, compare by path
	if (a->path) {
		if (!b->path)
			return -1;
		else
			return strcmp(a->path, b->path);
	} else {
		if (b->path)
			return 1;
		else
			return 0;
	}
}

//
// Constructor
//
CAddFilterDlg::CAddFilterDlg(GtkWindow *pParent):m_Pages(NULL), m_Files(NULL)
{
	m_pDlg = gtk_dialog_new_with_buttons("Add files...", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
		GTK_STOCK_GO_BACK, GTK_RESPONSE_NO, GTK_STOCK_GO_FORWARD, GTK_RESPONSE_YES, NULL);
	m_pClose = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_CLOSE);
	m_pCancel = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_CANCEL);
	m_pBack = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_NO);
	m_pNext = get_dialog_widget_by_response(m_pDlg, GTK_RESPONSE_YES);

	// Dialog icon
	gchar *icon = get_icon_file("addfiles");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Page 1: Color filter
	m_FilList = gtk_list_store_new(DATA_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INT);
	m_Pages = g_slist_append(m_Pages, MakePage("color filter", m_FilList, RenderFilter));

	// Page 2: Exposure duration
	m_ExpList = gtk_list_store_new(DATA_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_DOUBLE, G_TYPE_INT);
	m_Pages = g_slist_append(m_Pages, MakePage("exposure duration", m_ExpList, RenderExposure));

	// Page 3: Object name
	m_ObjList = gtk_list_store_new(DATA_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_INT);
	m_Pages = g_slist_append(m_Pages, MakePage("object name", m_ObjList, RenderObject));

	// Page 4: summary
	m_pLast = MakeSummary();

	m_pTabs = gtk_notebook_new();
	gtk_widget_set_size_request(m_pTabs, 400, 320);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), m_pTabs);
	gtk_container_set_border_width(GTK_CONTAINER(m_pTabs), 8);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(m_pTabs), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(m_pTabs), FALSE);
	for (GSList *ptr=m_Pages; ptr!=NULL; ptr=ptr->next) 
		gtk_notebook_append_page(GTK_NOTEBOOK(m_pTabs), ((tPageData*)ptr->data)->box, NULL);
	gtk_notebook_append_page(GTK_NOTEBOOK(m_pTabs), m_pLast, NULL);
	gtk_widget_show_all(m_pTabs);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
}

CAddFilterDlg::~CAddFilterDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_slist_foreach(m_Pages, (GFunc)PageFree, NULL);
	g_slist_free(m_Pages);
	m_Pages = NULL;
}


//
// Add files from the list
//
void CAddFilterDlg::Execute(const GSList *files)
{
	m_Files = PrepareFiles(files);
	m_Files = g_list_sort(m_Files, (GCompareFunc)CompareByTime);
	UpdateModels();
	GoToFirstPage();
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
	FreeFiles(m_Files);
}


//
// Add files from the folder
//
void CAddFilterDlg::Execute(const char *dirpath, bool recursive)
{
	m_Files = PrepareFolder(NULL, dirpath, recursive);
	m_Files = g_list_sort(m_Files, (GCompareFunc)CompareByTime);
	UpdateModels();
	GoToFirstPage();
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
	FreeFiles(m_Files);
}


//
// Make list of files
//
GList *CAddFilterDlg::PrepareFiles(const GSList *files)
{
	GList *list = NULL;
	for (const GSList *ptr=files; ptr!=NULL; ptr=ptr->next) {
		tFileData *data = MakeFile((char*)ptr->data);
		if (data)
			list = g_list_prepend(list, data);
	}
	return list;
}

GList *CAddFilterDlg::PrepareFolder(GList *list, const char *dirpath, bool recursive)
{
	char *filepath;
	const gchar *filename;
	GDir *dir;

	dir = g_dir_open(dirpath, 0, NULL);
	if (dir) {
		filename = g_dir_read_name(dir);
		while (filename) {
			filepath = g_build_filename(dirpath, filename, NULL);
			if (g_file_test(filepath, G_FILE_TEST_IS_DIR)) {
				if (recursive)
					list = PrepareFolder(list, filepath, true);
			} else 
			if (g_file_test(filepath, G_FILE_TEST_IS_REGULAR)) {
				tFileData *data = MakeFile(filepath);
				if (data)
					list = g_list_prepend(list, data);
			}
			g_free(filepath);
			filename = g_dir_read_name(dir);
		}
		g_dir_close(dir);
	}
	return list;
}


//
// Free allocated memory
//
void CAddFilterDlg::FreeFiles(GList *list)
{
	g_list_foreach(list, (GFunc)FileFree, NULL);
	g_list_free(list);
}

void CAddFilterDlg::response_dialog(GtkDialog *pDlg, gint response_id, CAddFilterDlg *pMe)
{
	if (!pMe->OnCloseQuery(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CAddFilterDlg::OnCloseQuery(int response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_YES:
		// Go to next page
		GoToForward();
		return false;

	case GTK_RESPONSE_NO:
		// Go to previous page
		GoToBack();
		return false;
	}
	return true;
}

void CAddFilterDlg::UpdateModels(void)
{
	bool ok;
	GtkTreeIter iter;

	// Filters
	gtk_list_store_clear(GTK_LIST_STORE(m_FilList));
	for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
		ok = true;
		for (GList *aux=ptr->prev; aux!=NULL; aux=aux->prev) {
			if (StrCmp0(((tFileData*)ptr->data)->filter, ((tFileData*)aux->data)->filter)==0) {
				ok = false;
				break;
			}
		}
		if (ok) {
			gtk_list_store_append(GTK_LIST_STORE(m_FilList), &iter);
			gtk_list_store_set(GTK_LIST_STORE(m_FilList), &iter, DATA_CHECK, false, 
				DATA_VALUE, ((tFileData*)ptr->data)->filter, DATA_FILES, 0, -1);
		}
	}
	// Exposures
	gtk_list_store_clear(GTK_LIST_STORE(m_ExpList));
	for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
		ok = true;
		for (GList *aux=ptr->prev; aux!=NULL; aux=aux->prev) {
			if (((tFileData*)ptr->data)->exptime == ((tFileData*)aux->data)->exptime) {
				ok = false;
				break;
			}
		}
		if (ok) {
			gtk_list_store_append(GTK_LIST_STORE(m_ExpList), &iter);
			gtk_list_store_set(GTK_LIST_STORE(m_ExpList), &iter, DATA_CHECK, false, 
				DATA_VALUE, ((tFileData*)ptr->data)->exptime, DATA_FILES, 0, -1);
		}
	}
	// Object names
	gtk_list_store_clear(GTK_LIST_STORE(m_ObjList));
	for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
		ok = true;
		for (GList *aux=ptr->prev; aux!=NULL; aux=aux->prev) {
			if (StrCmp0(((tFileData*)ptr->data)->object, ((tFileData*)aux->data)->object)==0) {
				ok = false;
				break;
			}
		}
		if (ok) {
			gtk_list_store_append(GTK_LIST_STORE(m_ObjList), &iter);
			gtk_list_store_set(GTK_LIST_STORE(m_ObjList), &iter, DATA_CHECK, false, 
				DATA_VALUE, ((tFileData*)ptr->data)->object, DATA_FILES, 0, -1);
		}
	}
}


void CAddFilterDlg::UpdateFilterPage(void)
{
	bool ok;
	int count, files;
	char *filter, aux[64], buf[256];
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 0);
	GtkTreeIter iter;

	gtk_tree_view_set_model(GTK_TREE_VIEW(data->view), NULL);
	// For each filter, compute number of files
	files = 0;
	ok = gtk_tree_model_get_iter_first(data->model, &iter);
	while (ok) {
		count = 0;
		gtk_tree_model_get(data->model, &iter, DATA_VALUE, &filter, -1);
		for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
			if (StrCmp0(filter, ((tFileData*)ptr->data)->filter)==0)
				count++;
		}
		g_free(filter);
		gtk_list_store_set(GTK_LIST_STORE(data->model), &iter, DATA_FILES, count, -1);
		ok = gtk_tree_model_iter_next(data->model, &iter);
		files += count;
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(data->view), data->fmodel);

	PrintFiles(aux, files);
	g_snprintf(buf, sizeof(buf), "All files (%s)", aux);
	gtk_button_set_label(GTK_BUTTON(data->check), buf);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->check), data->allfiles);
	gtk_widget_set_sensitive(data->view, !data->allfiles);

	UpdateTotal(data);
}

void CAddFilterDlg::UpdateExposurePage(void)
{
	bool ok;
	int count, files;
	double exptime;
	char aux[64], buf[256];
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 1);
	GtkTreeIter iter;

	gtk_tree_view_set_model(GTK_TREE_VIEW(data->view), NULL);
	// For each filter, compute number of files
	files = 0;
	ok = gtk_tree_model_get_iter_first(data->model, &iter);
	while (ok) {
		count = 0;
		gtk_tree_model_get(data->model, &iter, DATA_VALUE, &exptime, -1);
		for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
			if (IsFilterSelected(((tFileData*)ptr->data)->filter)) {
				if (exptime == ((tFileData*)ptr->data)->exptime)
					count++;
			}
		}
		gtk_list_store_set(GTK_LIST_STORE(data->model), &iter, DATA_FILES, count, -1);
		ok = gtk_tree_model_iter_next(data->model, &iter);
		files += count;
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(data->view), data->fmodel);

	PrintFiles(aux, files);
	g_snprintf(buf, sizeof(buf), "All files (%s)", aux);
	gtk_button_set_label(GTK_BUTTON(data->check), buf);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->check), data->allfiles);
	gtk_widget_set_sensitive(data->view, !data->allfiles);

	UpdateTotal(data);
}

void CAddFilterDlg::UpdateObjectPage(void)
{
	bool ok;
	int count, files;
	char *object, aux[64], buf[256];
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 2);
	GtkTreeIter iter;

	gtk_tree_view_set_model(GTK_TREE_VIEW(data->view), NULL);
	// For each filter, compute number of files
	files = 0;
	ok = gtk_tree_model_get_iter_first(data->model, &iter);
	while (ok) {
		count = 0;
		gtk_tree_model_get(data->model, &iter, DATA_VALUE, &object, -1);
		for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
			if (IsFilterSelected(((tFileData*)ptr->data)->filter) &&
				IsExposureSelected(((tFileData*)ptr->data)->exptime)) {
				if (StrCmp0(object, ((tFileData*)ptr->data)->object)==0)
					count++;
			}
		}
		g_free(object);
		gtk_list_store_set(GTK_LIST_STORE(data->model), &iter, DATA_FILES, count, -1);
		ok = gtk_tree_model_iter_next(data->model, &iter);
		files += count;
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(data->view), data->fmodel);

	PrintFiles(aux, files);
	g_snprintf(buf, sizeof(buf), "All files (%s)", aux);
	gtk_button_set_label(GTK_BUTTON(data->check), buf);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->check), data->allfiles);
	gtk_widget_set_sensitive(data->view, !data->allfiles);

	UpdateTotal(data);
}

void CAddFilterDlg::ProcessFiles()
{
	int count;
	char buf[256];

	count = 0;
	for (GList *ptr=m_Files; ptr!=NULL; ptr=ptr->next) {
		if (IsFilterSelected(((tFileData*)ptr->data)->filter) &&
			IsExposureSelected(((tFileData*)ptr->data)->exptime) &&
			IsObjectSelected(((tFileData*)ptr->data)->object)) {
				if (Global.pProject->AddFile(((tFileData*)ptr->data)->path))
					count++;
		}
	}

	if (count==0)
		gtk_label_set_text(GTK_LABEL(m_pLast), "No file was added to the project.");
	else if (count==1)
		gtk_label_set_text(GTK_LABEL(m_pLast), "1 file was added to the project.");
	else {
		sprintf(buf, "%d files were added to the project.", count);
		gtk_label_set_text(GTK_LABEL(m_pLast), buf);
	}
}

void CAddFilterDlg::GoToFirstPage(void)
{
	tPageData *data;

	m_FirstPage = 0;
	UpdateFilterPage();
	if (FilterItemCount()>1) {
		GoToPage(0);
		return;
	}
	data = (tPageData*)g_slist_nth_data(m_Pages, 0);
	data->skip = true;
	m_FirstPage = 1;
	UpdateExposurePage();
	if (ExposureItemCount()>1) {
		GoToPage(1);
		return;
	}
	data = (tPageData*)g_slist_nth_data(m_Pages, 1);
	data->skip = true;
	m_FirstPage = 2;
	UpdateObjectPage();
	if (ObjectItemCount()>1) {
		GoToPage(2);
		return;
	}
	data = (tPageData*)g_slist_nth_data(m_Pages, 2);
	data->skip = true;
	m_FirstPage = 3;
	ProcessFiles();
	GoToPage(3);
}

void CAddFilterDlg::GoToForward(void)
{
	tPageData *data;

	int page = gtk_notebook_get_current_page(GTK_NOTEBOOK(m_pTabs));
	switch (page)
	{
	case 0:
		// Go to 'exposure' page
		UpdateExposurePage();
		if (ExposureItemCount()>1) {
			GoToPage(1);
			break;
		}
		data = (tPageData*)g_slist_nth_data(m_Pages, 1);
		data->skip = true;
		// fallthrough
	case 1:
		// Go to 'object' page
		UpdateObjectPage();
		if (ObjectItemCount()>1) {
			GoToPage(2);
			break;
		}
		data = (tPageData*)g_slist_nth_data(m_Pages, 2);
		data->skip = true;
		// fallthrough
	default:
		// Add files and go to 'summary' page
		ProcessFiles();
		GoToPage(3);
	}
}

void CAddFilterDlg::GoToBack()
{
	tPageData *data;

	int page = gtk_notebook_get_current_page(GTK_NOTEBOOK(m_pTabs));
	switch (page)
	{
	case 2:
		// Go to 'exposure' page
		data = (tPageData*)g_slist_nth_data(m_Pages, 1);
		if (!data->skip) {
			GoToPage(1);
			return;
		}
		// falltrough
	case 1:
		// Go to 'filter' page
		data = (tPageData*)g_slist_nth_data(m_Pages, 0);
		if (!data->skip) { 
			GoToPage(0);
			return;
		}
	default:
		// Do nothing
		break;
	}
}

void CAddFilterDlg::GoToPage(int page)
{
	switch (page)
	{
	case 0:
		// Go to 'filter' page
		gtk_widget_hide(m_pBack);
		gtk_widget_hide(m_pClose);
		gtk_widget_show(m_pCancel);
		gtk_widget_show(m_pNext);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(m_pTabs), 0);
		break;

	case 1:
		// Go to 'exposure' page
		if (m_FirstPage<1)
			gtk_widget_show(m_pBack);
		else
			gtk_widget_hide(m_pBack);
		gtk_widget_hide(m_pClose);
		gtk_widget_show(m_pCancel);
		gtk_widget_show(m_pNext);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(m_pTabs), 1);
		break;

	case 2:
		// Go to 'object' page
		if (m_FirstPage<2)
			gtk_widget_show(m_pBack);
		else
			gtk_widget_hide(m_pBack);
		gtk_widget_hide(m_pClose);
		gtk_widget_show(m_pCancel);
		gtk_widget_show(m_pNext);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(m_pTabs), 2);
		break;

	case 3:
		// Add files and go to 'summary' page
		gtk_widget_hide(m_pBack);
		gtk_widget_show(m_pClose);
		gtk_widget_hide(m_pCancel);
		gtk_widget_hide(m_pNext);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(m_pTabs), 3);
	}
}

bool CAddFilterDlg::IsFilterSelected(const char *filter)
{
	gboolean ok, checked, retval = true;
	char *str;
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 0);
	GtkTreeIter iter;

	if (!data->allfiles && !data->skip) {
		ok = gtk_tree_model_get_iter_first(data->model, &iter);
		while (ok) {
			gtk_tree_model_get(data->model, &iter, DATA_VALUE, &str, DATA_CHECK, &checked, -1);
			if (StrCmp0(str, filter)==0) {
				retval = checked;
				break;
			}
			g_free(str);
			ok = gtk_tree_model_iter_next(data->model, &iter);
		}
	}
	return retval;
}

int CAddFilterDlg::FilterItemCount()
{
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 0);
	return gtk_tree_model_iter_n_children(data->fmodel, NULL);
}

bool CAddFilterDlg::IsExposureSelected(double exptime)
{
	gboolean ok, checked, retval = true;
	double value;
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 1);
	GtkTreeIter iter;

	if (!data->allfiles && !data->skip) {
		ok = gtk_tree_model_get_iter_first(data->model, &iter);
		while (ok) {
			gtk_tree_model_get(data->model, &iter, DATA_VALUE, &value, DATA_CHECK, &checked, -1);
			if (value == exptime) {
				retval = checked;
				break;
			}
			ok = gtk_tree_model_iter_next(data->model, &iter);
		}
	}
	return retval;
}

int CAddFilterDlg::ExposureItemCount()
{
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 1);
	return gtk_tree_model_iter_n_children(data->fmodel, NULL);
}

bool CAddFilterDlg::IsObjectSelected(const char *object)
{
	gboolean ok, checked, retval = true;
	char *str;
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 2);
	GtkTreeIter iter;

	if (!data->allfiles && !data->skip) {
		ok = gtk_tree_model_get_iter_first(data->model, &iter);
		while (ok) {
			gtk_tree_model_get(data->model, &iter, DATA_VALUE, &str, DATA_CHECK, &checked, -1);
			if (StrCmp0(str, object)==0) {
				retval = checked;
				break;
			}
			g_free(str);
			ok = gtk_tree_model_iter_next(data->model, &iter);
		}
	}
	return retval;
}

int CAddFilterDlg::ObjectItemCount()
{
	tPageData *data = (tPageData*)g_slist_nth_data(m_Pages, 2);
	return gtk_tree_model_iter_n_children(data->fmodel, NULL);
}
