/**************************************************************

amasslist_dlg.cpp (C-Munipack project)
The 'Plot deviations' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "project.h"
#include "configuration.h"
#include "graph.h"
#include "progress_dlg.h"
#include "amasslist_dlg.h"
#include "main.h"
#include "utils.h"
#include "proc_classes.h"

//-------------------------   LIST COLUMNS   --------------------------------

struct tTreeViewColumn {
	const char *caption;		// Column name
	int column;					// Model column index
	GtkTreeCellDataFunc datafn;	// Data function
	gfloat align;				// Text alignment
};

enum tColumnTypeId {
	DATA_AIRMASS,				// Airmass coefficient
	DATA_ALTITUDE				// Altitude in degrees
};

enum tDataColumnId {
	DATA_COLUMN,				// Column ordinal number
	DATA_NAME,					// Column name
	DATA_TYPE,					// Column type
	DATA_MIN,					// Minimum value
	DATA_MAX,					// Maximum value
	DATA_NCOLS					// Number of columns
};

const static tTreeViewColumn DataListColumns[] = {
	{ "Data",			DATA_NAME,	NULL,	0 },
	{ NULL }
};

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CAirMassListDlg::CAirMassListDlg(GtkWindow *pParent):m_Column(0)
{
	int i;
	GtkWidget *frame, *hbox, *vbox, *scrwnd;

	// Create table context
	m_Table = cmpack_tab_init(CMPACK_TABLE_APERTURES);
	m_AirMass = cmpack_amass_init();

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Plot air-mass coefficients", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_SAVE, GTK_RESPONSE_NONE, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("airmass");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	hbox = gtk_hbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), hbox, TRUE, TRUE, 0);
	vbox = gtk_vbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);

	// List of data columns
	m_DataList = gtk_list_store_new(DATA_NCOLS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, 
		G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	m_DataView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(m_DataList));
	for (i=0; DataListColumns[i].caption!=NULL; i++) {
		GtkTreeViewColumn *col = gtk_tree_view_column_new();
		// Set column name and alignment
		gtk_tree_view_column_set_title(col, DataListColumns[i].caption);
		gtk_tree_view_append_column(GTK_TREE_VIEW(m_DataView), col);
		// Add text renderer
		GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(col, renderer, TRUE);
		g_object_set(renderer, "xalign", DataListColumns[i].align, NULL);
		if (DataListColumns[i].datafn)
			gtk_tree_view_column_set_cell_data_func(col, renderer, DataListColumns[i].datafn, NULL, NULL);
		else
			gtk_tree_view_column_add_attribute(col, renderer, "text", DataListColumns[i].column);
	}
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(m_DataView)), GTK_SELECTION_SINGLE);
	gtk_widget_set_size_request(m_DataView, 140, 0);

	// Register callback for selection change
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_DataView));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selection_changed), this);

	// Make scrolled window
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_DataView);
	gtk_box_pack_start(GTK_BOX(vbox), scrwnd, TRUE, TRUE, 0);

	// Graph
	frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox), frame, true, true, 0);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Graph.Handle())));
	gtk_widget_set_size_request(m_Graph.Handle(), 640, 480);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_Graph.SetData(&m_Data);
	m_Graph.SetGraphPadding(2, 2, 48, 2);
	m_Graph.EnableTracking(false);

	gtk_widget_show_all(hbox);

	// Signals:
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
}

CAirMassListDlg::~CAirMassListDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_object_unref(m_DataList);
	cmpack_unref(m_Table);
	cmpack_unref(m_AirMass);
}

void CAirMassListDlg::Execute()
{
	bool first;
	int i, index, j, res, count, columns, row_count, channels;
	double jd, jdmin, jdmax, *yvals, y, ymin, ymax;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();
	GtkTreeIter iter;
	CAirMassProc airmass;

	m_Data.Clear();
	cmpack_tab_clear(m_Table);

	// Init the table
	res = airmass.Open(m_Table);
	if (res!=0) {
		char *msg = cmpack_formaterror(res);
		ShowError(pMainWnd, msg);
		cmpack_free(msg);
		return;
	}

	// Make list of input files
	gtk_tree_model_foreach(Global.pProject->FileList(), (GtkTreeModelForeachFunc)foreach_all_files, &airmass);
	if (cmpack_tab_nrows(m_Table)==0) {
		ShowError(pMainWnd, "There are no valid frames in the list.");
		return;
	}

	// Finish the table
	airmass.Close();

	// Copy data from table to graph
	channels = cmpack_tab_ncolumns(m_Table)-1;
	row_count = cmpack_tab_nrows(m_Table);
	m_Data.LockRedraw();
	m_Data.Clear();
	m_Data.Init(channels, row_count);
	if (channels>0 && row_count>0) {
		yvals = (double*)g_malloc(channels*sizeof(double));
		for (i=0; i<row_count; i++) {
			cmpack_tab_gtdd(m_Table, i, 0, &jd);
			for (j=0; j<channels; j++) 
				cmpack_tab_gtdd(m_Table, i, j+1, yvals+j);
			m_Data.Append(jd, yvals);
		}
	}
	m_Data.UnlockRedraw();

	CmpackTabColumn *col = cmpack_tab_get_column(m_Table, 0);
	if (col) {
		jdmin = jdmax = 0;
		first = true;
		count = m_Data.Count(); 
		for (j=0; j<count; j++) {
			jd = m_Data.GetX(j);
			if (jd>0.0) {
				if (first) {
					jdmin = jdmax = jd;
					first = false;
				} else {
					if (jd < jdmin)
						jdmin = jd;
					if (jd > jdmax)
						jdmax = jd;
				}
			}
		}
		m_Graph.SetXParams(cmpack_tab_column_get_name(col), jdmin, jdmax, 
			1.0/144.0, 0, 2);
		cmpack_unref(col);
	}

	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), NULL);
	gtk_list_store_clear(m_DataList);
	columns = 0;
	for (j=0; j<channels; j++) {
		CmpackTabColumn *col = cmpack_tab_get_column(m_Table, j+1);
		if (col) {
			const char *name = cmpack_tab_column_get_name(col);
			if (strstr(name, "AIRMASS")) {
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>0.0) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
				}
				index = m_Graph.AddColumn(j, -1, 0);
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_TYPE, DATA_AIRMASS, DATA_MIN, ymin, DATA_MAX, ymax, -1);
			} else
			if (strstr(name, "ALTITUDE")) {
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>=-90.0 && y<=90.0) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
				}
				index = m_Graph.AddColumn(j, -1, 0);
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_TYPE, DATA_ALTITUDE, DATA_MIN, ymin, DATA_MAX, ymax, -1);
			}
			cmpack_unref(col);
		}
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), GTK_TREE_MODEL(m_DataList));

	// Show first column
	SetData(0);
	m_Graph.AutoZoom(true, true);

	gtk_dialog_run(GTK_DIALOG(m_pDlg));
}

gboolean CAirMassListDlg::foreach_all_files(GtkTreeModel *model, GtkTreePath *path, 
										GtkTreeIter *iter, void *data)
{
	((CAirMassProc*)data)->Add(path);
	return FALSE;
}

void CAirMassListDlg::SetData(int column)
{
	tColumnTypeId type;
	gchar *name;
	double ymin, ymax;
	GtkTreeIter iter;

	m_Column = column;
	m_Graph.ShowColumn(-1, false);
	m_Graph.ShowColumn(m_Column, true); 
	if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(m_DataList), &iter, NULL, m_Column)) {
		gtk_tree_model_get(GTK_TREE_MODEL(m_DataList), &iter, DATA_NAME, &name, 
			DATA_TYPE, &type, DATA_MIN, &ymin, DATA_MAX, &ymax, -1);
		switch (type)
		{
		case DATA_AIRMASS:
			m_Graph.SetYParams(name, ymin, ymax, 0.5, 0, 2);
			break;
		case DATA_ALTITUDE:
			m_Graph.SetYParams(name, ymin, ymax, 10.0, 0, 0);
			break;
		}
		g_free(name);
	}
	m_Graph.AutoZoom(false, true);
}

void CAirMassListDlg::selection_changed(GtkTreeSelection *widget, CAirMassListDlg *pMe)
{
	pMe->OnSelectionChanged(widget);
}

void CAirMassListDlg::OnSelectionChanged(GtkTreeSelection *selection)
{
	int id;
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		if (model == GTK_TREE_MODEL(m_DataList)) {
			gtk_tree_model_get(model, &iter, DATA_COLUMN, &id, -1);
			SetData(id);
		}
	}
}

void CAirMassListDlg::response_dialog(GtkDialog *pDlg, gint response_id, CAirMassListDlg *pMe)
{
	if (!pMe->OnCloseQuery(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CAirMassListDlg::OnCloseQuery(gint response_id)
{
	if (response_id == GTK_RESPONSE_NONE) {
		SaveData();
		return false;
	}
	return true;
}

void CAirMassListDlg::col_name_free(char *str, void *data)
{
	g_free(str);
}

void CAirMassListDlg::SaveData(void)
{
	char *filename, *folder, *fpath, *f;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save data to...",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("Deviations", "Name", "output.txt");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
		g_free(filename);
	}

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		filename = g_path_get_basename(fpath);
		CConfig::SetStr("Deviations", "Name", filename);
		g_free(filename);
		f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
		cmpack_tab_save(m_Table, f);
		g_free(f);
		g_free(fpath);
		Global.pProject->Save();
	}
	gtk_widget_destroy(pSaveDlg);
}
