/**************************************************************

chart.cpp (C-Munipack project)
Widget which can draw a chart
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#define _USE_MATH_DEFINES

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <glib.h>
#include <cmunipack.h>

#include "configuration.h"
#include "chart_ex.h"
#include "scale.h"
#include "qphot.h"

//------------------------------   CONSTANTS   -------------------------------------------

#define CURSOR_SIZE		12

#define ZOOM_TABLE_SIZE	26

// Renard numbers (R5) multiplied by power of 10:
const double ZoomTable[ZOOM_TABLE_SIZE] = { 0.0010, 0.0016, 0.0025, 0.0040, 0.0063, 0.010, 
	0.016, 0.025, 0.040, 0.063, 0.10, 0.16, 0.25, 0.40, 0.63, 1.0, 1.6, 2.5, 4.0, 6.3, 
	10.0, 16.0, 25.0, 40.0, 63.0, 100.0 };

//--------------------------   HELPER FUNCTIONS   ----------------------------------

// Zoom factor to nearest zoom index
static int ZoomToIndex(double zoom, int min, int max)
{
	int i, j;
	double q, d;

	if (zoom <= ZoomTable[min])
		return min;
	if (zoom >= ZoomTable[max])
		return max;

	// Find zoom index nearest to the current zoom factor
	q = 1e99; j = min;
	for (i=min; i<=max; i++) {
		d = fabs(log10(ZoomTable[i]/zoom));
		if (q > d) {
			j = i;
			q = d;
		}
	}
	return j;
}

//--------------------------   CHART CLASS   ----------------------------------

// Default constructor
CChartEx::CChartEx():m_ScrWnd(NULL), m_Width(0), m_Height(0), m_ZIndexMin(0), 
	m_ZIndexMax(ZOOM_TABLE_SIZE-1)
{
	m_ScrWnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(m_ScrWnd), GTK_WIDGET(g_object_ref(m_DrawArea)));
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_ScrWnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
}

// Destructor
CChartEx::~CChartEx()
{
	gtk_widget_destroy(m_ScrWnd);
}

// Change zoom factor
void CChartEx::SetZoom(double zoom)
{
	if (zoom!=m_Zoom) {
		m_Zoom = zoom;
		if (m_Zoom < ZoomTable[m_ZIndexMin])
			m_Zoom = ZoomTable[m_ZIndexMin];
		if (m_Zoom > ZoomTable[m_ZIndexMax])
			m_Zoom = ZoomTable[m_ZIndexMax];
		Resize();
		Redraw();
		Callback(CB_ZOOM_CHANGE);
	}
}

// Zoom in
void CChartEx::ZoomIn(void)
{
	int j = ZoomToIndex(m_Zoom, m_ZIndexMin, m_ZIndexMax);
	if (j>=0) {
		if (m_Zoom < ZoomTable[j] || j==m_ZIndexMax)
			SetZoom(ZoomTable[j]);
		else
			SetZoom(ZoomTable[j+1]);
	}
		
}

// Zoom out
void CChartEx::ZoomOut(void)
{
	int j = ZoomToIndex(m_Zoom, m_ZIndexMin, m_ZIndexMax);
	if (j>=0) {
		if (m_Zoom > ZoomTable[j] || j==m_ZIndexMin)
			SetZoom(ZoomTable[j]);
		else
			SetZoom(ZoomTable[j-1]);
	}
}

/* Set size of the drawing area */
void CChartEx::OnImageSizeChanged(void)
{
	int width, height;

	if (m_MapWidth>0 && m_MapHeight>0) {
		m_ZIndexMin = ZoomToIndex(200.0/m_MapWidth, 0, ZOOM_TABLE_SIZE-1);
		m_ZIndexMax = ZoomToIndex(100.0, 0, ZOOM_TABLE_SIZE-1);
		if (m_Zoom < ZoomTable[m_ZIndexMin])
			m_Zoom = ZoomTable[m_ZIndexMin];
		if (m_Zoom > ZoomTable[m_ZIndexMax])
			m_Zoom = ZoomTable[m_ZIndexMax];
	}

	width = (int)(ceil(m_Zoom*m_MapWidth) + 0.5);
	height = (int)(ceil(m_Zoom*m_MapHeight) + 0.5);
	if (width!=m_Width || height!=m_Height) {
		m_Width = width;
		m_Height = height;
		gtk_widget_set_size_request(GTK_WIDGET(m_DrawArea), width, height);
	}
}
