/**************************************************************

matching_dlg.cpp (C-Munipack project)
The 'Match stars' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <cmunipack.h>

#include "project.h"
#include "configuration.h"
#include "matching_dlg.h"
#include "progress_dlg.h"
#include "main.h"
#include "utils.h"
#include "proc_classes.h"

static gboolean make_list(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GList **list)
{
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*list = g_list_append(*list, rowref);
	return FALSE;
}

//-------------------------   LIST COLUMNS   --------------------------------

static void GetFrameID(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int id;
	char buf[64];

	gtk_tree_model_get(tree_model, iter, FRAME_ID, &id, -1);
	g_snprintf(buf, sizeof(buf), "%d", id); 
	g_object_set(cell, "text", buf, NULL);
}

static void GetStars(GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
	GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
{
	int nstars;
	char buf[64];

	gtk_tree_model_get(tree_model, iter, FRAME_STARS, &nstars, -1);

	g_snprintf(buf, sizeof(buf), "%d", nstars);
	g_object_set(cell, "text", buf, NULL);
}

const static struct tTreeViewColumn {
	const char *caption;		// Column name
	int column;					// Model column index
	GtkTreeCellDataFunc datafn;	// Data function
	gfloat align;				// Text alignment
	int col_id;
} TreeViewColumns[] = {
	{ "Frame #",		0,	GetFrameID,	 1, FRAME_ID },
	{ "Stars",			0,	GetStars,	 1, FRAME_STARS },
	{ NULL }
};

struct tColData
{
	GtkTreeViewColumn *col;
	const tTreeViewColumn *data;
};

//-------------------------   MAIN WINDOW   --------------------------------

CMatchingDlg::CMatchingDlg(GtkWindow *pParent):m_FileList(NULL), m_Columns(NULL), m_CatFile(NULL)
{
	int i;
	GSList *group;
	GtkWidget *mainbox, *hbox, *scrwnd, *vbox, *frame;
	GtkFileFilter *filter[2];

	// Initial state
	m_SelectMode = (tSelectMode)CConfig::GetInt("MatchingDlg", "Select", REFERENCE_FRAME);
	m_SortMode = (tSortMode)CConfig::GetInt("MatchingDlg", "Sort", SORT_STARS);

	// List of frames
	m_Files = gtk_tree_model_sort_new_with_model(Global.pProject->FileList());

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Choose reference frame", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("matchstars");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	mainbox = gtk_vbox_new(FALSE, 4);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), mainbox);
	gtk_widget_set_size_request(mainbox, 640, 480);

	// Reference frame or catalogue file?
	frame = gtk_frame_new("Use");
	gtk_box_pack_start(GTK_BOX(mainbox), frame, FALSE, TRUE, 0);
	vbox = gtk_vbox_new(TRUE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	m_RefBtn = gtk_radio_button_new_with_label(NULL, "reference frame");
	gtk_box_pack_start(GTK_BOX(vbox), m_RefBtn, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_RefBtn)); 
	m_CatBtn = gtk_radio_button_new_with_label(group, "catalogue file");
	gtk_box_pack_start(GTK_BOX(vbox), m_CatBtn, TRUE, TRUE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_RefBtn), m_SelectMode==REFERENCE_FRAME);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_CatBtn), m_SelectMode==CATALOGUE_FILE);

	// List of reference frames
	m_RefFiles = gtk_frame_new("Select a reference frame");
	gtk_box_pack_start(GTK_BOX(mainbox), m_RefFiles, TRUE, TRUE, 0);
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
	gtk_container_add(GTK_CONTAINER(m_RefFiles), hbox);
	m_View = gtk_tree_view_new();
	for (i=0; TreeViewColumns[i].caption!=NULL; i++) {
		GtkTreeViewColumn *col = gtk_tree_view_column_new();
		// Set column name and alignment
		gtk_tree_view_column_set_title(col, TreeViewColumns[i].caption);
		gtk_tree_view_append_column(GTK_TREE_VIEW(m_View), col);
		// Add text renderer
		GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(col, renderer, TRUE);
		g_object_set(renderer, "xalign", TreeViewColumns[i].align, NULL);
		if (TreeViewColumns[i].datafn)
			gtk_tree_view_column_set_cell_data_func(col, renderer, TreeViewColumns[i].datafn, NULL, NULL);
		else
			gtk_tree_view_column_add_attribute(col, renderer, "text", TreeViewColumns[i].column);
		g_signal_connect(G_OBJECT(col), "clicked", G_CALLBACK(column_clicked), this);
		tColData *data = (tColData*)g_malloc(sizeof(tColData));
		data->col = col;
		data->data = &TreeViewColumns[i];
		m_Columns = g_slist_append(m_Columns, data);
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_View), GTK_TREE_MODEL(m_Files));
	gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(m_View), true);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(m_View)), GTK_SELECTION_SINGLE);
	gtk_widget_set_size_request(m_View, 100, 320);
	scrwnd = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_View);
	gtk_box_pack_start(GTK_BOX(hbox), scrwnd, FALSE, TRUE, 0);

	// Reference frame preview
	frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox), frame, true, true, 0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Chart.Handle())));
	m_Chart.EnableTracking(false);
	m_Chart.HighlightNonMatchedStars(false);
	m_Chart.ShowNonMatchedStars(true);

	// Catalogue file selection
	m_CatFiles = gtk_frame_new("Select a catalogue file");
	gtk_box_pack_start(GTK_BOX(mainbox), m_CatFiles, TRUE, TRUE, 0);
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
	gtk_container_add(GTK_CONTAINER(m_CatFiles), hbox);
	m_FileChooser = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_box_pack_start(GTK_BOX(hbox), m_FileChooser, TRUE, TRUE, 0);
	filter[0] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter[0], "*.xml");
	gtk_file_filter_set_name(filter[0], "Catalogue files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_FileChooser), filter[0]);
	filter[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter[1], "*.*");
	gtk_file_filter_set_name(filter[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_FileChooser), filter[1]);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_FileChooser), filter[0]);

	// Catalogue file preview
	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(m_FileChooser), 
		GTK_WIDGET(g_object_ref(m_Preview.Handle())));
	m_Preview.SetTypeMask(FILE_TYPE_CAT);
	m_Preview.SetParent(GTK_WINDOW(m_pDlg));

	// Signals
	g_signal_connect(G_OBJECT(m_RefBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_CatBtn), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_FileChooser), "update-preview", G_CALLBACK(update_preview), this);

	// Register callback for selection change
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_View));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selection_changed), this);

	gtk_widget_show_all(mainbox);
	SetSelectMode(m_SelectMode);
	SetSortMode(m_SortMode);
}

CMatchingDlg::~CMatchingDlg()
{
	for (GSList *ptr=m_Columns; ptr!=NULL; ptr=ptr->next)
		g_free(ptr->data);
	g_slist_free(m_Columns);
	g_list_free(m_FileList);
	g_object_unref(m_Files);
	gtk_widget_destroy(m_pDlg);
	g_free(m_CatFile);
}

void CMatchingDlg::Execute()
{
	int res, frame;
	char msg[256];
	const char *file;
	gchar *path;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();
	GtkTreeIter iter;
	GtkTreeSelection *sel;
	GtkTreePath *path1, *path2;

	m_FrameId = 0;
	g_free(m_CatFile);
	m_CatFile = NULL;
	m_FileList = NULL;

	// Check inputs
	if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(m_Files), NULL)==0) {
		ShowError(pMainWnd, "There are no frames usable as a reference frame.");
		return;
	}

	// Select last reference frame or the first frame
	Global.pProject->GetReference(&frame, &file);
	if (frame>0) {
		// Reference frame
		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_View));
		path1 = Global.pProject->GetFilePath(frame);
		path2 = gtk_tree_model_sort_convert_child_path_to_path(GTK_TREE_MODEL_SORT(m_Files), path1);
		gtk_tree_selection_select_path(sel, path2);
		gtk_tree_path_free(path2);
		gtk_tree_path_free(path1);
	} else {
		// Default is the first row in the table
		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_View));
		if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(m_Files), &iter))
			gtk_tree_selection_select_iter(sel, &iter);
	}
	if (file) {
		// Catalogue file used
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(m_FileChooser), file);
		m_Preview.Update(file);
	} else {
		// Default is the last used catalogue file
		path = CConfig::GetStr("MatchingDlg", "File", NULL);
		if (path) {
			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(m_FileChooser), path);
			m_Preview.Update(path);
			g_free(path);
		}
	}
	gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(m_FileChooser), true);

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return;
	gtk_widget_hide(m_pDlg);

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_RefBtn))) {
		// Reference frame
		CConfig::SetInt("MatchingDlg", "Select", REFERENCE_FRAME);
		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_View));
		GList *rows = gtk_tree_selection_get_selected_rows(sel, NULL);
		if (rows) {
			GtkTreePath *path = (GtkTreePath*)g_list_first(rows)->data;
			gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Files), &iter, path);
			gtk_tree_model_get(GTK_TREE_MODEL(m_Files), &iter, FRAME_ID, &m_FrameId, -1);
			g_list_foreach(rows, GFunc(gtk_tree_path_free), NULL);
			g_list_free(rows);
		}
	} else {
		// Catalogue file
		CConfig::SetInt("MatchingDlg", "Select", CATALOGUE_FILE);
		m_CatFile = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_FileChooser));
		if (m_CatFile)
			CConfig::SetStr("MatchingDlg", "File", m_CatFile);
	}

	// Always all files
	GtkTreeModel *pList = Global.pProject->FileList();
	gtk_tree_model_foreach(pList, GtkTreeModelForeachFunc(make_list), &m_FileList);
	if ((m_FrameId>0 || m_CatFile) && m_FileList) {
		CProgressDlg pDlg(Global.pMainWnd->GetHandle(), "Matching photometry files...");
		pDlg.SetMinMax(0, g_list_length(m_FileList));
		res = pDlg.Execute(ExecuteProc, this);
		if (res!=0) {
			char *msg = cmpack_formaterror(res);
			ShowError(pMainWnd, msg);
			cmpack_free(msg);
		} else if (m_OutFiles==0) {
			ShowError(pMainWnd, "No file was successfully processed.");
		} else if (m_OutFiles!=m_InFiles) {
			sprintf(msg, "%d file(s) was successfully processed, %d file(s) failed.", 
				m_OutFiles, m_InFiles-m_OutFiles);
			ShowWarning(pMainWnd, msg);
		} else {
			sprintf(msg, "All %d file(s) was successfully processed.", m_OutFiles);
			ShowInformation(pMainWnd, msg);
		}
	}

	// Free allocated memory
	g_list_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
	g_list_free(m_FileList);
	m_FileList = NULL;
	g_free(m_CatFile);
	m_CatFile = NULL;
}

int CMatchingDlg::ExecuteProc(CProgressDlg *sender, void *userdata)
{
	return ((CMatchingDlg*)userdata)->ProcessFiles(sender);
}

int CMatchingDlg::ProcessFiles(CProgressDlg *sender)
{
	int frameid, res;
	gchar *phot, msg[128];
	GtkTreePath *path;
	CMatchingProc match;

	m_InFiles = m_OutFiles = 0;
	Global.pProject->ClearReference();
	sender->Print("------ Matching ------");

	res = match.Init(sender, m_FrameId, m_CatFile);
	if (res==0) {
		// Set reference frame
		Global.pProject->SetReference(m_FrameId, m_CatFile);
		const CSelection *sel = match.Selection();
		if (m_CatFile)
			Global.pProject->SetCatFileSelection(sel);
		Global.pProject->SetSelection(sel);
		// Set object's coordinates
		const CObjectCoords *obj = match.Object();
		if (obj) {
			Global.pProject->SetReferenceObjectCoords(obj);
			Global.pProject->SetObjectCoords(obj);
		}
		// Set observer's coordinates
		const CObserverCoords *obs = match.Location();
		if (obs) {
			Global.pProject->SetReferenceLocation(obs);
			Global.pProject->SetUserLocation(obs);
		}
		// Process files
		for (GList *node = m_FileList; node != NULL; node = node->next) {
			path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
			frameid = Global.pProject->GetFrameID(path);
			phot = Global.pProject->GetPhotFileName(path);
			sender->SetFileName(phot);
			sprintf(msg, "Frame #%d:", frameid);
			sender->Print(msg);
			g_free(phot);
			sender->SetProgress(m_InFiles++);
			if (match.Execute(path)==0)
				m_OutFiles++;
			gtk_tree_path_free(path);
			if (sender->Cancelled()) 
				break;
		}
		if (sender->Cancelled())
			sprintf(msg, "Cancelled at the user's request");
		else
			sprintf(msg, "====== %d succeeded, %d failed ======", m_OutFiles, m_InFiles-m_OutFiles);
		sender->Print(msg);
	} else {
		char *aux = cmpack_formaterror(res);
		sender->Print(msg);
		cmpack_free(aux);
	}
	return res;
}

void CMatchingDlg::selection_changed(GtkTreeSelection *widget, CMatchingDlg *pMe)
{
	pMe->SelectionChanged(widget);
}

void CMatchingDlg::SelectionChanged(GtkTreeSelection *selection)
{
	int frame_id;
	char *file;
	GtkTreeModel *model;
	GtkTreeIter iter;

	m_Chart.Clear();
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, FRAME_ID, &frame_id, -1);
		file = Global.pProject->GetImageFile(frame_id);
		if (file)
			m_Chart.LoadImage(file);
		g_free(file);
		file = Global.pProject->GetPhotFile(frame_id);
		if (file)
			m_Chart.LoadChart(file, true);
		g_free(file);
	}
}

void CMatchingDlg::button_clicked(GtkWidget *button, CMatchingDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CMatchingDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==m_RefBtn)
		SetSelectMode(REFERENCE_FRAME);
	else if (pBtn==m_CatBtn)
		SetSelectMode(CATALOGUE_FILE);
}

void CMatchingDlg::SetSelectMode(tSelectMode mode)
{
	m_SelectMode = mode;
	switch (mode)
	{
	case REFERENCE_FRAME:
		gtk_widget_hide(m_CatFiles);
		gtk_widget_show(m_RefFiles);
		break;
	case CATALOGUE_FILE:
		gtk_widget_hide(m_RefFiles);
		gtk_widget_show(m_CatFiles);
		break;
	}
}

void CMatchingDlg::SetSortMode(tSortMode mode)
{
	m_SortMode = mode;
	CConfig::SetInt("MatchingDlg", "Sort", m_SortMode);

	for (GSList *ptr=m_Columns; ptr!=NULL; ptr=ptr->next) {
		tColData *col = (tColData*)ptr->data;
		if ((col->data->col_id==FRAME_ID && mode==SORT_ID) || (col->data->col_id==FRAME_STARS && mode==SORT_STARS)) {
			gtk_tree_view_column_set_sort_indicator(col->col, true);
			if (col->data->col_id == FRAME_ID) {
				gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(m_Files), FRAME_ID, GTK_SORT_ASCENDING);
				gtk_tree_view_column_set_sort_order(col->col, GTK_SORT_ASCENDING);
			} else {
				gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(m_Files), FRAME_STARS, GTK_SORT_DESCENDING);
				gtk_tree_view_column_set_sort_order(col->col, GTK_SORT_DESCENDING);
			}
		} else {
			gtk_tree_view_column_set_sort_indicator(col->col, false);
		}
	}
}

void CMatchingDlg::column_clicked(GtkTreeViewColumn *treeviewcolumn, CMatchingDlg *pDlg)
{
	pDlg->OnColumnClicked(treeviewcolumn);
}

void CMatchingDlg::OnColumnClicked(GtkTreeViewColumn *pCol)
{
	GSList *ptr;

	for (ptr=m_Columns; ptr!=NULL; ptr=ptr->next) {
		tColData *col = (tColData*)ptr->data;
		if (col->col == pCol) {
			if (col->data->col_id == FRAME_ID)
				SetSortMode(SORT_ID);
			if (col->data->col_id == FRAME_STARS)
				SetSortMode(SORT_STARS);
			break;
		}
	}
}

void CMatchingDlg::update_preview(GtkFileChooser *pChooser, CMatchingDlg *pMe)
{
	pMe->OnUpdatePreview(pChooser);
}

void CMatchingDlg::OnUpdatePreview(GtkFileChooser *pChooser)
{
	gchar *file = gtk_file_chooser_get_preview_filename(pChooser);
	if (file) {
		m_Preview.Update(file);
		g_free(file);
	} else {
		m_Preview.Clear();
	}
}
