/**************************************************************

proc_classes.h (C-Munipack project)
Process helper classes
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_PROC_CLASSES_H
#define MUNIWIN_PROC_CLASSES_H

#include <gtk/gtk.h>
#include <cmunipack.h>

#include "progress_dlg.h"
#include "helper_classes.h"

//
// Frame conversion context
//
class CConvertProc
{
public:
	// Constructor
	CConvertProc();

	// Destructor
	virtual ~CConvertProc();

	// Initialization
	int Init(CProgressDlg *pProgress);

	// Convert file
	int Execute(GtkTreePath *pPath);

private:
	CProgressDlg	*m_Progress;
	CmpackKonv		*m_Konv;
};


//
// Time correction context
//
class CTimeCorrProc
{
public:
	// Constructor
	CTimeCorrProc();

	// Destructor
	virtual ~CTimeCorrProc();

	// Initialization
	int Init(CProgressDlg *pProgress, double seconds);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	double			m_Seconds;
	CProgressDlg	*m_Progress;
	CmpackTimeCorr	*m_Time;
};


//
// Bias correction context
//
class CBiasCorrProc
{
public:
	// Constructor
	CBiasCorrProc();

	// Destructor
	virtual ~CBiasCorrProc();

	// Initialization
	int Init(CProgressDlg *pProgress, const gchar *bias);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CmpackBiasCorr	*m_Bias;
	CProgressDlg	*m_Progress;

	// Convert bias frame
	int ConvertBiasFrame(CProgressDlg *pProgress, const gchar *orig_filepath, const gchar *temp_filename);
};


//
// Dark correction context
//
class CDarkCorrProc
{
public:
	// Constructor
	CDarkCorrProc();

	// Destructor
	virtual ~CDarkCorrProc();

	// Initialization
	int Init(CProgressDlg *pProgress, const gchar *dark);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CProgressDlg	*m_Progress;
	CmpackDarkCorr	*m_Dark;

	// Convert bias frame
	int ConvertDarkFrame(CProgressDlg *pProgress, const gchar *orig_filepath, const gchar *temp_filename);
};


//
// Flat correction context
//
class CFlatCorrProc
{
public:
	// Constructor
	CFlatCorrProc();

	// Destructor
	virtual ~CFlatCorrProc();

	// Initialization
	int Init(CProgressDlg *pProgress, const gchar *flat);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CProgressDlg	*m_Progress;
	CmpackFlatCorr	*m_Flat;

	// Convert bias frame
	int ConvertFlatFrame(CProgressDlg *pProgress, const gchar *orig_filepath, const gchar *temp_filename);
};


//
// Photometry context
//
class CPhotometryProc
{
public:
	// Constructor
	CPhotometryProc();

	// Destructor
	virtual ~CPhotometryProc();

	// Initialization
	int Init(CProgressDlg *pProgress);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CProgressDlg	*m_Progress;
	CmpackPhot		*m_Phot;
};


//
// Matching context
//
class CMatchingProc
{
public:
	// Constructor
	CMatchingProc();

	// Destructor
	virtual ~CMatchingProc();

	// Initialization
	int Init(CProgressDlg *pProgress, int frame_id, const gchar *cat_file);

	// Match file
	int Execute(GtkTreePath *pPath);

	// Get selection 
	const CSelection *Selection(void) const
	{ return &m_Selection; }

	// Object coordinates
	const CObjectCoords *Object(void) const
	{ return &m_Object; }

	// Observer coordinates
	const CObserverCoords *Location(void) const
	{ return &m_Location; }

private:
	CmpackMatch		*m_Match;
	CSelection		m_Selection;
	CProgressDlg	*m_Progress;
	CObjectCoords	m_Object;
	CObserverCoords	m_Location;

	// Copy catalogue photometry and image file
	void ConvertCatFile(const gchar *orig_file, const gchar *temp_file);
};


//
// Master bias context
//
class CMasterBiasProc
{
public:
	// Constructor
	CMasterBiasProc();

	// Destructor
	virtual ~CMasterBiasProc();

	// Initialization
	int Open(CProgressDlg *pProgress, const gchar *out_file);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CProgressDlg		*m_Progress;
	CmpackMasterBias	*m_MBias;
};


//
// Master dark context
//
class CMasterDarkProc
{
public:
	// Constructor
	CMasterDarkProc();

	// Destructor
	virtual ~CMasterDarkProc();

	// Initialization
	int Open(CProgressDlg *pProgress, const gchar *out_file);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CProgressDlg		*m_Progress;
	CmpackMasterDark	*m_MDark;
};


//
// Master flat context
//
class CMasterFlatProc
{
public:
	// Constructor
	CMasterFlatProc();

	// Destructor
	virtual ~CMasterFlatProc();

	// Initialization
	int Open(CProgressDlg *pProgress, const gchar *out_file);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CProgressDlg		*m_Progress;
	CmpackMasterFlat	*m_MFlat;
};


//
// Listings
//
class CListProc
{
public:
	// Constructor
	CListProc();

	// Destructor
	virtual ~CListProc();

	// Initialization
	virtual int Open(CProgressDlg *pProgress, CmpackTable *pTable, CmpackListFormat format);

	// Add file
	virtual int Add(GtkTreePath *pPath);

	// Finalization
	virtual int Close(void);

private:
	bool				m_DoHelCor, m_AddHelCor;
	bool				m_AddAirMass, m_AddAltitude;
	CProgressDlg		*m_Progress;
	CmpackTable			*m_Table;
	CmpackLister		*m_List;
	CmpackHelCorr		*m_HelCor;
	CmpackAirMass		*m_AirMass;
};

//
// Air-mass coefficient table
//
class CAirMassProc
{
public:
	// Constructor
	CAirMassProc();

	// Destructor
	virtual ~CAirMassProc();

	// Initialization
	virtual int Open(CmpackTable *pTable);

	// Add file
	virtual int Add(GtkTreePath *pPath);

	// Finalization
	virtual void Close(void);

private:
	CmpackTable			*m_Table;
	CmpackAirMass		*m_AirMass;
};

#endif
