/**************************************************************

project.cpp (C-Munipack project)
Table of input frames
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <glib.h>
#include <cmunipack.h>
#include <glib/gstdio.h>

#include "project.h"
#include "utils.h"
#include "configuration.h"
#include "main_dlg.h"
#include "main.h"

#define ICON_SIZE 128

//-------------------------------   PRIVATE DATA TYPES   -----------------------------------

struct tGetFileInfo
{
	int			id;
	GtkTreePath *path;
};

//-------------------------------   HELPER FUNCTIONS   -----------------------------------

static void foreach_func(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer userdata)
{
	GList **list = ((GList**)userdata);
	GtkTreeRowReference *rowref = gtk_tree_row_reference_new(model, path);
	*list = g_list_append(*list, rowref);
}

//-------------------------------   PROJECT CLASS   -----------------------------------

CProject::CProject(void):m_FreeID(1), m_RefID(0), m_Aperture(1), m_TimeCorr(0.0)
{
	m_Frames = gtk_list_store_new(FRAME_NCOLS, G_TYPE_UINT, G_TYPE_INT,
		G_TYPE_DOUBLE, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_DOUBLE, 
		G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, 
		G_TYPE_STRING, GDK_TYPE_PIXBUF);
}

CProject::~CProject(void)
{
	g_object_unref(m_Frames);
}


// Get project's file name
const char *CProject::GetFileName(void)
{
	return "project.ini";
}

// Get project's home path
const char *CProject::GetDirPath(void)
{
	return get_temp_dir();
}

// Clear list of selected stars
void CProject::SetSelection(const CSelection *sel)
{
	if (sel) 
		m_Selection = *sel;
	else
		m_Selection.Clear();
}

// Clear list of selected stars
void CProject::SetCatFileSelection(const CSelection *sel)
{
	if (sel) 
		m_CatFileSelection = *sel;
	else
		m_CatFileSelection.Clear();
}

// Clear list of selected stars
void CProject::SetVarFindSelection(const CSelection *sel)
{
	if (sel) 
		m_VarFindSelection = *sel;
	else
		m_VarFindSelection.Clear();
}

// Set obejct's coordinates
void CProject::SetObjectCoords(const CObjectCoords *obj)
{
	if (obj) 
		m_ObjectCoords = *obj;
	else
		m_ObjectCoords.Clear();
}

// Set obejct's coordinates
void CProject::SetReferenceObjectCoords(const CObjectCoords *obj)
{
	if (obj) 
		m_RefObjectCoords = *obj;
	else
		m_RefObjectCoords.Clear();
}

// Set user observer's coordinates
void CProject::SetUserLocation(const CObserverCoords *obj)
{
	if (obj) 
		m_UserLocation = *obj;
	else
		m_UserLocation.Clear();
}

// Set reference observer's coordinates
void CProject::SetReferenceLocation(const CObserverCoords *obj)
{
	if (obj) 
		m_RefLocation = *obj;
	else
		m_RefLocation.Clear();
}

// Load project file
void CProject::Load(void)
{
	int i, count, state, id, nstars, mstars, frames;
	double juldat, exptime, ccdtemp, tcorr;
	char *str, *cfg_path, file[64], buf[128], *ftemp, *fpath;
	char *filter, *result, *fphot, *fbias, *fdark, *fflat;
	GKeyFile *cfg;
	GdkPixbuf *icon;
	GtkTreeIter iter;
	GError *err = NULL;

	gtk_list_store_clear(m_Frames);
	m_Selection.Clear();
	m_CatFileSelection.Clear();
	m_VarFindSelection.Clear();
	m_ObjectCoords.Clear();
	m_RefObjectCoords.Clear();
	m_UserLocation.Clear();
	m_RefLocation.Clear();
	m_DefaultLocation = true;

	OnConfigChange();

	cfg_path = g_build_filename(GetDirPath(), GetFileName(), NULL);
	cfg = g_key_file_new();
	g_key_file_load_from_file(cfg, cfg_path, G_KEY_FILE_NONE, NULL);
	// Input files
	count = g_key_file_get_integer(cfg, "PROJECT", "files", NULL);
	for (i=1; i<=count; i++) {
		g_snprintf(file, sizeof(file), "FILE%d", i);
		fpath = g_key_file_get_string(cfg, file, "path", NULL);
		id = g_key_file_get_integer(cfg, file, "fileid", NULL);
		if (fpath && id>0) {
			gtk_list_store_append(m_Frames, &iter);
			juldat = g_key_file_get_double(cfg, file, "juldat", NULL);
			state = g_key_file_get_integer(cfg, file, "state", NULL);
			tcorr = g_key_file_get_double(cfg, file, "tcorr", NULL);
			exptime = g_key_file_get_double(cfg, file, "exptime", NULL);
			ccdtemp = g_key_file_get_double(cfg, file, "ccdtemp", &err);
			frames = g_key_file_get_integer(cfg, file, "frames", NULL);
			if (err) {
				ccdtemp = -999.9;
				g_error_free(err);
				err = NULL;
			}
			ftemp = g_key_file_get_string(cfg, file, "temp", NULL);
			fphot = g_key_file_get_string(cfg, file, "phot", NULL);
			fbias = g_key_file_get_string(cfg, file, "bias", NULL);
			fdark = g_key_file_get_string(cfg, file, "dark", NULL);
			fflat = g_key_file_get_string(cfg, file, "flat", NULL);
			filter = g_key_file_get_string(cfg, file, "filter", NULL);
			result = g_key_file_get_string(cfg, file, "result", NULL);
			nstars = g_key_file_get_integer(cfg, file, "nstars", NULL);
			mstars = g_key_file_get_integer(cfg, file, "mstars", NULL);

			if (state & CFILE_CONVERSION) {
				gchar *f = g_build_filename(get_temp_dir(), ftemp, NULL);
				icon = MakeIcon(f);
				g_free(f);
			} else {
				icon = MakeIcon(fpath);
			}

			sprintf(buf, "%d", id);
			gtk_list_store_set(m_Frames, &iter, FRAME_STATE, state,
				FRAME_ID, id, FRAME_JULDAT, juldat, FRAME_REPORT, result,
				FRAME_ORIGFILE, fpath, FRAME_BIASFILE, fbias, FRAME_DARKFILE, fdark, 
				FRAME_FLATFILE, fflat, FRAME_FILTER, filter, 
				FRAME_TIMECORR, tcorr, FRAME_EXPTIME, exptime, 
				FRAME_CCDTEMP, ccdtemp, FRAME_STARS, nstars, 
				FRAME_MSTARS, mstars, FRAME_TEMPFILE, ftemp, 
				FRAME_PHOTFILE, fphot, FRAME_SUBFRAMES, frames, 
				FRAME_STRINGID, buf, FRAME_ICON, icon, -1);
			if (icon)
				gdk_pixbuf_unref(icon);

			if (id>=m_FreeID)
				m_FreeID = id+1;

			g_free(fflat);
			g_free(fdark);
			g_free(fbias);
			g_free(filter);
			g_free(result);
			g_free(fpath);
			g_free(ftemp);
			g_free(fphot);
		}
	}

	// Selected stars
	str = g_key_file_get_string(cfg, "STARS", "var", NULL);
	m_Selection.SetStarList(ST_VARIABLE, str);
	g_free(str);
	str = g_key_file_get_string(cfg, "STARS", "comp", NULL);
	m_Selection.SetStarList(ST_COMPARISON, str);
	g_free(str);
	str = g_key_file_get_string(cfg, "STARS", "check", NULL);
	m_Selection.SetStarList(ST_CHECK, str);
	g_free(str);

	// Selected stars from catalogue file
	str = g_key_file_get_string(cfg, "CATFILE", "var", NULL);
	m_CatFileSelection.SetStarList(ST_VARIABLE, str);
	g_free(str);
	str = g_key_file_get_string(cfg, "CATFILE", "comp", NULL);
	m_CatFileSelection.SetStarList(ST_COMPARISON, str);
	g_free(str);
	str = g_key_file_get_string(cfg, "CATFILE", "check", NULL);
	m_CatFileSelection.SetStarList(ST_CHECK, str);
	g_free(str);

	// Selected stars from 'Find variables' tool
	str = g_key_file_get_string(cfg, "VARFIND", "var", NULL);
	m_VarFindSelection.SetStarList(ST_VARIABLE, str);
	g_free(str);
	str = g_key_file_get_string(cfg, "VARFIND", "comp", NULL);
	m_VarFindSelection.SetStarList(ST_COMPARISON, str);
	g_free(str);
	str = g_key_file_get_string(cfg, "VARFIND", "check", NULL);
	m_VarFindSelection.SetStarList(ST_CHECK, str);
	g_free(str);

	// Object's coordinates
	str = g_key_file_get_string(cfg, "OBJECT", "name", NULL);
	m_ObjectCoords.SetName(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "OBJECT", "ra", NULL);
	m_ObjectCoords.SetRA(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "OBJECT", "dec", NULL);
	m_ObjectCoords.SetDec(str);
	g_free(str);

	// Reference object's coordinates
	str = g_key_file_get_string(cfg, "REF-OBJECT", "name", NULL);
	m_RefObjectCoords.SetName(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "REF-OBJECT", "ra", NULL);
	m_RefObjectCoords.SetRA(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "REF-OBJECT", "dec", NULL);
	m_RefObjectCoords.SetDec(str);
	g_free(str);

	// Object's coordinates
	str = g_key_file_get_string(cfg, "LOCATION", "default", NULL);
	m_DefaultLocation = (str ? atoi(str)!=0 : true);
	g_free(str);
	str = g_key_file_get_string(cfg, "LOCATION", "name", NULL);
	m_UserLocation.SetName(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "LOCATION", "lon", NULL);
	m_UserLocation.SetLon(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "LOCATION", "lat", NULL);
	m_UserLocation.SetLat(str);
	g_free(str);

	// Reference location
	str = g_key_file_get_string(cfg, "REF-LOCATION", "name", NULL);
	m_RefLocation.SetName(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "REF-LOCATION", "lon", NULL);
	m_RefLocation.SetLon(str);
	g_free(str);
	str = g_key_file_get_string(cfg, "REF-LOCATION", "lat", NULL);
	m_RefLocation.SetLat(str);
	g_free(str);

	// Common
	m_RefID = g_key_file_get_integer(cfg, "PROJECT", "refid", NULL);
	m_Aperture = g_key_file_get_integer(cfg, "PROJECT", "aperture", NULL);
	m_TimeCorr = g_key_file_get_double(cfg, "PROJECT", "time_corr", NULL);

	m_OrigBiasFile.Load(cfg, "BIAS", "orig");
	m_TempBiasFile.Load(cfg, "BIAS", "temp");
	m_OrigDarkFile.Load(cfg, "DARK", "orig");
	m_TempDarkFile.Load(cfg, "DARK", "temp");
	m_OrigFlatFile.Load(cfg, "FLAT", "orig");
	m_TempFlatFile.Load(cfg, "FLAT", "temp");
	m_OrigCatFile.Load(cfg, "REF", "orig");
	m_TempCatFile.Load(cfg, "REF", "temp");

	g_free(cfg_path);
	g_key_file_free(cfg);

	if (m_Aperture < 1)
		m_Aperture = 1;
	if (m_Aperture > MAX_APERTURES)
		m_Aperture = MAX_APERTURES;
}

void CProject::Save(void)
{
	gboolean ok;
	int id, count, frames, nstars, mstars, state;
	char *cfg_path, file[64], *fpath, *ftemp, *fbias, *fdark, *fflat;
	char *str, *filter, *result, *fphot, *data;
	gsize datalen;
	GKeyFile *cfg;
	GtkTreeIter iter;
	double juldat, ccdtemp, exptime, tcorr;

	const char *temp_dir = get_temp_dir();
	force_directory(temp_dir);

	cfg_path = g_build_filename(GetDirPath(), GetFileName(), NULL);
	cfg = g_key_file_new();

	// Input files
	count = 0;
	ok = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(m_Frames), &iter);
	while (ok) {
		g_snprintf(file, sizeof(file), "FILE%d", ++count);
		gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), &iter,
			FRAME_ORIGFILE, &fpath, FRAME_ID, &id, 
			FRAME_STATE, &state, FRAME_JULDAT, &juldat,
			FRAME_REPORT, &result, FRAME_BIASFILE, &fbias, FRAME_DARKFILE, &fdark, 
			FRAME_FLATFILE, &fflat, FRAME_FILTER, &filter, 
			FRAME_TIMECORR, &tcorr, FRAME_EXPTIME, &exptime, 
			FRAME_CCDTEMP, &ccdtemp, FRAME_STARS, &nstars, 
			FRAME_MSTARS, &mstars, FRAME_TEMPFILE, &ftemp, 
			FRAME_PHOTFILE, &fphot, FRAME_SUBFRAMES, &frames, -1);
		g_key_file_set_string(cfg, file, "path", fpath);
		g_key_file_set_integer(cfg, file, "fileid", id);
		g_key_file_set_integer(cfg, file, "state", state);
		if (ftemp)
			g_key_file_set_string(cfg, file, "temp", ftemp);
		if (fphot)
			g_key_file_set_string(cfg, file, "phot", fphot);
		if (juldat>0)
			g_key_file_set_double(cfg, file, "juldat", juldat);
		if (tcorr>0)
			g_key_file_set_double(cfg, file, "tcorr", tcorr);
		if (filter)
			g_key_file_set_string(cfg, file, "filter", filter);
		if (exptime>0.0)
			g_key_file_set_double(cfg, file, "exptime", exptime);
		if (ccdtemp>-999.0)
			g_key_file_set_double(cfg, file, "ccdtemp", ccdtemp);
		if (frames>-999.0)
			g_key_file_set_integer(cfg, file, "frames", frames);
		if (fbias)
			g_key_file_set_string(cfg, file, "bias", fbias);
		if (fdark)
			g_key_file_set_string(cfg, file, "dark", fdark);
		if (fflat)
			g_key_file_set_string(cfg, file, "flat", fflat);
		if (nstars>0)
			g_key_file_set_integer(cfg, file, "nstars", nstars);
		if (mstars>0)
			g_key_file_set_integer(cfg, file, "mstars", mstars);
		if (result) 
			g_key_file_set_string(cfg, file, "result", result);
		g_free(filter);
		g_free(ftemp);
		g_free(fbias);
		g_free(fdark);
		g_free(fflat);
		g_free(fpath);
		g_free(result);
		ok = gtk_tree_model_iter_next(GTK_TREE_MODEL(m_Frames), &iter);
	}
	g_key_file_set_integer(cfg, "PROJECT", "files", count);

	// Selected stars
	str = m_Selection.GetStarList(ST_VARIABLE);
	if (str)
		g_key_file_set_string(cfg, "STARS", "var", str);
	g_free(str);
	str = m_Selection.GetStarList(ST_COMPARISON);
	if (str)
		g_key_file_set_string(cfg, "STARS", "comp", str);
	g_free(str);
	str = m_Selection.GetStarList(ST_CHECK);
	if (str)
		g_key_file_set_string(cfg, "STARS", "check", str);
	g_free(str);

	// Selected stars from catalogue file
	str = m_CatFileSelection.GetStarList(ST_VARIABLE);
	if (str)
		g_key_file_set_string(cfg, "CATFILE", "var", str);
	g_free(str);
	str = m_CatFileSelection.GetStarList(ST_COMPARISON);
	if (str)
		g_key_file_set_string(cfg, "CATFILE", "comp", str);
	g_free(str);
	str = m_CatFileSelection.GetStarList(ST_CHECK);
	if (str)
		g_key_file_set_string(cfg, "CATFILE", "check", str);
	g_free(str);

	// Selected stars from 'Find variables' tool
	str = m_VarFindSelection.GetStarList(ST_VARIABLE);
	if (str)
		g_key_file_set_string(cfg, "VARFIND", "var", str);
	g_free(str);
	str = m_VarFindSelection.GetStarList(ST_COMPARISON);
	if (str)
		g_key_file_set_string(cfg, "VARFIND", "comp", str);
	g_free(str);
	str = m_VarFindSelection.GetStarList(ST_CHECK);
	if (str)
		g_key_file_set_string(cfg, "VARFIND", "check", str);
	g_free(str);

	// Object's coordinates
	if (m_ObjectCoords.Name())
		g_key_file_set_string(cfg, "OBJECT", "name", m_ObjectCoords.Name());
	if (m_ObjectCoords.RA())
		g_key_file_set_string(cfg, "OBJECT", "ra", m_ObjectCoords.RA());
	if (m_ObjectCoords.Dec())
		g_key_file_set_string(cfg, "OBJECT", "dec", m_ObjectCoords.Dec());

	// Reference object's coordinates
	if (m_RefObjectCoords.Name())
		g_key_file_set_string(cfg, "REF-OBJECT", "name", m_RefObjectCoords.Name());
	if (m_RefObjectCoords.RA())
		g_key_file_set_string(cfg, "REF-OBJECT", "ra", m_RefObjectCoords.RA());
	if (m_RefObjectCoords.Dec())
		g_key_file_set_string(cfg, "REF-OBJECT", "dec", m_RefObjectCoords.Dec());

	// Object's coordinates
	g_key_file_set_integer(cfg, "LOCATION", "default", (!m_DefaultLocation ? 0 : 1));
	if (m_UserLocation.Name())
		g_key_file_set_string(cfg, "LOCATION", "name", m_UserLocation.Name());
	if (m_UserLocation.Lon())
		g_key_file_set_string(cfg, "LOCATION", "lon", m_UserLocation.Lon());
	if (m_UserLocation.Lat())
		g_key_file_set_string(cfg, "LOCATION", "lat", m_UserLocation.Lat());

	// Reference location
	if (m_UserLocation.Name())
		g_key_file_set_string(cfg, "REF-LOCATION", "name", m_RefLocation.Name());
	if (m_UserLocation.Lon())
		g_key_file_set_string(cfg, "REF-LOCATION", "lon", m_RefLocation.Lon());
	if (m_UserLocation.Lat())
		g_key_file_set_string(cfg, "REF-LOCATION", "lat", m_RefLocation.Lat());

	// Common
	g_key_file_set_integer(cfg, "PROJECT", "refid", m_RefID);
	g_key_file_set_integer(cfg, "PROJECT", "aperture", m_Aperture);
	g_key_file_set_double(cfg, "PROJECT", "time_corr", m_TimeCorr);

	m_OrigBiasFile.Save(cfg, "BIAS", "orig");
	m_TempBiasFile.Save(cfg, "BIAS", "temp");
	m_OrigDarkFile.Save(cfg, "DARK", "orig");
	m_TempDarkFile.Save(cfg, "DARK", "temp");
	m_OrigFlatFile.Save(cfg, "FLAT", "orig");
	m_TempFlatFile.Save(cfg, "FLAT", "temp");
	m_OrigCatFile.Save(cfg, "REF", "orig");
	m_TempCatFile.Save(cfg, "REF", "temp");

	// Save content to file
	data = g_key_file_to_data(cfg, &datalen, NULL);
	GIOChannel *pFile = g_io_channel_new_file(cfg_path, "w+", NULL);
	if (pFile) {
		g_io_channel_write_chars(pFile, data, datalen, NULL, NULL);
		g_io_channel_close(pFile);
	}
	g_free(data);
	g_free(cfg_path);
	g_key_file_free(cfg);
}

	// Get number of files
int CProject::GetFileCount(void)
{ 
	return gtk_tree_model_iter_n_children(GTK_TREE_MODEL(m_Frames), NULL); 
}

void CProject::Clear(void)
{
	gtk_list_store_clear(m_Frames);
	m_FreeID = 1;
	SetAperture(1);
	ClearReference();
	ClearCorrections();
	ClearTempFiles();
	ClearObject();
	ClearObserver();
	ClearSelection();
	Save();
}

void CProject::ClearCorrections(void)
{
	m_TimeCorr = 0;
	m_OrigBiasFile.Clear();
	m_TempBiasFile.Clear();
	m_OrigDarkFile.Clear();
	m_TempDarkFile.Clear();
	m_OrigFlatFile.Clear();
	m_TempFlatFile.Clear();
	m_OrigCatFile.Clear();
	m_TempCatFile.Clear();
}

void CProject::ClearObject(void)
{
	m_ObjectCoords.Clear();
	m_RefObjectCoords.Clear();
}

void CProject::ClearObserver(void)
{
	m_DefaultLocation = true;
	m_UserLocation.Clear();
	m_RefLocation.Clear();
}

void CProject::ClearSelection(void)
{
	m_Selection.Clear();
	m_CatFileSelection.Clear();
	m_VarFindSelection.Clear();
}

int CProject::AddFile(const char *fpath)
{
	char buf[128];
	GtkTreeIter iter;

	// File must be regular file, not a directory
	if (!g_file_test(fpath, G_FILE_TEST_IS_REGULAR))
		return 0;

	// Discard duplicities
	if (FileInProject(fpath))
		return 0;

	// Add file to a list
	int fileid = m_FreeID++;
	sprintf(buf, "%d", fileid);
	gtk_list_store_append(m_Frames, &iter);
	gtk_list_store_set(m_Frames, &iter, FRAME_ID, fileid, 
		FRAME_STRINGID, buf, FRAME_ORIGFILE, fpath, 
		FRAME_CCDTEMP, -999.9, -1);
	return fileid;
}

// Returns true if the file is already in the project
bool CProject::FileInProject(const char *filepath)
{
	bool ok, retval = false;
	gchar *path;
	GtkTreeIter iter;

	ok = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(m_Frames), &iter);
	while (ok && !retval) {
		gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), &iter, FRAME_ORIGFILE, &path, -1);
		if (StrCmp0(filepath, path)==0)
			retval = true;
		g_free(path);
		ok = gtk_tree_model_iter_next(GTK_TREE_MODEL(m_Frames), &iter);
	}
	return retval;
}

//
// Remove single file
//
void CProject::RemoveFile(GtkTreePath *pPath)
{
	int fileid;
	GtkTreeIter iter;

	if (gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath)) {
		gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), &iter, FRAME_ID, &fileid, -1);
		if (fileid==m_RefID)
			ClearReference();
		gtk_list_store_remove(m_Frames, &iter);
	}
}

//
// Remove multiple files
//
void CProject::RemoveFiles(GtkTreeSelection *pSel)
{
	GList *rr_list, *node;
	GtkTreePath *path;

	rr_list = NULL;
	gtk_tree_selection_selected_foreach(pSel, foreach_func, &rr_list);
	for (node = rr_list; node != NULL; node = node->next) {
		path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
		RemoveFile(path);
		gtk_tree_path_free(path);
	}
	g_list_foreach(rr_list, (GFunc)gtk_tree_row_reference_free, NULL);
	g_list_free(rr_list);
}


//
// Clear the reference file
//
void CProject::ClearReference(void)
{
	m_RefID = -1;
	m_OrigCatFile.Clear();
	ClearSelection();
}


//
// Delete all temporary files
//
void CProject::ClearTempFiles(void)
{
	char *filepath;
	const gchar *filename;
	GSList *list = NULL;

	// Make list files to be deleted
	GDir *dir = g_dir_open(get_temp_dir(), 0, NULL);
	if (dir) {
		filename = g_dir_read_name(dir);
		while (filename) {
			filepath = g_build_filename(get_temp_dir(), filename, NULL);
			if (g_file_test(filepath, G_FILE_TEST_IS_REGULAR) && strcmp(filename, "project.ini")) 
				list = g_slist_prepend(list, filepath);
			else
				g_free(filepath);
			filename = g_dir_read_name(dir);
		}
		g_dir_close(dir);
	}

	// Delete files
	if (list) {
		for (GSList *ptr=list; ptr!=NULL; ptr=ptr->next) {
			g_unlink((gchar*)ptr->data);
			g_free(ptr->data);
		}
		g_slist_free(list);
	}
}


//
// Set reference file
//
void CProject::SetReference(int frame_id, const char *orig_cat_file)
{
	ClearReference();
	if (frame_id>0) {
		m_RefID = frame_id;
	} else {
		m_OrigCatFile = CFileInfo(orig_cat_file);
	}
}


//
// Set reference file
//
bool CProject::GetReference(int *frame_id, const char **orig_file)
{
	if (m_RefID>0) {
		// Set reference frame
		if (frame_id)
			*frame_id = m_RefID;
		if (orig_file)
			*orig_file = NULL;
		return true;
	} else if (m_OrigCatFile.Valid()) {
		// Set catalogue file
		if (frame_id)
			*frame_id = 0;
		if (orig_file)
			*orig_file = m_OrigCatFile.FullPath();
		return true;
	} else {
		if (frame_id)
			*frame_id = 0;
		if (orig_file)
			*orig_file = NULL;
		return false;
	}
}


//
// Find file by frame id
//
gboolean CProject::foreach_find_file(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	int fileid;
	tGetFileInfo *pData = (tGetFileInfo*)data;

	gtk_tree_model_get(model, iter, FRAME_ID, &fileid, -1);
	if (fileid == pData->id) {
		pData->path = gtk_tree_path_copy(path);
		return TRUE;
	}
	return FALSE;
}

// Get reference file name
GtkTreePath *CProject::GetFilePath(int id)
{
	tGetFileInfo info;

	if (id>0) {
		info.id = id;
		info.path = NULL;
		gtk_tree_model_foreach(GTK_TREE_MODEL(m_Frames), foreach_find_file, &info);
		return info.path;
	}
	return NULL;
}

// Get tree path to the first file
GtkTreePath *CProject::GetFirstFile(void)
{
	GtkTreeIter iter;

	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(m_Frames), &iter))
		return gtk_tree_model_get_path(GTK_TREE_MODEL(m_Frames), &iter);
	else
		return NULL;
}

// Get tree path to the last file
GtkTreePath *CProject::GetLastFile(void)
{
	int count;
	GtkTreeIter iter;

	count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(m_Frames), NULL);
	if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(m_Frames), &iter, NULL, count-1))
		return gtk_tree_model_get_path(GTK_TREE_MODEL(m_Frames), &iter);
	else
		return NULL;
}

// Get path to the previous file
GtkTreePath *CProject::GetPreviousFile(GtkTreePath *path)
{
	GtkTreePath *pPrev = gtk_tree_path_copy(path);
	if (gtk_tree_path_prev(pPrev))
		return pPrev;
	else
		return NULL;
}

// Get path to the next file
GtkTreePath *CProject::GetNextFile(GtkTreePath *path)
{
	GtkTreeIter iter;

	gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, path);
	if (gtk_tree_model_iter_next(GTK_TREE_MODEL(m_Frames), &iter))
		return gtk_tree_model_get_path(GTK_TREE_MODEL(m_Frames), &iter);
	else
		return NULL;
}

gboolean CProject::foreach_update_icon(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	int state;
	gchar *ftemp, *f, *fpath;
	GdkPixbuf *icon;

	gtk_tree_model_get(model, iter, FRAME_STATE, &state, -1);
	if (state & CFILE_CONVERSION) {
		gtk_tree_model_get(model, iter, FRAME_TEMPFILE, &ftemp, -1);
		f = g_build_filename(get_temp_dir(), ftemp, NULL);
		icon = MakeIcon(f);
		g_free(f);
		g_free(ftemp);
	} else {
		gtk_tree_model_get(model, iter, FRAME_ORIGFILE, &fpath, -1);
		icon = MakeIcon(fpath);
		g_free(fpath);
	}
	gtk_list_store_set(GTK_LIST_STORE(model), iter, FRAME_ICON, icon, -1);
	if (icon)
		gdk_pixbuf_unref(icon);
	return FALSE;
}

// Update image thumbnail
void CProject::UpdateIcon(GtkTreePath *pPath)
{
	GtkTreeIter iter;

	if (gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath)) 
		foreach_update_icon(GTK_TREE_MODEL(m_Frames), pPath, &iter, 0); 
}

// Update image thumbnail
void CProject::UpdateAllIcons(void)
{
	if (Global.pMainWnd) {
		Global.pMainWnd->BeginUpdate();
		gtk_tree_model_foreach(GTK_TREE_MODEL(m_Frames), foreach_update_icon, 0);
		Global.pMainWnd->EndUpdate();
	}
}

// Configuration has been changed
void CProject::OnConfigChange(void)
{
	UpdateAllIcons();
}

// Make thumbnail image
GdkPixbuf *CProject::MakeIcon(const gchar *filename)
{
	double a, val, datamin, datamax, minval, maxval;
	bool ok;
	int x, y, res, width, height, index, i, j, k, npixels, count;
	int *hist, fullrowwidth, icon_width, icon_height;
	unsigned char *pixels, color, *dptr;
	double scale, offset, *ccd, *sptr;
	CmpackCcdFile *ccd_file;
	GdkPixbuf *pixbuf, *icon;

	if (!filename)
		return NULL;

	// Open file
	char *f = g_locale_from_utf8(filename, -1, NULL, NULL, NULL);
	res = cmpack_ccd_open(&ccd_file, f, CMPACK_OPEN_READONLY);
	g_free(f);
	if (res!=0) 
		return NULL;

	// Get image size
	width = height = 0;
	cmpack_ccd_get_size(ccd_file, &width, &height);
	if (width<=0 || height<=0) {
		cmpack_unref(ccd_file);
		return NULL;
	}

	// Get image data
	npixels = width*height;
	ccd = (double*)g_malloc(npixels*sizeof(double));
	cmpack_ccd_get_image(ccd_file, CMPACK_BITPIX_DOUBLE, 0, 0, ccd);
	cmpack_unref(ccd_file);

	// Autocontrast
	scale = 1.0; 
	offset = datamin = datamax = 0.0;
	ok = false;
	minval = CConfig::GetDbl("CCD", "MinValue", 0.0);
	maxval = CConfig::GetDbl("CCD", "MaxValue", 65535.0);
	for (i=0; i<npixels; i++) {
		val = ccd[i];
		if (val>minval && val<maxval) {
			if (!ok) {
				datamin = datamax = val;
				ok = true;
			} else {
				if (datamin > val)
					datamin = val;
				if (datamax < val)
					datamax = val;
			}
		}
	}
	if (ok && (datamax-datamin)>65536*DBL_EPSILON) {
		a = 65535.0/(datamax-datamin);
		hist = (int*)g_malloc0(65536*sizeof(int));
		for (i=0; i<npixels; i++) {
			val = ccd[i];
			if (val>minval && val<maxval) {
				val = a*(val-datamin);
				if (val<0)
					index = 0;
				else if (val>65535)
					index = 65535;
				else
					index = (int)(val+0.5);
				hist[index]++;
			}
		}
		for (j=count=0; j<65535 && count<0.01*npixels; j++)
			count += hist[j];
		for (k=65535, count=0; k>j && count<0.01*npixels; k--)
			count += hist[k];
		if (k-j>16) {
			scale  = -a/(k-j);
			offset = -scale*(k/a+datamin);
		} else {
			scale = 1.0/(datamax-datamin);
			offset = -scale*datamin;
		}
		g_free(hist);
	}

	// Negative image
	if (CConfig::GetBool("Environment", "ImageInvert")) {
		scale = scale;
		offset = offset;
	} else {
		scale = -scale;
		offset = 1.0-offset;
	}
	scale *= 255.0;
	offset *= 255.0;

	// Convert image data to pixbuf
	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, width, height);
	if (!pixbuf)
		return NULL;
	pixels = gdk_pixbuf_get_pixels(pixbuf);
	fullrowwidth = gdk_pixbuf_get_rowstride(pixbuf);
	for (y=0; y<height; y++) {
		dptr = pixels + (y*fullrowwidth);
		sptr = ccd + (y*width);
		for (x=0; x<width; x++) {
			double value = scale*(*sptr++) + offset;
			if (value <= 0)
				color = 0;
			else if (value >= 255)
				color = 255;
			else
				color = (int)value;
			*dptr++ = color;
			*dptr++ = color;
			*dptr++ = color;
		}
	}
	g_free(ccd);

	// Compute size of the target image
	a = (double)ICON_SIZE / (width>height ? width : height);

	icon_width = (int)(a * width);
	icon_height = (int)(a * height);
	icon = gdk_pixbuf_scale_simple(pixbuf, icon_width, icon_height, GDK_INTERP_BILINEAR);
	gdk_pixbuf_unref(pixbuf);
	return icon;
}

// Get frame state
int CProject::GetState(int frame_id)
{
	GtkTreePath *pPath = GetFilePath(frame_id);
	if (pPath) {
		int state = GetState(pPath);
		gtk_tree_path_free(pPath);
		return state;
	}
	return 0;
}
int CProject::GetState(GtkTreePath *pPath)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		return GetState(&iter);
	}
	return 0;
}
int CProject::GetState(GtkTreeIter *pIter)
{
	int state = 0;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_STATE, &state, -1);
	return state;
}

// Get frame ID
int CProject::GetFrameID(GtkTreePath *pPath)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		return GetFrameID(&iter);
	}
	return 0;
}
int CProject::GetFrameID(GtkTreeIter *pIter)
{
	int id = 0;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_ID, &id, -1);
	return id;
}

// Get full path to the original file
gchar *CProject::GetSourceFile(int frame_id)
{
	GtkTreePath *pPath = GetFilePath(frame_id);
	if (pPath) {
		gchar *file = GetSourceFile(pPath);
		gtk_tree_path_free(pPath);
		return file;
	}
	return NULL;
}
gchar *CProject::GetSourceFile(GtkTreePath *pPath)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		return GetSourceFile(&iter);
	}
	return NULL;
}
gchar *CProject::GetSourceFile(GtkTreeIter *pIter)
{
	gchar *file = NULL;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_ORIGFILE, &file, -1);
	return file;
}


// Get name of the temporary file
gchar *CProject::GetImageFileName(int frame_id)
{
	GtkTreePath *pPath = GetFilePath(frame_id);
	if (pPath) {
		gchar *file = GetImageFileName(pPath);
		gtk_tree_path_free(pPath);
		return file;
	}
	return NULL;
}
gchar *CProject::GetImageFileName(GtkTreePath *pPath)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		return GetImageFileName(&iter);
	}
	return NULL;
}
gchar *CProject::GetImageFileName(GtkTreeIter *pIter)
{
	gchar *file = NULL;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_TEMPFILE, &file, -1);
	return file;
}


// Get full path to the temporary file
gchar *CProject::GetImageFile(int frame_id)
{
	gchar *name = GetImageFileName(frame_id);
	if (name) 
		return g_build_filename(get_temp_dir(), name, NULL);
	return NULL;
}
gchar *CProject::GetImageFile(GtkTreePath *pPath)
{
	gchar *name = GetImageFileName(pPath);
	if (name) 
		return g_build_filename(get_temp_dir(), name, NULL);
	return NULL;
}
gchar *CProject::GetImageFile(GtkTreeIter *pIter)
{
	gchar *name = GetImageFileName(pIter);
	if (name) 
		return g_build_filename(get_temp_dir(), name, NULL);
	return NULL;
}


// Get name of the phootmetry file
gchar *CProject::GetPhotFileName(int frame_id)
{
	GtkTreePath *pPath = GetFilePath(frame_id);
	if (pPath) {
		gchar *file = GetPhotFileName(pPath);
		gtk_tree_path_free(pPath);
		return file;
	}
	return NULL;
}
gchar *CProject::GetPhotFileName(GtkTreePath *pPath)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		return GetPhotFileName(&iter);
	}
	return NULL;
}
gchar *CProject::GetPhotFileName(GtkTreeIter *pIter)
{
	gchar *file = NULL;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_PHOTFILE, &file, -1);
	return file;
}

// Get full path to the photometry file
gchar *CProject::GetPhotFile(int frame_id)
{
	gchar *name = GetPhotFileName(frame_id);
	if (name) 
		return g_build_filename(get_temp_dir(), name, NULL);
	return NULL;
}
gchar *CProject::GetPhotFile(GtkTreePath *pPath)
{
	gchar *name = GetPhotFileName(pPath);
	if (name) 
		return g_build_filename(get_temp_dir(), name, NULL);
	return NULL;
}
gchar *CProject::GetPhotFile(GtkTreeIter *pIter)
{
	gchar *name = GetPhotFileName(pIter);
	if (name) 
		return g_build_filename(get_temp_dir(), name, NULL);
	return NULL;
}

// Set error message, change the state and set the error flag
void CProject::SetError(int frame_id, const char *message, int state_mask, int state_value)
{
	GtkTreePath *pPath = GetFilePath(frame_id);
	if (pPath) {
		SetError(pPath, message, state_mask, state_value);
		gtk_tree_path_free(pPath);
	}
}
void CProject::SetError(GtkTreePath *pPath, const char *message, int state_mask, int state_value)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		SetError(&iter, message, state_mask, state_value);
	}
}
void CProject::SetError(GtkTreeIter *pIter, const char *message, int state_mask, int state_value)
{
	int state;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_STATE, &state, -1);
	state = (state & ~state_mask) | (CFILE_ERROR | state_value);
	gtk_list_store_set(m_Frames, pIter, 
		FRAME_STATE, state, FRAME_REPORT, message, -1);
}

// Set report, change the state and clear the error flag
void CProject::SetResult(int frame_id, const char *message, int state_mask, int state_value)
{
	GtkTreePath *pPath = GetFilePath(frame_id);
	if (pPath) {
		SetResult(pPath, message, state_mask, state_value);
		gtk_tree_path_free(pPath);
	}
}
void CProject::SetResult(GtkTreePath *pPath, const char *message, int state_mask, int state_value)
{
	GtkTreeIter iter;
	if (pPath) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_Frames), &iter, pPath);
		SetResult(&iter, message, state_mask, state_value);
	}
}
void CProject::SetResult(GtkTreeIter *pIter, const char *message, int state_mask, int state_value)
{
	int state;
	gtk_tree_model_get(GTK_TREE_MODEL(m_Frames), pIter, FRAME_STATE, &state, -1);
	state = (state & ~(CFILE_ERROR | state_mask)) | (state_value & state_mask);
	gtk_list_store_set(m_Frames, pIter, 
		FRAME_STATE, state, FRAME_REPORT, message, -1);
}

// Set time correction
void CProject::SetTimeCorrection(double seconds)
{ 
	m_TimeCorr = seconds; 
}

void CProject::SetOrigBiasFile(const gchar *orig_file)
{
	m_OrigBiasFile = CFileInfo(orig_file);
}

void CProject::SetTempBiasFile(const gchar *temp_file)
{
	m_TempBiasFile = CFileInfo(temp_file);
}

void CProject::SetOrigDarkFile(const gchar *orig_file)
{
	m_OrigDarkFile = CFileInfo(orig_file);
}

void CProject::SetTempDarkFile(const gchar *temp_file)
{
	m_TempDarkFile = CFileInfo(temp_file);
}

void CProject::SetOrigFlatFile(const gchar *orig_file)
{
	m_OrigFlatFile = CFileInfo(orig_file);
}

void CProject::SetTempFlatFile(const gchar *temp_file)
{
	m_TempFlatFile = CFileInfo(temp_file);
}

void CProject::SetTempCatFile(const gchar *temp_file)
{
	m_TempCatFile = CFileInfo(temp_file);
}

bool CProject::CheckBiasFile(const gchar *orig_file)
{
	if (!m_OrigBiasFile.Valid() || !m_TempBiasFile.Valid())
		return false;

	CFileInfo f1(orig_file);
	if (f1!=m_OrigBiasFile)
		return false;

	CFileInfo f2(m_TempBiasFile.FullPath());
	if (f2!=m_TempBiasFile)
		return false;

	return true;
}

bool CProject::CheckDarkFile(const gchar *orig_file)
{
	if (!m_OrigDarkFile.Valid() || !m_TempDarkFile.Valid())
		return false;

	CFileInfo f1(orig_file);
	if (f1!=m_OrigDarkFile)
		return false;

	CFileInfo f2(m_TempDarkFile.FullPath());
	if (f2!=m_TempDarkFile)
		return false;

	return true;
}

bool CProject::CheckFlatFile(const gchar *orig_file)
{
	if (!m_OrigFlatFile.Valid() || !m_TempFlatFile.Valid())
		return false;

	CFileInfo f1(orig_file);
	if (f1!=m_OrigFlatFile)
		return false;

	CFileInfo f2(m_TempFlatFile.FullPath());
	if (f2!=m_TempFlatFile)
		return false;

	return true;
}

bool CProject::CheckCatFile(const gchar *orig_file)
{
	if (!m_OrigCatFile.Valid() || !m_TempCatFile.Valid())
		return false;

	CFileInfo f1(orig_file);
	if (f1!=m_OrigCatFile)
		return false;

	CFileInfo f2(m_TempCatFile.FullPath());
	if (f2!=m_TempCatFile)
		return false;

	return true;
}

void CProject::SetUseDefaultLocation(bool value)
{
	m_DefaultLocation = value;
}

CObserverCoords CProject::DefaultLocation() const
{
	CObserverCoords obs;
	char *name, *lon, *lat;

	if (CConfig::TryGetStr("Observer", "Location", &name)) {
		obs.SetName(name);
		g_free(name);
	}
	if (CConfig::TryGetStr("Observer", "Longitude", &lon)) {
		obs.SetLon(lon);
		g_free(lon);
	}
	if (CConfig::TryGetStr("Observer", "Latitude", &lat)) {
		obs.SetLat(lat);
		g_free(lat);
	}
	return obs;
}
