/**************************************************************

chart.cpp (C-Munipack project)
Widget which can draw a chart
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/
#define _USE_MATH_DEFINES

#include <stdlib.h>
#include <glib.h>
#include <math.h>
#include <string.h>
#include <cmunipack.h>

#include "qphot.h"

static bool sky(const double *ccd, int width, int height, int x, int y, 
			   double lo_data, double hi_data, double is, double os, 
			   double *mean, double *stddev)
{
	bool res = false;
	int i, j, nsky, hw = (int)ceil(os);
	int left, right, top, bottom, pixels;
	double rinsq, routsq, dysq, rsq;
	double *s, value;

	if (hw<=1 || is<=1 || os<=1 || is>os-1)
		return false;

	left = x - hw;
	if (left < 0)
		left = 0;
	right = x + hw;
	if (right >= width)
		right = width-1;
	top = y - hw;
	if (top < 0)
		top = 0;
	bottom = y + hw;
	if (bottom >= height)
		bottom = height-1;
	if (left>=right || top>=bottom)
		return false;

	pixels = (right-left+1)*(bottom-top+1);
	s = (double*)g_malloc(pixels*sizeof(double));
	if (s) {
		rinsq  = is*is;		// Inner sky radius squared
		routsq = os*os;		// Outer sky radius squared
		nsky   = 0;
		for (j=top; j<=bottom; j++) {
			dysq = (j-y)*(j-y);
			for (i=left; i<=right; i++) {
				rsq = dysq + (i-x)*(i-x);    // squared distance from [i,j] to the center of the star
				if (rsq>=rinsq && rsq<=routsq) {
					value = ccd[i+j*width];
					if (value>lo_data && value<hi_data) 
						s[nsky++] = value;
				}
			}
		}
		if (nsky > 1) {
			cmpack_robustmean(nsky, s, mean, stddev);
			res = true;
		}
		g_free(s);
	}
	return res;
}

static double full_width(const double *buf, int step, int count, int maxpos, double half)
{
	double x_left, x_right, y1, y2, res;
	int i;

	res = -1;
    // Find first value lower than Y on the left of the maximum
	x_left = maxpos;
	for (i=maxpos-1; i>=0; i--) {
		y1 = buf[i*step];
		y2 = buf[(i+1)*step];
		if (y1>=y2)
			break;
		if (y1<=half) {
			x_left = i + (half-y1)/(y2-y1);
			break;
		}
	}
	// Find first value lower than Y on the right of the maximum
	x_right = maxpos;
	for (i=maxpos+1; i<count; i++) {
		y1 = buf[i*step];
		y2 = buf[(i-1)*step];
		if (y1>=y2)
			break;
		if (y1<=half) {
			x_right = i - (half-y1)/(y2-y1);
			break;
		}
	}
	if (x_left<maxpos && x_right>maxpos)
		return x_right - x_left;
	else
		return 0.0;
}

static bool signal(const double *ccd, int width, int height, int x, int y, 
				   double lo_data, double hi_data, double aperture, 
				   double *psignal, double *parea)
{
	int i, j, hw = (int)ceil(aperture);
	int left, top, right, bottom;
	double f, dysq, rsq, d, area, apmag, rmin_sq, rmax_sq;

	left = x - hw;
	if (left < 0)
		left = 0;
	right = x + hw;
	if (right >= width)
		right = width-1;
	top = y - hw;
	if (top < 0)
		top = 0;
	bottom = y + hw;
	if (bottom >= height)
		bottom = height-1;
	if (left>right || top>bottom)
		return false;

	apmag = area = 0.0;
	rmin_sq = (aperture-0.5)*(aperture-0.5);
	rmax_sq = (aperture+0.5)*(aperture+0.5);
	for (j=top; j<=bottom; j++) {
		dysq = (j-y)*(j-y);
		for (i=left; i<=right; i++) {
			rsq = dysq + (i-x)*(i-x);    // squared distance from [i,j] to the center of the star
			if (rsq<rmax_sq) {
		        d = ccd[i+j*width];
				if (d>lo_data && d<hi_data) {
					f = (rsq<rmin_sq ? 1 : aperture - sqrt(rsq) + 0.5);
					apmag += f*d;	   // sum of pixel values that fits to the aperture k
					area  += f;         // size of the area of the aperture k
				}
			}
		}
	}
	if (area>=1.0) {
		if (psignal) 
			*psignal = apmag;
		if (parea) 
			*parea = area;
		return true;
	}
	return false;
}

bool QuickPhotometry(const double *ccd, int width, int height,
	double lo_data, double hi_data, double aperture, double sky_inner,
	double sky_outer, int x, int y, tPhotResult *result)
{
	double d, rmax_sq, dysq, rsq, d_max;
	double skyval, skysig, thresh, area, starval, fwhm_x, fwhm_y;
	int i, j, left, top, right, bottom, hw = 5, max_x, max_y;

	result->aperture = aperture;
	left = x - hw;
	if (left < 0)
		left = 0;
	right = x + hw;
	if (right >= width)
		right = width-1;
	top = y - hw;
	if (top < 0)
		top = 0;
	bottom = y + hw;
	if (bottom >= height)
		bottom = height-1;
	if (left>right || top>bottom)
		return false;

	// Search for maximum in the region of interest }
	rmax_sq = hw*hw;
	max_x = x; 
	max_y = y; 
	d_max = ccd[x+y*width];
	for (j=top; j<=bottom; j++) {
		dysq = (j-y)*(j-y);
		for (i=left; i<=right; i++) {
			rsq = dysq + (i-x)*(i-x);    // squared distance from [i,j] to the center of the star
			if (rsq<rmax_sq) {
				d = ccd[i+j*width];
				if (d>lo_data && d<hi_data && d>d_max) {
					max_x = i;
					max_y = j;
					d_max = d;
				}
			}
		}
	}
	result->center_x = max_x;
	result->center_y = max_y;
	result->maxval = d_max;

	if (sky(ccd, width, height, max_x, max_y, lo_data, hi_data, sky_inner, sky_outer, &skyval, &skysig)) {
		result->skyval = skyval;
		result->skysig = skysig;
		if (skyval < d_max) {
			thresh = 0.5*(d_max + result->skyval);
			fwhm_x = full_width(ccd+(max_y*width), 1, width, max_x, thresh);
			fwhm_y = full_width(ccd+max_x, width, height, max_y, thresh);
			if (fwhm_x>0 && fwhm_y>0)
				result->fwhm = 0.5*(fwhm_x + fwhm_y);
			else if (fwhm_x<=0)
				result->fwhm = fwhm_y;
			else if (fwhm_y<=0)
				result->fwhm = fwhm_x;
			else
				result->fwhm = -1.0;
		}
	}

	if (signal(ccd, width, height, max_x, max_y, lo_data, hi_data, aperture, &starval, &area)) {
		starval = starval - skyval*area;
		result->snr = (skysig>0.0 ? starval/skysig : -1.0);
		result->signal = starval;
	}
	return true;
}
