/**************************************************************

readall_dlg.cpp (C-Munipack project)
The preview dialog for tables
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <math.h>
#include <cmunipack.h>

#include "readall_dlg.h"
#include "configuration.h"
#include "showheader_dlg.h"
#include "utils.h"
#include "chartexport_dlg.h"

enum tMenuId
{
	MENU_FILE = 1,
	MENU_VIEW
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_PRINT,
	CMD_EXPORT,
	CMD_PROPERTIES
};

enum tFieldId
{
	FIELD_FILE,
	FIELD_FILENAME,
	FIELD_DIRPATH,
	FIELD_FILTER,
	FIELD_APERTURE
};

//-------------------------   MENU FOR PREVIEW WINDOW   ---------------------------

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_PROPERTIES,			"Show _Properties" },
	{ CMenuBar::MB_SEPARATOR },
	//{ CMenuBar::MB_ITEM, CMD_EXPORT,			"E_xport" },
	//{ CMenuBar::MB_ITEM, CMD_PRINT,			"P_rint" },
	//{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_CLOSE,				"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu PreviewMenu[] = {
	{ "_File", MENU_FILE, FileMenu },
	{ NULL }
};

//-------------------------   PREVIEW DIALOG   --------------------------------

//
// Constructor
//
CReadAllDlg::CReadAllDlg(GtkWindow *parent):CFileDlg(parent), m_MagDevData(1), 
	m_LightCurveData(2), m_AllFile(NULL), m_Updating(false)
{
	static const GdkGeometry geom = { 600, 400 };

	GtkWidget *main_vbox, *frame, *table, *hbox, *vbox, *separator, *box2;
	GSList *group;

	gtk_window_set_geometry_hints(GTK_WINDOW(m_pDlg), NULL, (GdkGeometry*)&geom, GDK_HINT_MIN_SIZE);
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), 600, 400);

	m_VarID = m_CompID = -1;
	m_SelectMode = SELECT_VAR;
	m_Updating = false;
	m_LastStarIndex = m_LastPtIndex = -1;
	m_DispMode = (tDispMode)CConfig::GetInt("ReadAllDlg", "Mode", DISP_FULL);

	// Create table context
	m_MagDevTable = cmpack_tab_init(CMPACK_TABLE_MUNIFIND);
	m_LightCurveTable = cmpack_tab_init(CMPACK_TABLE_LIGHTCURVE);

	// Dialog icon
	gchar *icon = get_icon_file("varfind");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Window layout
	main_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(m_pDlg), main_vbox);

	// Menu bar
	m_Menu.Create(PreviewMenu, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(main_vbox), m_Menu.Handle(), false, false, 0);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(main_vbox), hbox, TRUE, TRUE, 0);

	// Table layout
	table = gtk_table_new(2, 2, true);
	gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
	gtk_widget_set_size_request(table, 640, 480);

	// Varfind-curve graph
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_MagDevGraph.SetGraphPadding(2, 2, 32, 2);
	m_MagDevGraph.EnableTracking(true);
	m_MagDevGraph.RegisterCallback(GraphCallback, this);
	m_MagDevGraph.SetData(&m_MagDevData);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_MagDevGraph.Handle())));
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 1, 0, 1);

	// Chart
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_Chart.ShowBadStars(false);
	m_Chart.ShowNonMatchedStars(false);
	m_Chart.EnableTracking(true);
	m_Chart.RegisterCallback(ChartCallback, this);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Chart.Handle())));
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 1, 2, 0, 1);

	// Light-curve graph
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	m_LightCurveGraph.EnableTracking(false);
	m_LightCurveGraph.SetData(&m_LightCurveData);
	m_MagDevGraph.SetGraphPadding(2, 2, 48, 2);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_LightCurveGraph.Handle())));
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 2, 1, 2);

	// Selection mode
	vbox = gtk_vbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 0);
	m_VarBtn = gtk_toggle_button_new_with_label("Select variable star");
	gtk_box_pack_start(GTK_BOX(vbox), m_VarBtn, FALSE, TRUE, 0);
	m_CompBtn = gtk_toggle_button_new_with_label("Select comparison star");
	gtk_box_pack_start(GTK_BOX(vbox), m_CompBtn, FALSE, TRUE, 0);

	// Display mode
	frame = gtk_frame_new("Show");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 2);
	box2 = gtk_vbox_new(TRUE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(box2), 4);
	gtk_container_add(GTK_CONTAINER(frame), box2);
	m_ShowImage = gtk_radio_button_new_with_label(NULL, "Image only");
	gtk_box_pack_start(GTK_BOX(box2), m_ShowImage, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ShowImage));
	m_ShowChart = gtk_radio_button_new_with_label(group, "Chart only");
	gtk_box_pack_start(GTK_BOX(box2), m_ShowChart, TRUE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ShowImage));
	m_ShowFull = gtk_radio_button_new_with_label(group, "Image + chart");
	gtk_box_pack_start(GTK_BOX(box2), m_ShowFull, TRUE, TRUE, 0);

	// Separator
	separator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, TRUE, 0);

	// Save buttons
	m_SaveMagDev = gtk_button_new_with_label("Save mag-dev data");
	gtk_box_pack_start(GTK_BOX(vbox), m_SaveMagDev, FALSE, TRUE, 0);
	m_SaveChart = gtk_button_new_with_label("Save chart");
	gtk_box_pack_start(GTK_BOX(vbox), m_SaveChart, FALSE, TRUE, 0);
	m_SaveCurve = gtk_button_new_with_label("Save light curve");
	gtk_box_pack_start(GTK_BOX(vbox), m_SaveCurve, FALSE, TRUE, 0);

	SetDisplayMode(m_DispMode);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowImage), m_DispMode==DISP_IMAGE);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowChart), m_DispMode==DISP_CHART);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowFull), m_DispMode==DISP_FULL);

	gtk_widget_show_all(hbox);

	// Signals:
	g_signal_connect(G_OBJECT(m_VarBtn), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_CompBtn), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_SaveMagDev), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_SaveChart), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_SaveCurve), "clicked", G_CALLBACK(clicked), this);
	g_signal_connect(G_OBJECT(m_ShowImage), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_ShowChart), "toggled", G_CALLBACK(toggled), this);
	g_signal_connect(G_OBJECT(m_ShowFull), "toggled", G_CALLBACK(toggled), this);

	gtk_widget_show_all(main_vbox);

	UpdateControls();
}


//
// Destructor
//
CReadAllDlg::~CReadAllDlg()
{
	cmpack_unref(m_MagDevTable);
	cmpack_unref(m_LightCurveTable);
	cmpack_unref(m_AllFile);
}

void CReadAllDlg::UpdateControls(void)
{
	// Frame menu
	m_Menu.EnableMenu(MENU_FILE, true);
	m_Menu.Enable(CMD_PROPERTIES, true);
	m_Menu.Enable(CMD_EXPORT, true);
	m_Menu.Enable(CMD_PRINT, true);
}

void CReadAllDlg::MenuCallback(int message, int wparam, void *lparam, void *cb_data)
{
	CReadAllDlg *pMe = (CReadAllDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CReadAllDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_CLOSE:
		Destroy();
		break;
	case CMD_PROPERTIES:
		ShowProperties();
		break;
	}
}


//
// Show file properties
//
void CReadAllDlg::ShowProperties(void)
{
	CReadAllInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_Path, &m_Info);
}


//
// Maximize window
//
void CReadAllDlg::OnBeforeShow(void)
{
	gtk_window_maximize(GTK_WINDOW(m_pDlg));
}


//
// Load a file
//
bool CReadAllDlg::LoadFile(const char *fpath)
{
	bool ok;
	int aperture;
	char *fts_file, *pht_file, *cat_file;

	m_CompID = m_VarID = -1;
	if (m_AllFile) {
		cmpack_unref(m_AllFile);
		m_AllFile = NULL;
	}
	if (cmpack_all_load(&m_AllFile, fpath, CMPACK_LOAD_DEFAULT)==0) {
		ok = false;
		pht_file = SetFileExtension(fpath, "pht");
		if (g_file_test(pht_file, G_FILE_TEST_IS_REGULAR)) {
			m_Chart.LoadChart(pht_file, true);
			ok = true;
		}
		g_free(pht_file);
		if (!ok) {
			pht_file = SetFileExtension(fpath, "mat");
			if (g_file_test(pht_file, G_FILE_TEST_IS_REGULAR)) {
				m_Chart.LoadChart(pht_file, true);
				ok = true;
			}
			g_free(pht_file);
		}
		if (!ok) {
			cat_file = SetFileExtension(fpath, "xml");
			if (g_file_test(cat_file, G_FILE_TEST_IS_REGULAR)) {
				m_Chart.LoadChart(cat_file, true);
				ok = true;
			}
			g_free(cat_file);
		}
		if (ok) {
			fts_file = SetFileExtension(fpath, "fts");
			if (g_file_test(fts_file, G_FILE_TEST_IS_REGULAR)) 
				m_Chart.LoadImage(fts_file);
			g_free(fts_file);
		}
		aperture = cmpack_all_get_aperture(m_AllFile);
		if (aperture)
			m_Chart.SetAperture(aperture);
		else
			m_Chart.SetAperture(1);
		RebuildData();
		return ok;
	}
	return false;
}

void CReadAllDlg::RebuildData(void)
{
	int res, row_count, i;
	double mag, data[2];
	CmpackMuniFind *mfind;

	cmpack_tab_clear(m_MagDevTable);
	cmpack_tab_clear(m_LightCurveTable);
	m_MagDevData.Clear();
	m_LightCurveData.Clear();

	mfind = cmpack_mfind_init();
	cmpack_mfind_set_threshold(mfind, CConfig::GetDbl("MuniFind", "Threshold", 60.0));
	if (m_CompID>=0)
		res = cmpack_mfind(mfind, m_AllFile, m_MagDevTable, m_CompID);
	else
		res = cmpack_mfind(mfind, m_AllFile, m_MagDevTable, CMPACK_MFIND_AUTO);
	if (res==0) 
		m_CompID = cmpack_mfind_get_comparison(mfind);
	cmpack_unref(mfind);

	// Load mag-dev data
	m_MagDevData.LockRedraw();
	m_MagDevData.Clear();
	row_count = cmpack_tab_nrows(m_MagDevTable);
	m_MagDevData.Init(2, row_count);
	m_MinMag = m_MaxMag = m_MaxDev = 0.0;
	for (i=0; i<row_count; i++) {
		cmpack_tab_gtdd(m_MagDevTable, i, 1, &mag);
		cmpack_tab_gtdd(m_MagDevTable, i, 2, &data[0]);
		cmpack_tab_gtdd(m_MagDevTable, i, 0, &data[1]);
		if (mag>-99 && mag<99) {
			if (mag < m_MinMag)
				m_MinMag = mag;
			if (mag > m_MaxMag)
				m_MaxMag = mag;
			if (data[0] > m_MaxDev)
				m_MaxDev = data[0];
			m_MagDevData.Append(mag, data);
		}
	}
	m_MagDevData.UnlockRedraw();
	m_MagDevGraph.ClearColumns();
	m_MagDevGraph.SetXParams("mag", m_MinMag, m_MaxMag, 0.5, 0, 1);
	m_MagDevGraph.SetYParams("dev", 0, m_MaxDev, 0.1, 0, 1);
	m_MagDevGraph.AddColumn(0, -1, 0);
	m_MagDevGraph.AutoZoom(true, true);

	m_LastStarIndex = m_LastPtIndex = -1;
	UpdateSelection();
}

void CReadAllDlg::toggled(GtkToggleButton *pBtn, CReadAllDlg *pDlg)
{
	pDlg->OnToggled(pBtn);
}

void CReadAllDlg::OnToggled(GtkToggleButton *pBtn)
{
	if (gtk_toggle_button_get_active(pBtn)!=0) {
		if (pBtn==GTK_TOGGLE_BUTTON(m_VarBtn))
			SetMode(SELECT_VAR);
		else if (pBtn==GTK_TOGGLE_BUTTON(m_CompBtn))
			SetMode(SELECT_COMP);
		else if (pBtn == GTK_TOGGLE_BUTTON(m_ShowImage)) 
			SetDisplayMode(DISP_IMAGE);
		else if (pBtn == GTK_TOGGLE_BUTTON(m_ShowChart))
			SetDisplayMode(DISP_CHART);
		else if (pBtn == GTK_TOGGLE_BUTTON(m_ShowFull))
			SetDisplayMode(DISP_FULL);
	}
}

void CReadAllDlg::clicked(GtkButton *pBtn, CReadAllDlg *pDlg)
{
	pDlg->OnClicked(pBtn);
}

void CReadAllDlg::OnClicked(GtkButton *pBtn)
{
	if (pBtn==GTK_BUTTON(m_SaveMagDev))
		SaveMagDev();
	else if (pBtn==GTK_BUTTON(m_SaveChart))
		SaveChart();
	else if (pBtn==GTK_BUTTON(m_SaveCurve))
		SaveCurve();
}

void CReadAllDlg::SetMode(tSelectMode mode)
{
	if (!m_Updating) {
		m_Updating = true;
		m_SelectMode = mode;
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_VarBtn), mode==SELECT_VAR);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_CompBtn), mode==SELECT_COMP);
		m_Updating = false;
	}
}

void CReadAllDlg::ChartCallback(int message, int wparam, void *lparam, void *cb_data)
{
	CReadAllDlg *pMe = (CReadAllDlg*)cb_data;
		
	switch (message)
	{
	case CChartBase::CB_MOUSE_MOVE:
		pMe->OnChartMouseMove();
		break;
	case CChartBase::CB_LEFT_CLICK:
		pMe->OnChartLButtonClick((GdkEventButton*)lparam);
		break;
	}
}

void CReadAllDlg::OnChartMouseMove()
{
	int index, x, y;

	if (m_Chart.GetCursorPos(&x, &y)) 
		index = m_Chart.FindStarByPos(x, y, false);
	else
		index = -1;
	if (index>=0) {
		if (m_LastStarIndex!=index) {
			m_LastStarIndex = index;
			m_Chart.SetFocusedStar(index);
		}
	} else {
		if (m_LastStarIndex>=0) {
			m_LastStarIndex = -1;
			m_Chart.SetFocusedStar(-1);
		}
	}
}

void CReadAllDlg::OnChartLButtonClick(const GdkEventButton *event)
{
	CChartBase::tStarInfo star;

	if (m_Chart.GetStar(m_Chart.FocusedStar(), &star)) {
		if (m_SelectMode == SELECT_VAR) {
			if (star.id!=m_VarID && star.id!=m_CompID) {
				m_VarID = star.id;
				UpdateSelection();
			}
		} else {
			if (star.id!=m_CompID) {
				m_CompID = star.id;
				if (m_CompID == m_VarID)
					m_VarID = -1;
				RebuildData();
				UpdateSelection();
			}
		}
	}
}

void CReadAllDlg::GraphCallback(int message, int wparam, void *lparam, void *cb_data)
{
	CReadAllDlg *pMe = (CReadAllDlg*)cb_data;
		
	switch (message)
	{
	case CGraph::CB_MOUSE_MOVE:
		pMe->OnGraphMouseMove();
		break;
	case CGraph::CB_LEFT_CLICK:
		pMe->OnGraphLButtonClick((GdkEventButton*)lparam);
		break;
	}
}

void CReadAllDlg::OnGraphMouseMove(void)
{
	int index;
	double x, y;

	if (m_MagDevGraph.GetMousePos(&x, &y)) 
		index = m_MagDevGraph.FindByPos(x, y, 0);
	else
		index = -1;
	if (index>=0) {
		if (m_LastPtIndex!=index) {
			m_LastPtIndex = index;
			m_MagDevGraph.SetFocusedItem(index, 0);
		}
	} else {
		if (m_LastPtIndex>=0) {
			m_LastPtIndex = -1;
			m_MagDevGraph.SetFocusedItem(-1, -1);
		}
	}
}

void CReadAllDlg::OnGraphLButtonClick(const GdkEventButton *event)
{
	if (m_LastPtIndex>=0) {
		int starid = (int)m_MagDevData.GetY(m_LastPtIndex, 1);
		if (m_SelectMode == SELECT_VAR) {
			if (starid!=m_VarID && starid!=m_CompID) {
				m_VarID = starid;
				UpdateSelection();
			}
		} else {
			if (starid!=m_CompID) {
				m_CompID = starid;
				if (m_CompID == m_VarID)
					m_VarID = -1;
				RebuildData();
				UpdateSelection();
			}
		}
	}
}


void CReadAllDlg::UpdateSelection(void)
{
	int i, count, row_count;
	double jd, yvals[2];

	m_Chart.GetSelection()->Clear();
	if (m_VarID>=0)
		m_Chart.GetSelection()->Select(m_VarID, ST_VARIABLE);
	if (m_CompID>=0)
		m_Chart.GetSelection()->Select(m_CompID, ST_COMPARISON);
	
	m_MagDevGraph.DeleteAllTags();
	count = m_MagDevData.Count();
	for (i=0; i<count; i++) {
		if ((int)m_MagDevData.GetY(i, 1)==m_VarID) {
			m_MagDevGraph.SetTag(0, i, "var", RGB(255,0,0));
			break;
		}
	}

	/* Update light-curve */
	m_LightCurveData.LockRedraw();
	m_LightCurveData.Clear();
	m_JDMin = m_JDMax = 0.0;
	cmpack_tab_clear(m_LightCurveTable);
	if (m_VarID>=0 && m_CompID>=0) {
		cmpack_all_lightcurve(m_AllFile, m_VarID, m_CompID, m_LightCurveTable);
		row_count = cmpack_tab_nrows(m_LightCurveTable);
		if (row_count>0) {
			m_LightCurveData.Init(2, row_count);
			for (i=0; i<row_count; i++) {
				cmpack_tab_gtdd(m_LightCurveTable, i, 0, &jd);
				cmpack_tab_gtdd(m_LightCurveTable, i, 1, &yvals[0]);
				cmpack_tab_gtdd(m_LightCurveTable, i, 2, &yvals[1]);
				m_LightCurveData.Append(jd, yvals);
				if (i==0) {
					m_JDMin = m_JDMax = jd;
				} else {
					if (jd < m_JDMin)
						m_JDMin = jd;
					if (jd > m_JDMax)
						m_JDMax = jd;
				}
			}
			m_LightCurveGraph.SetXParams("JD", m_JDMin, m_JDMax, 1.0/144.0, 0, 2);
			m_LightCurveGraph.ClearColumns();
			m_LightCurveGraph.AddColumn(0, 1, 0);
			m_LightCurveGraph.SetYParams("V-C", -99, 99, 0.5, CGraph::REVERSE_SCALE, 2);
			m_LightCurveGraph.AutoZoom(true, true);
		}
	}
	m_LightCurveData.UnlockRedraw();
}

void CReadAllDlg::SaveMagDev(void)
{
	char *filename, *folder, *fpath, *f;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save varfind data to...",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("ReadAll", "MagDevName", "output.txt");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
		g_free(filename);
	}

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		filename = g_path_get_basename(fpath);
		CConfig::SetStr("ReadAll", "MagDevName", filename);
		g_free(filename);
		f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
		cmpack_tab_save(m_MagDevTable, f);
		g_free(f);
		g_free(fpath);
	}
	gtk_widget_destroy(pSaveDlg);
}

void CReadAllDlg::SaveCurve(void)
{
	char *filename, *folder, *fpath, *f;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save light-curve to...",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = CConfig::GetStr("Output", "Folder", NULL);
	if (folder) {
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
		g_free(folder);
	}
	filename = CConfig::GetStr("ReadAll", "LightCurveName", "output.txt");
	if (filename) {
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
		g_free(filename);
	}

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		filename = g_path_get_basename(fpath);
		CConfig::SetStr("ReadAll", "LightCurveName", filename);
		g_free(filename);
		f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
		cmpack_tab_save(m_LightCurveTable, f);
		g_free(f);
		g_free(fpath);
	}
	gtk_widget_destroy(pSaveDlg);
}

void CReadAllDlg::SaveChart()
{
	CChartExportDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(&m_Chart);
}

void CReadAllDlg::SetDisplayMode(tDispMode mode)
{
	m_DispMode = mode;
	CConfig::SetInt("ReadAllDlg", "Mode", m_DispMode);

	m_Chart.ShowImage(m_DispMode!=DISP_CHART);
	m_Chart.ShowChart(m_DispMode!=DISP_IMAGE);
}

//---------------------------   INFO DIALOG   --------------------------------

//
// Constructor
//
CReadAllInfoDlg::CReadAllInfoDlg(GtkWindow *pParent):CInfoDlg(pParent), m_Path(NULL)
{
	GtkWidget *bbox;

	AddHeading(FIELD_FILE, 0, 0, "File");
	AddField(FIELD_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_DIRPATH, 0, 2, "Location", PANGO_ELLIPSIZE_MIDDLE);
	AddSeparator(0, 4);
	AddField(FIELD_FILTER, 0, 5, "Optical filter", PANGO_ELLIPSIZE_END);
	AddField(FIELD_APERTURE, 0, 6, "Aperture");
	gtk_widget_show_all(m_Tab);

	bbox = gtk_hbutton_box_new();
	gtk_box_set_spacing(GTK_BOX(bbox), 8);
	gtk_container_set_border_width(GTK_CONTAINER(bbox), 4);
	m_HdrBtn = gtk_button_new_with_label("Show header...");
	gtk_box_pack_start(GTK_BOX(bbox), m_HdrBtn, true, true, 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), bbox);
	gtk_widget_show_all(bbox);

	g_signal_connect(G_OBJECT(m_HdrBtn), "clicked", G_CALLBACK(button_clicked), this);
}


//
// Destructor
//
CReadAllInfoDlg::~CReadAllInfoDlg()
{
	if (m_Path)
		g_free(m_Path);
}


//
// Show dialog
//
void CReadAllInfoDlg::ShowModal(const char *path, const CReadAllInfo *pData)
{
	char *basename, *dirpath;

	// Update properties
	g_free(m_Path);
	m_Path = g_strdup(path);
	basename = g_path_get_basename(path);
	SetField(FIELD_FILENAME, basename);
	g_free(basename);
	dirpath = g_path_get_dirname(path);
	SetField(FIELD_DIRPATH, dirpath);
	g_free(dirpath);

	if (pData->Filter())
		SetField(FIELD_FILTER, pData->Filter());
	else
		SetField(FIELD_FILTER, "");
	if (pData->Aperture()>0)
		SetField(FIELD_APERTURE, pData->Aperture());
	else
		SetField(FIELD_APERTURE, "");

	CInfoDlg::ShowModal();
}

void CReadAllInfoDlg::button_clicked(GtkWidget *button, CReadAllInfoDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CReadAllInfoDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==GTK_WIDGET(m_HdrBtn))
		ShowHeader();
}


void CReadAllInfoDlg::ShowHeader(void)
{
	CShowHeaderDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(m_Path);
}
