/**************************************************************

chart.cpp (C-Munipack project)
Widget which can draw a chart
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#define _USE_MATH_DEFINES

#include <stdlib.h>
#include <glib.h>
#include <math.h>
#include <string.h>

#include "scale.h"
#include "chart.h"

//------------------------------   CONSTANTS   -------------------------------------------

#define IMAGE_WIDTH		512

#define NUM_STEPS       15

static const int XSteps[NUM_STEPS] = {	
	50000, 20000, 10000, 5000, 2000, 1000, 500, 200, 100, 50, 20, 10, 5, 2, 1
};

//--------------------------   HELPER FUNCTIONS   ----------------------------------

//
// grayscale mapping
//
static unsigned long grayscale(double x)
{
	x = 255.0*x;

	if (x<=0.0)
		return 0x000000;
	if (x>=255.0)
		return 0xFFFFFF;

	int ci = (int)x;
	return ci | (ci<<8) | (ci<<16);
}


//
// pseudocolors mapping (black, violet, blue, cyan, green, yellow, red, white) }
//
static unsigned long pseudocolor(double x)
{
	x = x*1785.0;

	if (x>=0x6FF)
		return 0xFFFFFF;
	if (x<=0)
		return 0x000000;

	int ci = (int)x;
	switch (ci>>8)
	{
	case 0:
		return (ci & 0xFF) | (ci & 0xFF)<<16;
	case 1:
		return (~ci & 0xFF)<<16 | 0xFF;
	case 2:
		return (ci & 0xFF)<<8 | 0xFF;
	case 3:
		return 0xFF00 | (~ci & 0xFF);
	case 4:
		return (ci & 0xFF)<<16 | 0xFF00;
	case 5:
		return 0xFF0000 | (~ci & 0xFF)<<8;
	case 6:
		return 0xFF0000 | (ci & 0xFF)<<8 | (ci & 0xFF);
	default:
		return 0;
	}
}


//--------------------------   CHART CLASS   ----------------------------------

// Default constructor
CScale::CScale():m_PixWidth(0), m_PixHeight(0), m_Pixmap(NULL), m_pChart(NULL)
{
	m_Visible = false;
	m_Frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(m_Frame), GTK_SHADOW_IN);

	m_DrawArea = gtk_drawing_area_new();
	gtk_container_add(GTK_CONTAINER(m_Frame), m_DrawArea);
	gtk_widget_set_size_request(GTK_WIDGET(m_DrawArea), 0, 40);

	m_Image = cairo_image_surface_create(CAIRO_FORMAT_RGB24, IMAGE_WIDTH, 1);

	// Signals used to handle backing pixmap 
	g_signal_connect (G_OBJECT(m_DrawArea), "expose_event", G_CALLBACK(expose_event), this);
	g_signal_connect (G_OBJECT(m_DrawArea), "configure_event", G_CALLBACK(configure_event), this);
}

// Destructor
CScale::~CScale()
{
	if (m_pChart)
		SetChart(NULL);
	if (m_Image)
		cairo_surface_destroy(m_Image);
}

//
// Create a new backing pixmap of the appropriate size
//
gint CScale::configure_event(GtkWidget *widget, GdkEventConfigure *event, CScale *pMe)
{
	pMe->OnConfigure(widget, event);
	return TRUE;
}

void CScale::OnConfigure(GtkWidget *widget, GdkEventConfigure *event)
{
	int width, height;
	
	width = widget->allocation.width;
	height = widget->allocation.height;
	if (m_PixWidth!=width || m_PixHeight!=height) {
		if (m_Pixmap)
			g_object_unref(m_Pixmap);
		m_Pixmap = gdk_pixmap_new(widget->window, width, height, -1);
		m_PixWidth = width;
		m_PixHeight = height;
		Redraw();
	}
}

//
// Redraw the screen from the backing pixmap
//
gint CScale::expose_event(GtkWidget *widget, GdkEventExpose *event, CScale *pMe)
{
	pMe->OnExpose(widget, event);
	return FALSE;
}

void CScale::OnExpose(GtkWidget *widget, GdkEventExpose *event)
{
	gdk_draw_drawable(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		m_Pixmap, event->area.x, event->area.y, event->area.x, event->area.y,
		event->area.width, event->area.height);
}


//
// Show/hide widget
//
void CScale::Show(bool show)
{
	m_Visible = show;
	if (show)
		gtk_widget_show(m_Frame);
	else
		gtk_widget_hide(m_Frame);
}


//
// Set pseudocolors mode 
//
void CScale::SetChart(CChartBase *pChart)
{
	CChartBase *old;
	if (pChart!=m_pChart) {
		old = m_pChart;
		m_pChart = NULL;
		if (old)
			old->SetScale(NULL);
		m_pChart = pChart;
		if (m_pChart)
			m_pChart->SetScale(this);
		Update();
	}
}


// Update scale
void CScale::Update(void)
{
	RebuildImage();
	Redraw();
}


//
// Draw a rectangle on the screen 
//
void CScale::Redraw(void)
{
	char buf[64];
	int i, sx, sy, ex, tx;
	double v, range, tstep, kt, qt, min, max;
	cairo_text_extents_t extents;
	
	if (!m_Pixmap || !m_DrawArea || !m_pChart)
		return;

	cairo_t *cr = gdk_cairo_create(m_Pixmap);

	// Paint background
	cairo_set_source_rgb(cr, 0, 0, 0);
	cairo_paint(cr);

	// Text size
	cairo_identity_matrix(cr);
	cairo_text_extents(cr, "65535", &extents);
	sx = (int)ceil(extents.width);
	sy = (int)ceil(extents.height);
	cairo_text_extents(cr, "ADU", &extents);
	ex = (int)ceil(extents.width);

	// Paint scale image
	if (m_Image && m_PixWidth>40 && m_PixHeight>sy+22) {
		cairo_pattern_t *pat = cairo_pattern_create_for_surface(m_Image);
		cairo_pattern_set_filter(pat, CAIRO_FILTER_NEAREST);

		cairo_translate(cr, 20, 5);
		cairo_scale(cr, ((double)(m_PixWidth-20-20))/IMAGE_WIDTH, m_PixHeight-sy-17-5);
		cairo_set_source(cr, pat);
		cairo_paint(cr);

		cairo_identity_matrix(cr);
		cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
		cairo_set_source_rgb(cr, 1, 1, 1);
		cairo_set_line_width(cr, 1);
		cairo_rectangle(cr, 20, 5, m_PixWidth-20-20, m_PixHeight-sy-17-5);
		cairo_stroke(cr);

		cairo_pattern_destroy(pat);
	}

	// Paint labels
	m_pChart->GetScaleMinMax(&min, &max);
	if (max > min) {
		tx = (m_PixWidth-2*sx-ex) / (int)(sx*1.5+0.5);
		i = NUM_STEPS-1;
		range = max - min;
		while (i>0 && range/XSteps[i]>tx)
			i--;
		tstep = XSteps[i];
		kt = (m_PixWidth-40)/range;
		qt = 20-kt*min;
		cairo_identity_matrix(cr);
		cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
		cairo_move_to(cr, m_PixWidth-ex-3, m_PixHeight-sy);
		cairo_show_text(cr, "ADU");
		v = floor(min/tstep)*tstep;
		while (v < max+tstep) {
			i = (int)(kt*v+qt+0.5);
			if (i >= 20 && i <= m_PixWidth-ex-15) {
				cairo_set_source_rgb(cr, 1, 1, 1);
				cairo_move_to(cr, i, m_PixHeight-sy-16);
				cairo_line_to(cr, i, m_PixHeight-sy-14);
				cairo_stroke(cr);
				sprintf(buf, "%.0f", v);
				cairo_text_extents(cr, buf, &extents);
				cairo_move_to(cr, i-extents.width/2, m_PixHeight-sy);
				cairo_show_text(cr, buf);
			}
			v += tstep;
		}
	}

	cairo_destroy(cr);
	gtk_widget_queue_draw(m_DrawArea);
}


//
// Rebuild image
//
void CScale::RebuildImage()
{
	if (m_Image && m_pChart) {
		unsigned long *ptr = (unsigned long*)cairo_image_surface_get_data(m_Image);
		if (!m_pChart->Pseudocolors()) {
			if (!m_pChart->ImageInvert()) {
				for (int x=0; x<IMAGE_WIDTH; x++)
					*ptr++ = grayscale(((double)x)/IMAGE_WIDTH);
			} else {
				for (int x=0; x<IMAGE_WIDTH; x++)
					*ptr++ = grayscale(1.0-((double)x)/IMAGE_WIDTH);
			}
		} else {
			if (!m_pChart->ImageInvert()) {
				for (int x=0; x<IMAGE_WIDTH; x++)
					*ptr++ = pseudocolor(((double)x)/IMAGE_WIDTH);
			} else {
				for (int x=0; x<IMAGE_WIDTH; x++)
					*ptr++ = pseudocolor(1.0-((double)x)/IMAGE_WIDTH);
			}
		}
	}
}
