/**************************************************************

table_dlg.cpp (C-Munipack project)
The preview dialog for tables
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <math.h>

#include "table_dlg.h"
#include "configuration.h"
#include "showheader_dlg.h"
#include "utils.h"

enum tMenuId
{
	MENU_FILE = 1,
	MENU_VIEW
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_PRINT,
	CMD_EXPORT,
	CMD_PROPERTIES,
	CMD_REMOVE,
	CMD_ERRORS
};

enum tFieldId
{
	FIELD_FILE,
	FIELD_FILENAME,
	FIELD_DIRPATH,
	FIELD_FILTER,
	FIELD_APERTURE
};

//-------------------------   MENU FOR PREVIEW WINDOW   ---------------------------

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM, CMD_PROPERTIES,			"Show _Properties" },
	{ CMenuBar::MB_SEPARATOR },
	//{ CMenuBar::MB_ITEM, CMD_EXPORT,			"E_xport" },
	//{ CMenuBar::MB_ITEM, CMD_PRINT,			"P_rint" },
	//{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM, CMD_CLOSE,				"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_CHECKBTN, CMD_ERRORS,		"Show _errors" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu PreviewMenu[] = {
	{ "_File", MENU_FILE, FileMenu },
	{ "_View", MENU_VIEW, ViewMenu },
	{ NULL }
};

//-------------------------   LIST COLUMNS   --------------------------------

struct tTreeViewColumn {
	const char *caption;		// Column name
	int column;					// Model column index
	GtkTreeCellDataFunc datafn;	// Data function
	gfloat align;				// Text alignment
};

enum tColumnTypeId {
	DATA_MAGNITUDE,				// Magnitude
	DATA_HELCOR,				// Heliocentric correction
	DATA_AIRMASS,				// Airmass coefficient
	DATA_OFFSET,				// Pixel offset
	DATA_DEVIATION,				// Deviation
	DATA_ALTITUDE				// Altitude
};

enum tDataColumnId {
	DATA_COLUMN,				// Column ordinal number
	DATA_NAME,					// Column name
	DATA_TYPE,					// Column type
	DATA_MIN,					// Minimum value
	DATA_MAX,					// Maximum value
	DATA_NCOLS					// Number of columns
};

const static tTreeViewColumn DataListColumns[] = {
	{ "Data", DATA_NAME, NULL, 0 },
	{ NULL }
};

//------------------------  COMBO BOX STRINGS   ---------------------------------

/*
static GtkWidget *toolbar_new(GtkOrientation orientation)
{
	GtkWidget *tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), orientation);
	return tbox;
}

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_entry(GtkWidget *toolbar)
{
	GtkWidget *entry = gtk_entry_new();
	gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
	gtk_entry_set_alignment(GTK_ENTRY(entry), 0.5);
	gtk_editable_set_editable(GTK_EDITABLE(entry), false);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}*/

//-------------------------   PREVIEW DIALOG   --------------------------------

CTableDlg::CTableDlg(GtkWindow *parent):CFileDlg(parent), m_Updating(false), 
	m_Column(-1), m_LastPosX(-1), m_LastPosY(-1), m_LastIndex(-1)
{
	static const GdkGeometry geom = { 320, 240 };

	int i;
	GtkWidget *vbox, *hbox, *frame;

	gtk_window_set_geometry_hints(GTK_WINDOW(m_pDlg), NULL, (GdkGeometry*)&geom, GDK_HINT_MIN_SIZE);
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), 400, 320);

	// Dialog icon
	gchar *icon = get_icon_file("lightcurve");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Window layout
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(m_pDlg), vbox);

	// Menu bar
	m_Menu.Create(PreviewMenu, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(vbox), m_Menu.Handle(), false, false, 0);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	// List of data columns
	m_DataList = gtk_list_store_new(DATA_NCOLS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
	m_DataView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(m_DataList));
	for (i=0; DataListColumns[i].caption!=NULL; i++) {
		GtkTreeViewColumn *col = gtk_tree_view_column_new();
		// Set column name and alignment
		gtk_tree_view_column_set_title(col, DataListColumns[i].caption);
		gtk_tree_view_append_column(GTK_TREE_VIEW(m_DataView), col);
		// Add text renderer
		GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(col, renderer, TRUE);
		g_object_set(renderer, "xalign", DataListColumns[i].align, NULL);
		if (DataListColumns[i].datafn)
			gtk_tree_view_column_set_cell_data_func(col, renderer, DataListColumns[i].datafn, NULL, NULL);
		else
			gtk_tree_view_column_add_attribute(col, renderer, "text", DataListColumns[i].column);
	}
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(m_DataView)), GTK_SELECTION_SINGLE);
	gtk_widget_set_size_request(m_DataView, 100, 0);

	// Register callback for selection change
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_DataView));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selection_changed), this);

	// Make scrolled window
	GtkWidget *scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_DataView);
	gtk_box_pack_start(GTK_BOX(hbox), scrwnd, FALSE, TRUE, 0);
	gtk_widget_set_size_request(scrwnd, 100, 100);

	// Graph
	frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Graph.Handle())));
	
	m_Errors = CConfig::GetBool("PreviewDlg", "Errors", true);
	m_Graph.SetData(&m_Data);
	m_Graph.SetGraphPadding(2, 2, 48, 2);
	m_Graph.EnableTracking(false);

	// Status bar
	m_Status.Create();
	gtk_box_pack_start(GTK_BOX(vbox), m_Status.Handle(), FALSE, FALSE, 0);

	// Show the dialog
	gtk_widget_show_all(vbox);
	UpdateControls();
}

bool CTableDlg::LoadFile(const char *filename)
{
	CmpackTable *table;

	m_Data.LockRedraw();
	m_Data.Clear();
	m_Info.Clear();

	char *f = g_locale_from_utf8(filename, -1, NULL, NULL, NULL);
	if (cmpack_tab_load(&table, f, CMPACK_LOAD_DEFAULT)==0) {
		m_Info.Set(table);
		switch (cmpack_tab_get_type(table))
		{
		case CMPACK_TABLE_LIGHTCURVE:
			LoadLightCurve(table);
			break;
		case CMPACK_TABLE_TRACKLIST:
			LoadTrackList(table);
			break;
		case CMPACK_TABLE_MUNIFIND:
			LoadMuniFind(table);
			break;
		case CMPACK_TABLE_AIRMASS:
			LoadAirMass(table);
			break;
		default:
			break;
		}
		cmpack_unref(table);
	}
	g_free(f);

	SetData(0);
	m_Graph.AutoZoom(true, true);
	m_Data.UnlockRedraw();
	m_Status.Set(NULL);
	UpdateControls();
	return true;
}


void CTableDlg::LoadLightCurve(CmpackTable *table)
{
	int i, j, count, first, index, channels, row_count;
	double y, d, jd, jdmin, jdmax, ymin, ymax, dmax, *yvals;
	GtkTreeIter iter;

	// Table data
	channels = cmpack_tab_ncolumns(table)-1;
	row_count = cmpack_tab_nrows(table);
	m_Data.LockRedraw();
	m_Data.Init(channels, row_count);
	if (channels>0 && row_count>0) {
		yvals = (double*)g_malloc(channels*sizeof(double));
		for (i=0; i<row_count; i++) {
			cmpack_tab_gtdd(table, i, 0, &jd);
			for (j=0; j<channels; j++) 
				cmpack_tab_gtdd(table, i, j+1, yvals+j);
			m_Data.Append(jd, yvals);
		}
	}
	m_Data.UnlockRedraw();

	// X axis
	CmpackTabColumn *col = cmpack_tab_get_column(table, 0);
	if (col) {
		jdmin = jdmax = 0;
		first = true;
		count = m_Data.Count(); 
		for (j=0; j<count; j++) {
			jd = m_Data.GetX(j);
			if (jd>0.0) {
				if (first) {
					jdmin = jdmax = jd;
					first = false;
				} else {
					if (jd < jdmin)
						jdmin = jd;
					if (jd > jdmax)
						jdmax = jd;
				}
			}
		}
		m_Graph.SetXParams(cmpack_tab_column_get_name(col), 
			jdmin, jdmax, 1.0/144.0, 0, 2);
		cmpack_unref(col);
	}

	// Columns
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), NULL);
	gtk_list_store_clear(m_DataList);
	for (j=0; j<channels; j++) {
		CmpackTabColumn *col = cmpack_tab_get_column(table, j+1);
		if (col) {
			const char *name = cmpack_tab_column_get_name(col);
			if (strstr(name, "HELCOR")==name) {
				index = m_Graph.AddColumn(j, -1, 0);
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (first) {
						ymin = ymax = y;
						first = false;
					} else {
						if (y < ymin)
							ymin = y;
						if (y > ymax)
							ymax = y;
					}
				}
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_MIN, ymin, DATA_MAX, ymax, DATA_TYPE, DATA_HELCOR, -1);
			} else
			if (strstr(name, "AIRMASS")==name) {
				index = m_Graph.AddColumn(j, -1, 0);
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>0.0) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
				}
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_MIN, ymin, DATA_MAX, ymax, DATA_TYPE, DATA_AIRMASS, -1);
			} else
			if (strstr(name, "ALTITUDE")==name) {
				index = m_Graph.AddColumn(j, -1, 0);
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>=-90.0 && y<=90.0) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
				}
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_MIN, ymin, DATA_MAX, ymax, DATA_TYPE, DATA_ALTITUDE, -1);
			} else
			if (name[0]=='V' || name[0]=='C' || name[0]=='K') {
				dmax = ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>-99.9 && y<99.9) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
					d = m_Data.GetY(i, j+1);
					if (d>dmax && d<9.9) 
						dmax = d;
				}
				index = m_Graph.AddColumn(j, j+1, 0);
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_MIN, ymin-dmax, DATA_MAX, ymax+dmax, DATA_TYPE, DATA_MAGNITUDE, -1);
			}
			cmpack_unref(col);
		}
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), GTK_TREE_MODEL(m_DataList));
}

void CTableDlg::LoadTrackList(CmpackTable *table)
{
	bool first;
	int i, j, count, index, channels, row_count;
	double jd, jdmin, jdmax, y, ymin, ymax, *yvals;
	GtkTreeIter iter;

	// Initialize graph data
	channels = cmpack_tab_ncolumns(table)-1;
	row_count = cmpack_tab_nrows(table);
	m_Data.LockRedraw();
	m_Data.Init(channels, row_count);
	if (channels>0 && row_count>0) {
		yvals = (double*)g_malloc(channels*sizeof(double));
		for (i=0; i<row_count; i++) {
			cmpack_tab_gtdd(table, i, 0, &jd);
			for (j=0; j<channels; j++)
				cmpack_tab_gtdd(table, i, j+1, yvals+j);
			m_Data.Append(jd, yvals);
		}
	}
	m_Data.UnlockRedraw();

	// X axis
	CmpackTabColumn *col = cmpack_tab_get_column(table, 0);
	if (col) {
		jdmin = jdmax = 0;
		first = true;
		count = m_Data.Count(); 
		for (j=0; j<count; j++) {
			jd = m_Data.GetX(j);
			if (jd>0.0) {
				if (first) {
					jdmin = jdmax = jd;
					first = false;
				} else {
					if (jd < jdmin)
						jdmin = jd;
					if (jd > jdmax)
						jdmax = jd;
				}
			}
		}
		m_Graph.SetXParams(cmpack_tab_column_get_name(col), jdmin, jdmax, 
			1.0/144.0, 0, 2);
		cmpack_unref(col);
	}

	// Columns
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), NULL);
	gtk_list_store_clear(m_DataList);
	m_Graph.ClearColumns();
	for (j=0; j<channels; j++) {
		CmpackTabColumn *col = cmpack_tab_get_column(table, j+1);
		if (col) {
			ymin = ymax = 0.0;
			first = true;
			count = m_Data.Count(); 
			for (i=0; i<count; i++) {
				y = m_Data.GetY(i, j);
				if (first) {
					ymin = ymax = y;
					first = false;
				} else {
					if (y < ymin)
						ymin = y;
					if (y > ymax)
						ymax = y;
				}
			}
			index = m_Graph.AddColumn(j, -1, 0);
			gtk_list_store_append(m_DataList, &iter);
			gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
				cmpack_tab_column_get_name(col), DATA_TYPE, DATA_OFFSET, 
				DATA_MIN, ymin, DATA_MAX, ymax, -1);
			cmpack_unref(col);
		}
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), GTK_TREE_MODEL(m_DataList));
}

void CTableDlg::LoadAirMass(CmpackTable *table)
{
	bool first;
	int i, j, count, index, channels, row_count;
	double jd, jdmin, jdmax, y, ymin, ymax, *yvals;
	GtkTreeIter iter;

	// Initialize graph data
	channels = cmpack_tab_ncolumns(table)-1;
	row_count = cmpack_tab_nrows(table);
	m_Data.LockRedraw();
	m_Data.Init(channels, row_count);
	if (channels>0 && row_count>0) {
		yvals = (double*)g_malloc(channels*sizeof(double));
		for (i=0; i<row_count; i++) {
			cmpack_tab_gtdd(table, i, 0, &jd);
			for (j=0; j<channels; j++)
				cmpack_tab_gtdd(table, i, j+1, yvals+j);
			m_Data.Append(jd, yvals);
		}
	}
	m_Data.UnlockRedraw();

	// X axis
	CmpackTabColumn *col = cmpack_tab_get_column(table, 0);
	if (col) {
		jdmin = jdmax = 0;
		first = true;
		count = m_Data.Count(); 
		for (j=0; j<count; j++) {
			jd = m_Data.GetX(j);
			if (jd>0.0) {
				if (first) {
					jdmin = jdmax = jd;
					first = false;
				} else {
					if (jd < jdmin)
						jdmin = jd;
					if (jd > jdmax)
						jdmax = jd;
				}
			}
		}
		m_Graph.SetXParams(cmpack_tab_column_get_name(col), jdmin, jdmax, 
			1.0/144.0, 0, 2);
		cmpack_unref(col);
	}

	// Columns
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), NULL);
	gtk_list_store_clear(m_DataList);
	m_Graph.ClearColumns();
	for (j=0; j<channels; j++) {
		CmpackTabColumn *col = cmpack_tab_get_column(table, j+1);
		if (col) {
			const char *name = cmpack_tab_column_get_name(col);
			if (strstr(name, "AIRMASS")) {
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>0.0) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
				}
				index = m_Graph.AddColumn(j, -1, 0);
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_TYPE, DATA_AIRMASS, DATA_MIN, ymin, DATA_MAX, ymax, -1);
			} else
			if (strstr(name, "ALTITUDE")) {
				ymin = ymax = 0.0;
				count = m_Data.Count();
				first = true;
				for (i=0; i<count; i++) {
					y = m_Data.GetY(i, j);
					if (y>=-90.0 && y<=90.0) {
						if (first) {
							ymin = ymax = y;
							first = false;
						} else {
							if (y < ymin)
								ymin = y;
							if (y > ymax)
								ymax = y;
						}
					}
				}
				index = m_Graph.AddColumn(j, -1, 0);
				gtk_list_store_append(m_DataList, &iter);
				gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
					name, DATA_TYPE, DATA_ALTITUDE, DATA_MIN, ymin, DATA_MAX, ymax, -1);
			}
			cmpack_unref(col);
		}
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), GTK_TREE_MODEL(m_DataList));
}

void CTableDlg::LoadMuniFind(CmpackTable *table)
{
	bool first;
	int i, index, count, row_count;
	double x, y, mag, dev, xmin, xmax, ymin, ymax;
	GtkTreeIter iter;

	// Initialize graph data
	row_count = cmpack_tab_nrows(table);
	m_Data.LockRedraw();
	m_Data.Init(1, row_count);
	if (row_count>0) {
		for (i=0; i<row_count; i++) {
			cmpack_tab_gtdd(table, i, 1, &mag);
			cmpack_tab_gtdd(table, i, 2, &dev);
			if (mag<99)
				m_Data.Append(mag, &dev);
		}
	}
	m_Data.UnlockRedraw();

	// X axis
	CmpackTabColumn *col = cmpack_tab_get_column(table, 1);
	if (col) {
		xmin = xmax = 0.0;
		first = true;
		count = m_Data.Count(); 
		for (i=0; i<count; i++) {
			x = m_Data.GetX(i);
			if (x>-99.9 && x<99.9) {
				if (first) {
					xmin = xmax = x;
					first = false;
				} else {
					if (x < xmin)
						xmin = x;
					if (x > xmax)
						xmax = x;
				}
			}
		}
		m_Graph.SetXParams(cmpack_tab_column_get_name(col), 
			xmin, xmax, 0.5, 0, 1);
		cmpack_unref(col);
	}

	// Columns
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), NULL);
	gtk_list_store_clear(m_DataList);
	m_Graph.ClearColumns();
	col = cmpack_tab_get_column(table, 2);
	if (col) {
		index = m_Graph.AddColumn(0, -1, 0);
		gtk_list_store_append(m_DataList, &iter);
		ymin = ymax = 0.0;
		first = true;
		count = m_Data.Count(); 
		for (i=0; i<count; i++) {
			y = m_Data.GetY(i, 0);
			if (y>=0.0 && y<9.99) {
				if (first) {
					ymin = ymax = y;
					first = false;
				} else {
					if (y < ymin)
						ymin = y;
					if (y > ymax)
						ymax = y;
				}
			}
		}
		gtk_list_store_set(m_DataList, &iter, DATA_COLUMN, index, DATA_NAME, 
			cmpack_tab_column_get_name(col), DATA_TYPE, DATA_DEVIATION, 
			DATA_MIN, ymin, DATA_MAX, ymax, -1);
		cmpack_unref(col);
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_DataView), GTK_TREE_MODEL(m_DataList));
}

void CTableDlg::UpdateControls(void)
{
	// Frame menu
	m_Menu.EnableMenu(MENU_FILE, true);
	m_Menu.Enable(CMD_PROPERTIES, true);
	m_Menu.Enable(CMD_EXPORT, true);
	m_Menu.Enable(CMD_PRINT, true);
	m_Menu.Check(CMD_ERRORS, m_Errors);
}

void CTableDlg::MenuCallback(int message, int wparam, void *lparam, void *cb_data)
{
	CTableDlg *pMe = (CTableDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CTableDlg::OnCommand(int cmd_id)
{
	int i;

	switch (cmd_id)
	{
	// File menu
	case CMD_CLOSE:
		Destroy();
		break;
	case CMD_PROPERTIES:
		ShowProperties();
		break;

	// View menu
	case CMD_ERRORS:
		m_Errors = m_Menu.IsChecked(CMD_ERRORS);
		for (i=0; i<m_Graph.Columns(); i++) 
			m_Graph.SetColumnFlags(i, CGraph::NO_ERRORS, (m_Errors ? 0 : CGraph::NO_ERRORS));
		CConfig::SetBool("PreviewDlg", "Errors", m_Errors);
		break;
	}
}

void CTableDlg::ShowProperties(void)
{
	CTableInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_Path, &m_Info);
}

void CTableDlg::SetData(int column)
{
	tColumnTypeId type;
	gchar *name;
	double min, max;
	GtkTreeIter iter;

	m_Column = column;
	m_Graph.ShowColumn(-1, false);
	m_Graph.ShowColumn(m_Column, true); 
	if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(m_DataList), &iter, NULL, m_Column)) {
		gtk_tree_model_get(GTK_TREE_MODEL(m_DataList), &iter, DATA_NAME, &name, DATA_TYPE, &type, 
			DATA_MIN, &min, DATA_MAX, &max, -1);
		switch (type)
		{
		case DATA_MAGNITUDE:
			m_Graph.SetYParams(name, min, max, 0.05, CGraph::REVERSE_SCALE, 2);
			break;
		case DATA_DEVIATION:
			m_Graph.SetYParams(name, min, max, 0.05, 0, 2);
			break;
		case DATA_HELCOR:
			m_Graph.SetYParams(name, min, max, 0.005, 0, 4);
			break;
		case DATA_AIRMASS:
			m_Graph.SetYParams(name, min, max, 0.5, 0, 2);
			break;
		case DATA_OFFSET:
			m_Graph.SetYParams(name, min, max, 1.0, 0, 0);
			break;
		case DATA_ALTITUDE:
			m_Graph.SetYParams(name, min, max, 10.0, 0, 0);
			break;
		}
		g_free(name);
	}
	m_Graph.AutoZoom(false, true);
}

void CTableDlg::GraphCallback(int message, int wparam, void* lparam, void* cb_data)
{
	CTableDlg *pMe = (CTableDlg*)cb_data;

	switch (message)
	{
	case CChartBase::CB_MOUSE_MOVE:
		pMe->OnMouseMoved();
		break;
	case CChartBase::CB_LEFT_CLICK:
		pMe->OnLButtonClick(((GdkEventButton*)lparam)->time);
		break;
	}
}

void CTableDlg::OnMouseMoved(void)
{
	/*char	buf[256], aux[128];
	double	x, y;
	int		index;
	CChartEx::tStarInfo star;

	if (m_Graph.GetMousePos(&x, &y)) 
		index = m_Graph.FindByPos(x, y, 0);
	else
		index = -1;
	if (index>=0) {
		if (m_LastIndex!=index) {
			m_LastIndex = index;
			m_Graph.SetFocusedItem(index, 0);
			m_Graph.GetStar(index, &star);
			sprintf(buf, "Star #%d: X = %.1f, Y = %.1f", star.id, star.x, star.y);
			if (star.mag_valid && star.err_valid) {
				sprintf(aux, ", Mag = %.4f +- %.4f", star.mag, star.err);
				strcat(buf, aux);
			}
			if (star.mag_valid && !star.err_valid) {
				sprintf(aux, ", Mag = %.4f +- ????", star.mag);
				strcat(buf, aux);
			}
			if (!star.mag_valid && !star.err_valid) {
				sprintf(aux, ", Mag = ???? +- ????");
				strcat(buf, aux);
			}
			if (star.matched) {
				sprintf(aux, ", Ref. = %d", star.xref);
				strcat(buf, aux);
			}
			m_Status.Set(buf);
		}
	} else {
		if (m_LastIndex>=0) {
			m_LastIndex = -1;
			m_Graph.SetFocusedItem(-1, 0);
			m_Status.Set(NULL);
		}
	}*/
}


//
// Left button click
//
void CTableDlg::OnLButtonClick(guint32 timestamp)
{
}

void CTableDlg::selection_changed(GtkTreeSelection *widget, CTableDlg *pMe)
{
	pMe->OnSelectionChanged(widget);
}

void CTableDlg::OnSelectionChanged(GtkTreeSelection *selection)
{
	int id;
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		if (model == GTK_TREE_MODEL(m_DataList)) {
			gtk_tree_model_get(model, &iter, DATA_COLUMN, &id, -1);
			SetData(id);
		}
	}
}

void CTableDlg::col_name_free(char *str, void *data)
{
	g_free(str);
}

//-------------------------   FILE PREVIEW DIALOG   --------------------------------

//
// Constructor
//
CTableDlgModal::CTableDlgModal(GtkWindow *pParent):CFileDlgModal(pParent)
{
	static const CMenuBar::tMenuItem FileMenu[] = {
		{ CMenuBar::MB_ITEM, CMD_PROPERTIES, "_Properties" },
		{ CMenuBar::MB_END }
	};

	static const CMenuBar::tMenu PreviewMenu[] = {
		{ "_File", MENU_FILE, FileMenu },
		{ NULL }
	};

	GtkWidget *vbox;

	m_Menu.Create(PreviewMenu, false);
	m_Menu.RegisterCallback(MenuClick, this);
	m_Graph.EnableTracking(true);

	// Window layout
	vbox = GTK_DIALOG(m_pDlg)->vbox;
	gtk_box_pack_start (GTK_BOX(vbox), m_Menu.Handle(), false, false, 0);
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g_object_ref(m_Graph.Handle())), true, true, 0);
	gtk_widget_show_all(vbox);
}


//
// Menu handler
//
void CTableDlgModal::MenuClick(int message, int wparam, void *lparam, void *cb_data)
{
	CTableDlgModal *pMe = (CTableDlgModal*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}


bool CTableDlgModal::LoadFile(const char *fpath)
{
	//m_Data.Load(fpath);
	return true;
}

void CTableDlgModal::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	case CMD_PROPERTIES:
		ShowProperties();
		break;
	}
}

void CTableDlgModal::ShowProperties(void)
{
	CTableInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_Path, &m_Info);
}

//---------------------------   INFO DIALOG   --------------------------------

//
// Constructor
//
CTableInfoDlg::CTableInfoDlg(GtkWindow *pParent):CInfoDlg(pParent), m_Path(NULL)
{
	GtkWidget *bbox;

	AddHeading(FIELD_FILE, 0, 0, "File");
	AddField(FIELD_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_DIRPATH, 0, 2, "Location", PANGO_ELLIPSIZE_MIDDLE);
	AddSeparator(0, 4);
	AddField(FIELD_FILTER, 0, 5, "Optical filter", PANGO_ELLIPSIZE_END);
	AddField(FIELD_APERTURE, 0, 6, "Aperture");
	gtk_widget_show_all(m_Tab);

	bbox = gtk_hbutton_box_new();
	gtk_box_set_spacing(GTK_BOX(bbox), 8);
	gtk_container_set_border_width(GTK_CONTAINER(bbox), 4);
	m_HdrBtn = gtk_button_new_with_label("Show header...");
	gtk_box_pack_start(GTK_BOX(bbox), m_HdrBtn, true, true, 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), bbox);
	gtk_widget_show_all(bbox);

	g_signal_connect(G_OBJECT(m_HdrBtn), "clicked", G_CALLBACK(button_clicked), this);
}


//
// Destructor
//
CTableInfoDlg::~CTableInfoDlg()
{
	if (m_Path)
		g_free(m_Path);
}


//
// Show dialog
//
void CTableInfoDlg::ShowModal(const char *path, const CTableInfo *pData)
{
	char *basename, *dirpath;

	// Update properties
	g_free(m_Path);
	m_Path = g_strdup(path);
	basename = g_path_get_basename(path);
	SetField(FIELD_FILENAME, basename);
	g_free(basename);
	dirpath = g_path_get_dirname(path);
	SetField(FIELD_DIRPATH, dirpath);
	g_free(dirpath);

	if (pData->Filter())
		SetField(FIELD_FILTER, pData->Filter());
	else
		SetField(FIELD_FILTER, "");
	if (pData->Aperture()>0)
		SetField(FIELD_APERTURE, pData->Aperture());
	else
		SetField(FIELD_APERTURE, "");

	CInfoDlg::ShowModal();
}

void CTableInfoDlg::button_clicked(GtkWidget *button, CTableInfoDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CTableInfoDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==GTK_WIDGET(m_HdrBtn))
		ShowHeader();
}


void CTableInfoDlg::ShowHeader(void)
{
	CShowHeaderDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(m_Path);
}
