/**************************************************************

varcat.h (C-Munipack project)
Catalogues of variable stars
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>

#include "varcat.h"
#include "configuration.h"

#define BUFSIZE 4096
#define MAXLINE 1024

//----------------------   HELPER CLASSES   -------------------------------

//
// dBase file parser
//
class CDBFReader
{
public:
	// Constructor
	CDBFReader(FILE *f);

	// Destructor
	~CDBFReader();

	// Read next record
	bool Next();

	// Get field name
	const char *FieldName(int col);

	// Get field data
	bool GetField(int index, char *buf, size_t buflen);

private:
	struct tDBFHeader
	{
		unsigned char version;
		unsigned char year, month, day;
		unsigned int count;
		unsigned short head_len, rec_length;
		char reserved[20];
	};

	struct tDBFField
	{
		char name[16];
		int start, length;
	};

	FILE *file;
	tDBFField *cols;
	char *record;
	int colcount;
	tDBFHeader header;
};

// Constructor
CDBFReader::CDBFReader(FILE *f):file(f), cols(NULL), record(NULL), colcount(0)
{
	char fielddef[32];
	long i, j, start, count;

	memset(&header, 0, sizeof(tDBFHeader));
	
	// Read file header
	if (fread(&header, sizeof(tDBFHeader), 1, f)==1) {
		// Read field definitions
		count = (header.head_len - sizeof(tDBFHeader))/32;
		if (count>0 && count<256) {
			cols = (tDBFField*)g_malloc(count*sizeof(tDBFField));
			start = 1;
			colcount = 0;
			for (i=0; i<count; i++) {
				if (fread(fielddef, sizeof(fielddef), 1, f)==1) {
					for (j=0; fielddef[j]!='\0' && j<11; j++)
						cols[colcount].name[j] = fielddef[j];
					cols[colcount].name[j] = '\0';
					cols[colcount].length = fielddef[16];
					cols[colcount].start = start;
					start += fielddef[16];
					colcount++;
				}
			}
			fseek(file, header.head_len, SEEK_SET);
			record = (char*)g_malloc(header.rec_length);
		}
	}
}

// Destructor
CDBFReader::~CDBFReader()
{
	g_free(cols);
	g_free(record);
}

// Get next record
bool CDBFReader::Next()
{
	if (file && record) 
		return fread(record, header.rec_length, 1, file)==1;
	return false;
}

// Get field name
const char *CDBFReader::FieldName(int col)
{
	if (col>=0 && col<colcount)
		return cols[col].name;
	return NULL;
}

// Get field value
bool CDBFReader::GetField(int col, char *buf, size_t buflen)
{
	size_t	start, length, pos;

	if (record && col>=0 && col<colcount) {
		start = cols[col].start;
		length = cols[col].length;
		pos = strspn(record + start, " ");
		if (pos >= length) 
			length = 0;
		else
			length -= pos;
		if (length>0) {
			if (length+1 > buflen) {
				memcpy(buf, record+start+pos, buflen-1);
				length = buflen-1;
			} else {
				memcpy(buf, record+start+pos, length);
				while (length>0 && buf[length-1]==' ')
					length--;
			}
		}
		buf[length] = '\0';
		return true;
	} else {
		buf[0] = '\0';
		return false;
	}
}

//
// Text file parser
//
class CTextReader
{
public:
	// Constructor
	CTextReader(FILE *f);

	// Destructor
	~CTextReader();

	// Get next line
	bool ReadLine();

	// Get field value
	bool GetField(size_t start, size_t length, char *buf, size_t buflen) const;

	// Get line length
	size_t Length(void) const
	{ return linelength; }

	char Col(int index) const
	{ return line[index]; }

private:
	bool eof;
	char *line;
	size_t linesize, linelength;
	char buf[BUFSIZE], *bufptr, *bufend;
	FILE *file;
};

// Constructor
CTextReader::CTextReader(FILE *f):eof(false), linesize(BUFSIZE), linelength(0),
	bufptr(buf), bufend(buf), file(f)
{
	line = (char*)g_malloc(BUFSIZE);
}

// Destructor
CTextReader::~CTextReader()
{
	g_free(line);
}

// Read next line
bool CTextReader::ReadLine(void)
{
	int state;
	char ch, *lineptr;
	size_t bytes, len;

	state = 0;
	lineptr = line;
	linelength = 0;
	while (true) {
		if (bufptr >= bufend) {
			bytes = fread(buf, 1, BUFSIZE, file);
			if (bytes < BUFSIZE && !eof) {
				buf[bytes++] = '\x04';
				eof = true;
			}
			if (bytes <= 0) {
				bufptr = bufend = buf;
				return false;
			} else {
				bufptr = buf;
				bufend = buf + bytes;
			}
		}
		ch = *bufptr;
		if (ch=='\x4')
			break;
		if (state==0) {
			if (ch=='\xD') {
				bufptr++;
				state = 1;
			} else 
			if (ch=='\xA')  {
				linelength = (lineptr-line);
				bufptr++;
				return true;
			} else {
				len = lineptr-line;
				if (len >= linesize) {
					linesize += BUFSIZE;
					line = (char*)g_realloc(line, linesize);
					lineptr = line + len;
				}
				*lineptr++ = ch;
				bufptr++;
			}
		} else {
			if (ch=='\xA') 
				bufptr++;
			linelength = (lineptr-line);
			return true;
		}
	}
	if (lineptr > line) {
		linelength = (lineptr-line);
		return true;
	}
	return false;
}

bool CTextReader::GetField(size_t start, size_t length, char *buf, size_t buflen) const
{
	size_t pos, i;

	if (start < linelength) {
		if (start+length > linelength)
			length = linelength - start;
		if (length >= buflen)
			length = buflen-1;
		pos = strspn(line + start, " ");
		if (pos >= length) {
			buf[0] = '\0';
		} else {
			memcpy(buf, line+start+pos, length-pos);
			i = length-pos;
			while (i>0 && buf[i-1]==' ')
				i--;
			buf[i] = '\0';
		}
		return true;
	}
	buf[0] = '\0';
	return false;
}

static void InsertString(char *buf, size_t buflen, const char *substr, size_t index)
{
	size_t slen = strlen(buf), sublen = strlen(substr);
	if (sublen>0 && index<=slen) {
		if (slen+sublen+1 <= buflen) {
			memmove(buf+index+sublen, buf+index, slen-index+1);
			memmove(buf+index, substr, sublen);
		} else 
		if (index+sublen+1 < buflen) {
			memmove(buf+index+sublen, buf+index, buflen-index-sublen-1);
			memmove(buf+index, substr, sublen);
			buf[buflen-1] = '\0';
		} else {
			memmove(buf+index, substr, buflen-index-1);
			buf[buflen-1] = '\0';
		}
	}
}

static char *MakePattern(const char *searchString)
{
	int state;
	const char *sptr;
	char *buf, *dptr;
	size_t len = strlen(searchString);
	if (len>0) {
		buf = (char*)g_malloc(len+1);
		for (sptr=searchString, dptr=buf, state=0; *sptr!='\0'; sptr++) {
			if (state==0) {
				if (*sptr>='1' && *sptr<='9') {
					*dptr++ = *sptr;
					state = 1;
				} else {
					if (*sptr>='A' && *sptr<='Z') 
						*dptr++ = *sptr;
					else if (*sptr>='a' && *sptr<='z')
						*dptr++ = *sptr - 'a' + 'A';
				}
			} else {
				if (*sptr>='0' && *sptr<='9') {
					*dptr++ = *sptr;
				} else {
					if (*sptr>='A' && *sptr<='Z') 
						*dptr++ = *sptr;
					else if (*sptr>='a' && *sptr<='z') 
						*dptr++ = *sptr - 'a' + 'A';
					state = 0;
				}
			}
		}
		*dptr = '\0';
		if (strlen(buf)>0)
			return buf;
		else
			g_free(buf);
	}
	return NULL;
}

static bool TestName(const char *pattern, const char *name)
{
	char *buf = MakePattern(name);
	if (buf) {
		int res = strcmp(buf, pattern);
		g_free(buf);
		return res==0;
	}
	return false;
}

static void StripLeadingZeros(char *buf)
{
	if (strlen(buf)>1) {
		char *sptr = buf;
		while ((*sptr=='0' || *sptr==' ') && *(sptr+1)!='\0')
			sptr++;
		if (sptr!=buf)
			memmove(buf, sptr, strlen(sptr)+1);
	}
}

static void VarCat_BRKA(const char *filepath, const char *searchstr, 
				 tVarCatProc *cb_proc, void *cb_data)
{
	const char *catalog;
	char *f1, name[MAXLINE], aux[MAXLINE], objra[MAXLINE], objdec[MAXLINE], *pattern;
	FILE *f;

	pattern = MakePattern(searchstr);
	f1 = g_locale_from_utf8(filepath, -1, NULL, NULL, NULL);
	if (pattern && f1) {
		f = fopen(f1, "rb");
		if (f) {
			CDBFReader reader(f);
			if (strcmp(reader.FieldName(0), "NAZEV")==0 && 
				strcmp(reader.FieldName(1), "TYP")==0 &&
				strcmp(reader.FieldName(3), "RA2000")==0 &&
				strcmp(reader.FieldName(4), "DE2000")==0 && 
				strcmp(reader.FieldName(17), "GSC")==0) {
					while (reader.Next()) {
						if (reader.GetField(0, name, MAXLINE) && TestName(pattern, name)) {
							reader.GetField(3, objra, MAXLINE);
							InsertString(objra, MAXLINE, " ", 2);
							InsertString(objra, MAXLINE, " ", 5);
							reader.GetField(4, objdec, MAXLINE);
							InsertString(objdec, MAXLINE, " ", 3);
							InsertString(objdec, MAXLINE, " ", 6);
							reader.GetField(1, aux, MAXLINE);
							if (aux[0]=='H')
								catalog = "BRKA-H";
							else if (aux[0]=='C')
								catalog = "BRKA-C";
							else
								catalog = "BRKA-Z";
							reader.GetField(17, aux, MAXLINE);
							if (strlen(aux)>0)
								InsertString(aux, MAXLINE, "GSC ", 0);
							if (cb_proc)
								cb_proc(name, objra, objdec, catalog, aux, cb_data);
						}
					}
			}
			fclose(f);
		}
		g_free(pattern);
		g_free(f1);
	}
}

static void VarCat_GCVS(const char *filepath, const char *searchstr, 
				 tVarCatProc *cb_proc, void *cb_data)
{
	FILE *f;
	char *f1, name[MAXLINE], aux[MAXLINE], objra[MAXLINE], objdec[MAXLINE], *pattern;

	pattern = MakePattern(searchstr);
	f1 = g_locale_from_utf8(filepath, -1, NULL, NULL, NULL);
	if (pattern && f1) {
		f = fopen(f1, "rb");
		if (f) {
			CTextReader reader(f);
			while (reader.ReadLine()) {
				if (reader.Length()>53 && reader.Length()<MAXLINE && 
					reader.Col(7)=='|' && reader.Col(19)=='|' && reader.Col(53)=='|') {
					// GCVS I-III 2006
					reader.GetField(8, 5, name, MAXLINE);
					strcat(name, " ");
					reader.GetField(14, 3, aux, MAXLINE);
					strcat(name, aux);
					if (TestName(pattern, name)) {
						reader.GetField(37, 8, objra, MAXLINE);
						InsertString(objra, MAXLINE, " ", 2);
						InsertString(objra, MAXLINE, " ", 5);
						reader.GetField(45, 7, objdec, MAXLINE);
						InsertString(objdec, MAXLINE, " ", 3);
						InsertString(objdec, MAXLINE, " ", 6);
						if (cb_proc)
							cb_proc(name, objra, objdec, "GCVS I-III", NULL, cb_data);
					}
				} else
				if (reader.Length()>53 && reader.Length()<MAXLINE && 
					reader.Col(7)=='|' && reader.Col(19)=='|' && reader.Col(47)=='|') {
						// GCVS I-III 2007 - 2009
						reader.GetField(8, 5, name, MAXLINE);
						strcat(name, " ");
						reader.GetField(14, 3, aux, MAXLINE);
						strcat(name, aux);
						if (TestName(pattern, name)) {
							reader.GetField(20, 8, objra, MAXLINE);
							InsertString(objra, MAXLINE, " ", 2);
							InsertString(objra, MAXLINE, " ", 5);
							reader.GetField(28, 7, objdec, MAXLINE);
							InsertString(objdec, MAXLINE, " ", 3);
							InsertString(objdec, MAXLINE, " ", 6);
							if (cb_proc)
								cb_proc(name, objra, objdec, "GCVS I-III", NULL, cb_data);
						}
				}
			}
			fclose(f);
		}
		g_free(pattern);
		g_free(f1);
	}
}

static void VarCat_NSV(const char *filepath, const char *searchstr, 
				 tVarCatProc *cb_proc, void *cb_data)
{
	FILE *f;
	char *f1, name[MAXLINE], objra[MAXLINE];
	char objdec[MAXLINE], comment[MAXLINE], *pattern;

	pattern = MakePattern(searchstr);
	f1 = g_locale_from_utf8(filepath, -1, NULL, NULL, NULL);
	if (pattern && f1) {
		f = fopen(f1, "rb");
		if (f) {
			CTextReader reader(f);
			while (reader.ReadLine()) {
				if (reader.Length()>18 && reader.Length()<MAXLINE && 
					reader.Col(8)=='|' && reader.Col(26)=='|' && 
					(reader.Col(17)=='+' || reader.Col(17)=='-')) {
						// NSV
						reader.GetField(0, 5, name, MAXLINE);
						StripLeadingZeros(name);
						InsertString(name, MAXLINE, "NSV ", 0);
						if (TestName(pattern, name)) {
							reader.GetField(27, 8, objra, MAXLINE);
							InsertString(objra, MAXLINE, " ", 2);
							InsertString(objra, MAXLINE, " ", 5);
							reader.GetField(35, 7, objdec, MAXLINE);
							InsertString(objdec, MAXLINE, " ", 3);
							InsertString(objdec, MAXLINE, " ", 6);
							reader.GetField(81, 13, comment, MAXLINE);
							if (cb_proc)
								cb_proc(name, objra, objdec, "NSV", comment, cb_data);
						}
				}
			}
			fclose(f);
		}
		g_free(pattern);
		g_free(f1);
	}
}

static void VarCat_NSVS(const char *filepath, const char *searchstr, 
				 tVarCatProc *cb_proc, void *cb_data)
{
	FILE *f;
	char *f1, name[MAXLINE], objra[MAXLINE], objdec[MAXLINE], *pattern;

	pattern = MakePattern(searchstr);
	f1 = g_locale_from_utf8(filepath, -1, NULL, NULL, NULL);
	if (pattern && f1) {
		f = fopen(f1, "rb");
		if (f) {
			CTextReader reader(f);
			while (reader.ReadLine()) {
				if (reader.Length()>24 && reader.Length()<MAXLINE && 
					reader.Col(6)=='|' && reader.Col(23)=='|' && 
					(reader.Col(15)=='+' || reader.Col(15)=='-')) {
						// NSV Supplement
						reader.GetField(0, 5, name, MAXLINE);
						StripLeadingZeros(name);
						InsertString(name, MAXLINE, "NSV ", 0);
						if (TestName(pattern, name)) {
							reader.GetField(24, 8, objra, MAXLINE);
							InsertString(objra, MAXLINE, " ", 2);
							InsertString(objra, MAXLINE, " ", 5);
							reader.GetField(32, 7, objdec, MAXLINE);
							InsertString(objdec, MAXLINE, " ", 3);
							InsertString(objdec, MAXLINE, " ", 6);
							if (cb_proc)
								cb_proc(name, objra, objdec, "NSVS", NULL, cb_data);
						}
				}
			}
			fclose(f);
		}
		g_free(pattern);
		g_free(f1);
	}
}

bool VarCat_Test(void)
{
	bool ok = false;
	char *fpath;

	if (CConfig::GetBool("BRKA", "Enable")) {
		fpath = CConfig::GetStr("BRKA", "Path", "");
		if (g_file_test(fpath, G_FILE_TEST_IS_REGULAR)) {
			ok = true;
			g_free(fpath);
		}
	}
	if (!ok && CConfig::GetBool("GCVS", "Enable")) {
		fpath = CConfig::GetStr("GCVS", "Path", "");
		if (g_file_test(fpath, G_FILE_TEST_IS_REGULAR)) {
			ok = true;
			g_free(fpath);
		}
	}
	if (!ok && CConfig::GetBool("NSV", "Enable")) {
		fpath = CConfig::GetStr("NSV", "Path", "");
		if (g_file_test(fpath, G_FILE_TEST_IS_REGULAR)) {
			ok = true;
			g_free(fpath);
		}
	}
	if (!ok && CConfig::GetBool("NSVS", "Enable")) {
		fpath = CConfig::GetStr("NSVS", "Path", "");
		if (g_file_test(fpath, G_FILE_TEST_IS_REGULAR)) {
			ok = true;
			g_free(fpath);
		}
	}
	return ok;
}

void VarCat_Search(const char *searchstr, tVarCatProc *cb_proc, void *cb_data)
{
	char *fpath;

	if (CConfig::GetBool("BRKA", "Enable")) {
		fpath = CConfig::GetStr("BRKA", "Path", "");
		if (fpath) {
			VarCat_BRKA(fpath, searchstr, cb_proc, cb_data);
			g_free(fpath);
		}
	}
	if (CConfig::GetBool("GCVS", "Enable")) {
		fpath = CConfig::GetStr("GCVS", "Path", "");
		if (fpath) {
			VarCat_GCVS(fpath, searchstr, cb_proc, cb_data);
			g_free(fpath);
		}
	}
	if (CConfig::GetBool("NSV", "Enable")) {
		fpath = CConfig::GetStr("NSV", "Path", "");
		if (fpath) {
			VarCat_NSV(fpath, searchstr, cb_proc, cb_data);
			g_free(fpath);
		}
	}
	if (CConfig::GetBool("NSVS", "Enable")) {
		fpath = CConfig::GetStr("NSVS", "Path", "");
		if (fpath) {
			VarCat_NSVS(fpath, searchstr, cb_proc, cb_data);
			g_free(fpath);
		}
	}
}
